{
    "$graph": [
        {
            "class": "Workflow",
            "requirements": [
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "label": "Input BAM file",
                    "doc": "Input BAM file, sorted by coordinates",
                    "id": "#bam-bedgraph-bigwig.cwl/bam_file"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Output filename for generated bedGraph",
                    "id": "#bam-bedgraph-bigwig.cwl/bedgraph_filename"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Output filename for generated bigWig",
                    "id": "#bam-bedgraph-bigwig.cwl/bigwig_filename"
                },
                {
                    "type": "File",
                    "doc": "Tab delimited chromosome length file: <chromName><TAB><chromSize>",
                    "id": "#bam-bedgraph-bigwig.cwl/chrom_length_file"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "Change strand af the mate read, so both reads come from the same strand",
                    "id": "#bam-bedgraph-bigwig.cwl/dutp"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "Set fixed fragment size for genome coverage calculation",
                    "id": "#bam-bedgraph-bigwig.cwl/fragment_size"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "Parameter to calculate scale as 1000000/mapped_reads_number. Ignored by bedtools-genomecov.cwl in\nbam_to_bedgraph step if scale is provided\n",
                    "id": "#bam-bedgraph-bigwig.cwl/mapped_reads_number"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "Enable paired-end genome coverage calculation",
                    "id": "#bam-bedgraph-bigwig.cwl/pairchip"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "doc": "Coefficient to scale the genome coverage by a constant factor",
                    "id": "#bam-bedgraph-bigwig.cwl/scale"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "Calculate genome coverage for each part of the splitted by 'N' and 'D' read",
                    "id": "#bam-bedgraph-bigwig.cwl/split"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Calculate genome coverage of intervals from a specific strand",
                    "id": "#bam-bedgraph-bigwig.cwl/strand"
                }
            ],
            "steps": [
                {
                    "run": "#bedtools-genomecov.cwl",
                    "in": [
                        {
                            "default": "-bg",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/depth"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/dutp",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/du"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/fragment_size",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/fragment_size"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/bam_file",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/input_file"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/mapped_reads_number",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/mapped_reads_number"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/bedgraph_filename",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/output_filename"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/pairchip",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/pairchip"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/scale",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/scale"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/split",
                            "valueFrom": "${\n  if (self == null){\n    return true;\n  } else {\n    return self;\n  }\n}\n",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/split"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/strand",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/strand"
                        }
                    ],
                    "out": [
                        "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/genome_coverage_file"
                    ],
                    "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph"
                },
                {
                    "run": "#linux-sort.cwl",
                    "in": [
                        {
                            "default": [
                                "1,1",
                                "2,2n"
                            ],
                            "id": "#bam-bedgraph-bigwig.cwl/sort_bedgraph/key"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/genome_coverage_file",
                            "id": "#bam-bedgraph-bigwig.cwl/sort_bedgraph/unsorted_file"
                        }
                    ],
                    "out": [
                        "#bam-bedgraph-bigwig.cwl/sort_bedgraph/sorted_file"
                    ],
                    "id": "#bam-bedgraph-bigwig.cwl/sort_bedgraph"
                },
                {
                    "run": "#ucsc-bedgraphtobigwig.cwl",
                    "in": [
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/sort_bedgraph/sorted_file",
                            "id": "#bam-bedgraph-bigwig.cwl/sorted_bedgraph_to_bigwig/bedgraph_file"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/chrom_length_file",
                            "id": "#bam-bedgraph-bigwig.cwl/sorted_bedgraph_to_bigwig/chrom_length_file"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/bigwig_filename",
                            "id": "#bam-bedgraph-bigwig.cwl/sorted_bedgraph_to_bigwig/output_filename"
                        }
                    ],
                    "out": [
                        "#bam-bedgraph-bigwig.cwl/sorted_bedgraph_to_bigwig/bigwig_file"
                    ],
                    "id": "#bam-bedgraph-bigwig.cwl/sorted_bedgraph_to_bigwig"
                }
            ],
            "doc": "Workflow converts input BAM file into bigWig and bedGraph files.\n\nInput BAM file should be sorted by coordinates (required by `bam_to_bedgraph` step).\n\nIf `split` input is not provided use true by default. Default logic is implemented in `valueFrom` field of `split`\ninput inside `bam_to_bedgraph` step to avoid possible bug in cwltool with setting default values for workflow inputs.\n\n`scale` has higher priority over the `mapped_reads_number`. The last one is used to calculate `-scale` parameter for\n`bedtools genomecov` (step `bam_to_bedgraph`) only in a case when input `scale` is not provided. All logic is\nimplemented inside `bedtools-genomecov.cwl`.\n\n`bigwig_filename` defines the output name only for generated bigWig file. `bedgraph_filename` defines the output name\nfor generated bedGraph file and can influence on generated bigWig filename in case when `bigwig_filename` is not provided.\n\nAll workflow inputs and outputs don't have `format` field to avoid format incompatibility errors when workflow is used\nas subworkflow.",
            "id": "#bam-bedgraph-bigwig.cwl",
            "http://schema.org/name": "bam-bedgraph-bigwig",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "id": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        },
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Andrey Kartashov",
                                            "http://schema.org/email": "mailto:Andrey.Kartashov@cchmc.org",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "id": "#0000-0001-9102-5681"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputSource": "#bam-bedgraph-bigwig.cwl/sort_bedgraph/sorted_file",
                    "label": "bedGraph output file",
                    "doc": "Output bedGraph file",
                    "id": "#bam-bedgraph-bigwig.cwl/bedgraph_file"
                },
                {
                    "type": "File",
                    "outputSource": "#bam-bedgraph-bigwig.cwl/sorted_bedgraph_to_bigwig/bigwig_file",
                    "doc": "Output bigWig file",
                    "id": "#bam-bedgraph-bigwig.cwl/bigwig_file"
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function() { if (inputs.output_filename == \"\"){ var root = inputs.bam_file.basename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.bam_file.basename+\".bed\":root+\".bed\"; } else { return inputs.output_filename; } };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/bedtools2:v2.26.0"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 5,
                        "prefix": "-i"
                    },
                    "doc": "Input BAM file (not necessary sorted or indexed)",
                    "id": "#bedtools-bamtobed.cwl/bam_file"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "",
                    "doc": "Output BED filename",
                    "id": "#bedtools-bamtobed.cwl/output_filename"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(default_output_filename())"
                    },
                    "doc": "Sequences file",
                    "id": "#bedtools-bamtobed.cwl/bed_file"
                }
            ],
            "baseCommand": [
                "bedtools",
                "bamtobed"
            ],
            "stdout": "$(default_output_filename())",
            "doc": "Converts BAM to BED. All Options are not implemented.\n",
            "id": "#bedtools-bamtobed.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#bedtools-metadata.yaml",
                "http://schema.org/name": "bedtools",
                "http://schema.org/about": "A software suite for the comparison, manipulation and annotation of genomic features in browser extensible data (BED) and general feature format (GFF) format. BEDTools also supports the comparison of sequence alignments in BAM format to both BED and GFF features. The tools are extremely efficient and allow the user to compare large datasets (e.g. next-generation sequencing data) with both public and custom genome annotation tracks. BEDTools can be combined with one another as well as with standard UNIX commands, thus facilitating routine genomics tasks as well as pipelines that can quickly answer intricate questions of large genomic datasets.\n",
                "http://schema.org/url": "http://bedtools.readthedocs.org",
                "http://schema.org/codeRepository": "https://github.com/arq5x/bedtools2",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/GPL-2.0"
                ],
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "2.26.0",
                    "http://schema.org/applicationCategory": "command line tool"
                },
                "http://schema.org/programmingLanguage": "C++",
                "http://schema.org/publication": [
                    {
                        "class": "http://schema.org/ScholarlyArticle",
                        "id": "#bedtools-metadata.yaml/10.1093/bioinformatics/btq033",
                        "datePublished": "28 January 2010",
                        "http://schema.org/name": "BEDTools: a flexible suite of utilities for comparing genomic features",
                        "http://schema.org/author": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Aaron R. Quinlan"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Ira M. Hall"
                            }
                        ],
                        "http://schema.org/url": "https://academic.oup.com/bioinformatics/article-lookup/doi/10.1093/bioinformatics/btq033"
                    }
                ],
                "http://schema.org/discussionUrl": [
                    "https://github.com/arq5x/bedtools2/issues",
                    "https://groups.google.com/forum/#!forum/bedtools-discuss"
                ],
                "http://schema.org/creator": [
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "University of Utah",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Aaron Quinlan",
                                "http://schema.org/description": "Main author"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Neil Kindlon",
                                "http://schema.org/description": "Main author"
                            }
                        ]
                    }
                ]
            },
            "http://schema.org/name": "bedtools-bamtobed",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/bedtools-bamtobed.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Usage:   bedtools bamtobed [OPTIONS] -i <bam> \n\nOptions: \n  -bedpe\tWrite BEDPE format.\n    - Requires BAM to be grouped or sorted by query.\n\n  -mate1\tWhen writing BEDPE (-bedpe) format, \n    always report mate one as the first BEDPE \"block\".\n\n  -bed12\tWrite \"blocked\" BED format (aka \"BED12\"). Forces -split.\n\n    http://genome-test.cse.ucsc.edu/FAQ/FAQformat#format1\n\n  -split\tReport \"split\" BAM alignments as separate BED entries.\n    Splits only on N CIGAR operations.\n\n  -splitD\tSplit alignments based on N and D CIGAR operators.\n    Forces -split.\n\n  -ed\tUse BAM edit distance (NM tag) for BED score.\n    - Default for BED is to use mapping quality.\n    - Default for BEDPE is to use the minimum of\n      the two mapping qualities for the pair.\n    - When -ed is used with -bedpe, the total edit\n      distance from the two mates is reported.\n\n  -tag\tUse other NUMERIC BAM alignment tag for BED score.\n    - Default for BED is to use mapping quality.\n      Disallowed with BEDPE output.\n\n  -color\tAn R,G,B string for the color used with BED12 format.\n    Default is (255,0,0).\n\n  -cigar\tAdd the CIGAR string to the BED entry as a 7th column."
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function() { var ext = (inputs.depth == \"-bg\" || inputs.depth == \"-bga\")?\".bedGraph\":\".tab\"; return inputs.input_file.location.split('/').slice(-1)[0].split('.').slice(0,-1).join('.') + ext; };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/bedtools2:v2.26.0"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 17,
                        "prefix": "-g"
                    },
                    "doc": "Input genome file. Needed only when -i flag. The genome file is tab delimited <chromName><TAB><chromSize>\n",
                    "id": "#bedtools-genomecov.cwl/chrom_length_file"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "enum",
                            "name": "#bedtools-genomecov.cwl/depth/depth",
                            "symbols": [
                                "#bedtools-genomecov.cwl/depth/depth/-bg",
                                "#bedtools-genomecov.cwl/depth/depth/-bga",
                                "#bedtools-genomecov.cwl/depth/depth/-d",
                                "#bedtools-genomecov.cwl/depth/depth/-dz"
                            ]
                        }
                    ],
                    "inputBinding": {
                        "position": 5
                    },
                    "doc": "Report the depth type. By default, bedtools genomecov will compute a histogram of coverage\nfor the genome file provided (intputs.chrom_length_file)\n",
                    "id": "#bedtools-genomecov.cwl/depth"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 11,
                        "prefix": "-du"
                    },
                    "doc": "Change strand af the mate read (so both reads from the same strand) useful for strand specific.\nWorks for BAM files only\n",
                    "id": "#bedtools-genomecov.cwl/du"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 12,
                        "prefix": "-fs"
                    },
                    "doc": "Set fixed fragment size\n",
                    "id": "#bedtools-genomecov.cwl/fragment_size"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 16,
                        "valueFrom": "${\n  var prefix = ((/.*\\.bam$/i).test(inputs.input_file.path))?'-ibam':'-i';\n  return [prefix, inputs.input_file.path];\n}\n"
                    },
                    "doc": "The input file can be in BAM format (Note: BAM must be sorted by position) or <bed/gff/vcf>.\nPrefix is selected on the base of input file extension\n",
                    "id": "#bedtools-genomecov.cwl/input_file"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 15,
                        "prefix": "-3"
                    },
                    "doc": "Calculate coverage of 3\" positions (instead of entire interval)\n",
                    "id": "#bedtools-genomecov.cwl/m3"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 14,
                        "prefix": "-5"
                    },
                    "doc": "Calculate coverage of 5\" positions (instead of entire interval)\n",
                    "id": "#bedtools-genomecov.cwl/m5"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 7,
                        "prefix": "-scale",
                        "valueFrom": "${\n  if (inputs.scale){\n    return null;\n  } else if (inputs.mapped_reads_number) {\n    return 1000000/inputs.mapped_reads_number;\n  } else {\n    return null;\n  }\n}\n"
                    },
                    "doc": "Optional parameter to calculate scale as 1000000/mapped_reads_number if inputs.scale is not provided\n",
                    "id": "#bedtools-genomecov.cwl/mapped_reads_number"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 13,
                        "prefix": "-max"
                    },
                    "doc": "Combine all positions with a depth >= max into\na single bin in the histogram. Irrelevant\nfor -d and -bedGraph\n- (INTEGER)\n",
                    "id": "#bedtools-genomecov.cwl/max"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Name for generated output file\n",
                    "id": "#bedtools-genomecov.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 10,
                        "prefix": "-pc"
                    },
                    "doc": "pair-end chip seq experiment\n",
                    "id": "#bedtools-genomecov.cwl/pairchip"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 6,
                        "prefix": "-scale"
                    },
                    "doc": "Scale the coverage by a constant factor.\nEach coverage value is multiplied by this factor before being reported.\nUseful for normalizing coverage by, e.g., reads per million (RPM).\n- Default is 1.0; i.e., unscaled.\n- (FLOAT)\n",
                    "id": "#bedtools-genomecov.cwl/scale"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 8,
                        "prefix": "-split"
                    },
                    "doc": "treat \"split\" BAM or BED12 entries as distinct BED intervals.\nwhen computing coverage.\nFor BAM files, this uses the CIGAR \"N\" and \"D\" operations\nto infer the blocks for computing coverage.\nFor BED12 files, this uses the BlockCount, BlockStarts, and BlockEnds\nfields (i.e., columns 10,11,12).\n",
                    "id": "#bedtools-genomecov.cwl/split"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 9,
                        "prefix": "-strand"
                    },
                    "doc": "Calculate coverage of intervals from a specific strand.\nWith BED files, requires at least 6 columns (strand is column 6).\n- (STRING): can be + or -\n",
                    "id": "#bedtools-genomecov.cwl/strand"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "doc": "Generated genome coverage output file\n",
                    "id": "#bedtools-genomecov.cwl/genome_coverage_file",
                    "outputBinding": {
                        "glob": "${\n    return inputs.output_filename ? inputs.output_filename : default_output_filename();\n}\n"
                    }
                }
            ],
            "stdout": "${\n    return inputs.output_filename ? inputs.output_filename : default_output_filename();\n}\n",
            "baseCommand": [
                "bedtools",
                "genomecov"
            ],
            "doc": "Tool calculates genome coverage from input bam/bed/gff/vcf using `bedtools genomecov`\n\nDepending on `input_file` extension additional prefix is used: if `*.bam` use `-ibam`, else use `-i`.\n\n`scale` and `mapped_reads_number` inputs result in the same parameter `-scale`. If `scale` is not provided, check if\n`mapped_reads_number` is not null and calculate `-scale` as `1000000/mapped_reads_number`. If both inputs are\nnull, `bedtools genomecov` will use its default scaling value.\n\n`default_output_filename` function returns default output filename and is used when `output_filename` is not provided.\nDefault output file extention is `.tab`. If bedGraph should be generated (check flags `inputs.depth`), extension is\nupdated to `.bedGraph`. Default basename of the output file is generated on the base of `input_file` basename.\n",
            "id": "#bedtools-genomecov.cwl",
            "http://schema.org/mainEntity": {
                "$import": "#bedtools-metadata.yaml"
            },
            "http://schema.org/name": "bedtools-genomecov",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/bedtools-genomecov.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Usage: bedtools genomecov [OPTIONS] -i <bed/gff/vcf> -g <genome>\n\nOptions:\n\t-ibam\t\tThe input file is in BAM format.\n\t\t\tNote: BAM _must_ be sorted by position\n\n\t-d\t\tReport the depth at each genome position (with one-based coordinates).\n\t\t\tDefault behavior is to report a histogram.\n\n\t-dz\t\tReport the depth at each genome position (with zero-based coordinates).\n\t\t\tReports only non-zero positions.\n\t\t\tDefault behavior is to report a histogram.\n\n\t-bg\t\tReport depth in BedGraph format. For details, see:\n\t\t\tgenome.ucsc.edu/goldenPath/help/bedgraph.html\n\n\t-bga\t\tReport depth in BedGraph format, as above (-bg).\n\t\t\tHowever with this option, regions with zero\n\t\t\tcoverage are also reported. This allows one to\n\t\t\tquickly extract all regions of a genome with 0\n\t\t\tcoverage by applying: \"grep -w 0$\" to the output.\n\n\t-split\t\tTreat \"split\" BAM or BED12 entries as distinct BED intervals.\n\t\t\twhen computing coverage.\n\t\t\tFor BAM files, this uses the CIGAR \"N\" and \"D\" operations\n\t\t\tto infer the blocks for computing coverage.\n\t\t\tFor BED12 files, this uses the BlockCount, BlockStarts, and BlockEnds\n\t\t\tfields (i.e., columns 10,11,12).\n\n\t-strand\t\tCalculate coverage of intervals from a specific strand.\n\t\t\tWith BED files, requires at least 6 columns (strand is column 6).\n\t\t\t- (STRING): can be + or -\n\n\t-pc\t\tCalculate coverage of pair-end fragments.\n\t\t\tWorks for BAM files only\n\t-fs\t\tForce to use provided fragment size instead of read length\n\t\t\tWorks for BAM files only\n\t-du\t\tChange strand af the mate read (so both reads from the same strand) useful for strand specific\n\t\t\tWorks for BAM files only\n\t-5\t\tCalculate coverage of 5\" positions (instead of entire interval).\n\n\t-3\t\tCalculate coverage of 3\" positions (instead of entire interval).\n\n\t-max\t\tCombine all positions with a depth >= max into\n\t\t\ta single bin in the histogram. Irrelevant\n\t\t\tfor -d and -bedGraph\n\t\t\t- (INTEGER)\n\n\t-scale\t\tScale the coverage by a constant factor.\n\t\t\tEach coverage value is multiplied by this factor before being reported.\n\t\t\tUseful for normalizing coverage by, e.g., reads per million (RPM).\n\t\t\t- Default is 1.0; i.e., unscaled.\n\t\t\t- (FLOAT)\n\n\t-trackline\tAdds a UCSC/Genome-Browser track line definition in the first line of the output.\n\t\t\t- See here for more details about track line definition:\n\t\t\t      http://genome.ucsc.edu/goldenPath/help/bedgraph.html\n\t\t\t- NOTE: When adding a trackline definition, the output BedGraph can be easily\n\t\t\t      uploaded to the Genome Browser as a custom track,\n\t\t\t      BUT CAN NOT be converted into a BigWig file (w/o removing the first line).\n\n\t-trackopts\tWrites additional track line definition parameters in the first line.\n\t\t\t- Example:\n\t\t\t   -trackopts 'name=\"My Track\" visibility=2 color=255,30,30'\n\t\t\t   Note the use of single-quotes if you have spaces in your parameters.\n\t\t\t- (TEXT)\n\nNotes:\n\t(1) The genome file should tab delimited and structured as follows:\n\t <chromName><TAB><chromSize>\n\n\tFor example, Human (hg19):\n\tchr1\t249250621\n\tchr2\t243199373\n\t...\n\tchr18_gl000207_random\t4262\n\n\t(2) The input BED (-i) file must be grouped by chromosome.\n\t A simple \"sort -k 1,1 <BED> > <BED>.sorted\" will suffice.\n\n\t(3) The input BAM (-ibam) file must be sorted by position.\n\t A \"samtools sort <BAM>\" should suffice.\n\nTips:\n\tOne can use the UCSC Genome Browser's MySQL database to extract\n\tchromosome sizes. For example, H. sapiens:\n\n\tmysql --user=genome --host=genome-mysql.cse.ucsc.edu -A -e \\\n\t\"select chrom, size from hg19.chromInfo\" > hg19.genome\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function(ext) { if (inputs.output_filename != \"\" && !ext){ return inputs.output_filename; } ext = ext || \".sam\"; var root = \"\"; if (inputs.output_filename != \"\"){ root = inputs.output_filename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.output_filename+ext:root+ext; } else if (Array.isArray(inputs.upstream_filelist) && inputs.upstream_filelist.length > 0){ root = inputs.upstream_filelist[0].basename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.upstream_filelist[0].basename+ext:root+ext; } else if (inputs.upstream_filelist != null){ root = inputs.upstream_filelist.basename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.upstream_filelist.basename+ext:root+ext; } else if (Array.isArray(inputs.downstream_filelist) && inputs.downstream_filelist.length > 0){ root = inputs.downstream_filelist[0].basename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.downstream_filelist[0].basename+ext:root+ext; } else if (inputs.downstream_filelist != null){ root = inputs.downstream_filelist.basename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.downstream_filelist.basename+ext:root+ext; } else if (Array.isArray(inputs.crossbow_filelist) && inputs.crossbow_filelist.length > 0){ root = inputs.crossbow_filelist[0].basename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.crossbow_filelist[0].basename+ext:root+ext; } else if (inputs.crossbow_filelist != null){ root = inputs.crossbow_filelist.basename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.crossbow_filelist.basename+ext:root+ext; } else { return null; } };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/bowtie:v1.2.0"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-B"
                    },
                    "doc": "--offbase <int> leftmost ref offset = <int> in bowtie output (default: 0)\n",
                    "id": "#bowtie-alignreads.cwl/B"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-C"
                    },
                    "doc": "reads and index are in colorspace\n",
                    "id": "#bowtie-alignreads.cwl/C"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-I"
                    },
                    "doc": "--minins <int>  minimum insert size for paired-end alignment (default: 0)\n",
                    "id": "#bowtie-alignreads.cwl/I"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-M"
                    },
                    "doc": "<int> like -m, but reports 1 random hit (MAPQ=0)\n",
                    "id": "#bowtie-alignreads.cwl/M"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-Q"
                    },
                    "doc": "QV file(s) corresponding to CSFASTA inputs; use with -f -C\n",
                    "id": "#bowtie-alignreads.cwl/Q"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--Q1"
                    },
                    "doc": "--Q1/--Q2 <file>   same as -Q, but for mate files 1 and 2 respectively\n",
                    "id": "#bowtie-alignreads.cwl/Q1"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--Q2"
                    },
                    "doc": "--Q1/--Q2 <file>   same as -Q, but for mate files 1 and 2 respectively\n",
                    "id": "#bowtie-alignreads.cwl/Q2"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-X"
                    },
                    "doc": "--maxins <int>  maximum insert size for paired-end alignment (default: 250)\n",
                    "id": "#bowtie-alignreads.cwl/X"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-a"
                    },
                    "doc": "--all           report all alignments per read (much slower than low -k)\n",
                    "id": "#bowtie-alignreads.cwl/a"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--al",
                        "valueFrom": "$(self + \".fastq\")"
                    },
                    "doc": "<fname>       write aligned reads/pairs to file(s) <fname>\n",
                    "id": "#bowtie-alignreads.cwl/aligned_prefix"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--best"
                    },
                    "doc": "hits guaranteed best stratum; ties broken by quality\n",
                    "id": "#bowtie-alignreads.cwl/best"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-c"
                    },
                    "doc": "query sequences given on cmd line (as <mates>, <singles>)\n",
                    "id": "#bowtie-alignreads.cwl/c"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chunkmbs"
                    },
                    "doc": "<int>   max megabytes of RAM for best-first search frames (def: 64)\n",
                    "id": "#bowtie-alignreads.cwl/chunkmbs"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-3"
                    },
                    "doc": "--trim3 <int>   trim <int> bases from 3' (right) end of reads\n",
                    "id": "#bowtie-alignreads.cwl/clip_3p_end"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-5"
                    },
                    "doc": "--trim5 <int>   trim <int> bases from 5' (left) end of reads\n",
                    "id": "#bowtie-alignreads.cwl/clip_5p_end"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--col-cqual"
                    },
                    "doc": "print original colorspace quals, not decoded quals\n",
                    "id": "#bowtie-alignreads.cwl/col_cqual"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--col-cseq"
                    },
                    "doc": "print aligned colorspace seqs as colors, not decoded bases\n",
                    "id": "#bowtie-alignreads.cwl/col_cseq"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--col-keepends"
                    },
                    "doc": "keep nucleotides at extreme ends of decoded alignment\n",
                    "id": "#bowtie-alignreads.cwl/col_keepends"
                },
                {
                    "type": [
                        "null",
                        "File",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "inputBinding": {
                        "itemSeparator": ",",
                        "position": 86,
                        "prefix": "-12"
                    },
                    "doc": "Comma-separated list of files containing Crossbow-style reads.\nCan be a mixture of paired and unpaired.  Specify \"-\"for stdin.\n",
                    "id": "#bowtie-alignreads.cwl/crossbow_filelist"
                },
                {
                    "type": [
                        "null",
                        "File",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "inputBinding": {
                        "itemSeparator": ",",
                        "position": 85
                    },
                    "doc": "Comma-separated list of files containing downstream mates (or the\nsequences themselves if -c is set) paired with mates in <m1>\n",
                    "id": "#bowtie-alignreads.cwl/downstream_filelist"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-e"
                    },
                    "doc": "--maqerr <int>  max sum of mismatch quals across alignment for -n (def: 70)\n",
                    "id": "#bowtie-alignreads.cwl/e"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-f"
                    },
                    "doc": "query input files are (multi-)FASTA .fa/.mfa\n",
                    "id": "#bowtie-alignreads.cwl/f"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--ff"
                    },
                    "doc": "--fr/--rf/--ff     -1, -2 mates align fw/rev, rev/fw, fw/fw (default: --fr)\n",
                    "id": "#bowtie-alignreads.cwl/ff"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--fr"
                    },
                    "doc": "--fr/--rf/--ff     -1, -2 mates align fw/rev, rev/fw, fw/fw (default: --fr)\n",
                    "id": "#bowtie-alignreads.cwl/fr"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--fullref"
                    },
                    "doc": "write entire ref name (default: only up to 1st space)\n",
                    "id": "#bowtie-alignreads.cwl/fullref"
                },
                {
                    "type": "Directory",
                    "inputBinding": {
                        "position": 81,
                        "valueFrom": "${\n    for (var i = 0; i < self.listing.length; i++) {\n        if (self.listing[i].path.split('.').slice(-3).join('.') == 'rev.1.ebwt' ||\n            self.listing[i].path.split('.').slice(-3).join('.') == 'rev.1.ebwtl'){\n          return self.listing[i].path.split('.').slice(0,-3).join('.');\n        }\n    }\n    return null;\n}\n"
                    },
                    "doc": "Folder with Bowtie indices\n",
                    "id": "#bowtie-alignreads.cwl/indices_folder"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--integer-quals"
                    },
                    "doc": "qualities are given as space-separated integers (not ASCII)\n",
                    "id": "#bowtie-alignreads.cwl/integer_quals"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-k"
                    },
                    "doc": "<int>           report up to <int> good alignments per read (default: 1)\n",
                    "id": "#bowtie-alignreads.cwl/k"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-l"
                    },
                    "doc": "--seedlen <int> seed length for -n (default: 28)\n",
                    "id": "#bowtie-alignreads.cwl/l"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--large-index"
                    },
                    "doc": "force usage of a 'large' index, even if a small one is present\n",
                    "id": "#bowtie-alignreads.cwl/large_index"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-m"
                    },
                    "doc": "<int>           suppress all alignments if > <int> exist (def: no limit)\n",
                    "id": "#bowtie-alignreads.cwl/m"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--mapq"
                    },
                    "doc": "<int>       default mapping quality (MAPQ) to print for SAM alignments\n",
                    "id": "#bowtie-alignreads.cwl/mapq"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--maxbts"
                    },
                    "doc": "<int>     max # backtracks for -n 2/3 (default: 125, 800 for --best)\n",
                    "id": "#bowtie-alignreads.cwl/maxbts"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--mm"
                    },
                    "doc": "use memory-mapped I/O for index; many 'bowtie's can share\n",
                    "id": "#bowtie-alignreads.cwl/mm"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--max",
                        "valueFrom": "$(self + \".fastq\")"
                    },
                    "doc": "<fname>      write reads/pairs over -m limit to file(s) <fname>\n",
                    "id": "#bowtie-alignreads.cwl/multimapped_prefix"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-n"
                    },
                    "doc": "--seedmms <int> max mismatches in seed (can be 0-3, default: -n 2)\n",
                    "id": "#bowtie-alignreads.cwl/n"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--nofw"
                    },
                    "doc": "--norc      do not align to forward/reverse-complement reference strand\n",
                    "id": "#bowtie-alignreads.cwl/nofw"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--nomaqround"
                    },
                    "doc": "disable Maq-like quality rounding for -n (nearest 10 <= 30)\n",
                    "id": "#bowtie-alignreads.cwl/nomaqround"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-o"
                    },
                    "doc": "--offrate <int> override offrate of index; must be >= index's offrate\n",
                    "id": "#bowtie-alignreads.cwl/o"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 90,
                        "valueFrom": "$(default_output_filename())"
                    },
                    "default": "",
                    "doc": "Generates default output filename on the base of upstream_filelist/downstream_filelist files\n",
                    "id": "#bowtie-alignreads.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--pairtries"
                    },
                    "doc": "<int>  max # attempts to find mate for anchor hit (default: 100)\n",
                    "id": "#bowtie-alignreads.cwl/pairtries"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--phred33-quals"
                    },
                    "doc": "input quals are Phred+33 (default)\n",
                    "id": "#bowtie-alignreads.cwl/phred33_quals"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--phred64-quals"
                    },
                    "doc": "input quals are Phred+64 (same as --solexa1.3-quals)\n",
                    "id": "#bowtie-alignreads.cwl/phred64_quals"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-q"
                    },
                    "doc": "query input files are FASTQ .fq/.fastq (default)\n",
                    "id": "#bowtie-alignreads.cwl/q"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--quiet"
                    },
                    "doc": "print nothing but the alignments\n",
                    "id": "#bowtie-alignreads.cwl/quiet"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-r"
                    },
                    "doc": "query input files are raw one-sequence-per-line\n",
                    "id": "#bowtie-alignreads.cwl/r"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--reads-per-batch"
                    },
                    "doc": "# of reads to read from input file at once (default: 16)\n",
                    "id": "#bowtie-alignreads.cwl/reads_per_batch"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--refidx"
                    },
                    "doc": "refer to ref. seqs by 0-based index rather than name\n",
                    "id": "#bowtie-alignreads.cwl/refidx"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--refout"
                    },
                    "doc": "write alignments to files refXXXXX.map, 1 map per reference\n",
                    "id": "#bowtie-alignreads.cwl/refout"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--rf"
                    },
                    "doc": "--fr/--rf/--ff     -1, -2 mates align fw/rev, rev/fw, fw/fw (default: --fr)\n",
                    "id": "#bowtie-alignreads.cwl/rf"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-s"
                    },
                    "doc": "--skip <int>    skip the first <int> reads/pairs in the input\n",
                    "id": "#bowtie-alignreads.cwl/s"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-S"
                    },
                    "doc": "--sam write hits in SAM format\n",
                    "id": "#bowtie-alignreads.cwl/sam"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sam-RG"
                    },
                    "doc": "<text>    add <text> (usually \"lab=value\") to @RG line of SAM header\n",
                    "id": "#bowtie-alignreads.cwl/sam_RG"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sam-nohead"
                    },
                    "doc": "supppress header lines (starting with @) for SAM output\n",
                    "id": "#bowtie-alignreads.cwl/sam_nohead"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sam-nosq"
                    },
                    "doc": "supppress @SQ header lines for SAM output\n",
                    "id": "#bowtie-alignreads.cwl/sam_nosq"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seed"
                    },
                    "doc": "<int>       seed for random number generator\n",
                    "id": "#bowtie-alignreads.cwl/seed"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--shmem"
                    },
                    "doc": "use shared mem for index\n",
                    "id": "#bowtie-alignreads.cwl/shmem"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--snpfrac"
                    },
                    "doc": "<dec>    approx. fraction of SNP bases (e.g. 0.001); sets --snpphred\n",
                    "id": "#bowtie-alignreads.cwl/snpfrac"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--snpphred"
                    },
                    "doc": "<int>   Phred penalty for SNP when decoding colorspace (def: 30)\n",
                    "id": "#bowtie-alignreads.cwl/snpphred"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--solexa1.3-quals"
                    },
                    "doc": "input quals are from GA Pipeline ver. >= 1.3\n",
                    "id": "#bowtie-alignreads.cwl/solexa1.3_quals"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--solexa-quals"
                    },
                    "doc": "input quals are from GA Pipeline ver. < 1.3\n",
                    "id": "#bowtie-alignreads.cwl/solexa_quals"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--strata"
                    },
                    "doc": "hits in sub-optimal strata aren't reported (requires --best)\n",
                    "id": "#bowtie-alignreads.cwl/strata"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--suppress"
                    },
                    "doc": "<cols>  suppresses given columns (comma-delim'ed) in default output\n",
                    "id": "#bowtie-alignreads.cwl/suppress"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-t"
                    },
                    "doc": "--time          print wall-clock time taken by search phases\n",
                    "id": "#bowtie-alignreads.cwl/t"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-p"
                    },
                    "doc": "--threads <int> number of alignment threads to launch (default: 1)\n",
                    "id": "#bowtie-alignreads.cwl/threads"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-u"
                    },
                    "doc": "--qupto <int>   stop after first <int> reads/pairs (excl. skipped reads)\n",
                    "id": "#bowtie-alignreads.cwl/u"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--un",
                        "valueFrom": "$(self + \".fastq\")"
                    },
                    "doc": "<fname>       write unaligned reads/pairs to file(s) <fname>\n",
                    "id": "#bowtie-alignreads.cwl/unaligned_prefix"
                },
                {
                    "type": [
                        "null",
                        "File",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "inputBinding": {
                        "itemSeparator": ",",
                        "position": 83
                    },
                    "doc": "Comma-separated list of files containing upstream mates (or the\nsequences themselves, if -c is set) paired with mates in <m2>\n",
                    "id": "#bowtie-alignreads.cwl/upstream_filelist"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-v"
                    },
                    "doc": "report end-to-end hits w/ <=v mismatches; ignore qualities\n",
                    "id": "#bowtie-alignreads.cwl/v"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--verbose"
                    },
                    "doc": "verbose output (for debugging)\n",
                    "id": "#bowtie-alignreads.cwl/verbose"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-y"
                    },
                    "doc": "--tryhard try hard to find valid alignments, at the expense of speed\n",
                    "id": "#bowtie-alignreads.cwl/y"
                }
            ],
            "outputs": [
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "outputBinding": {
                        "glob": "$(inputs.aligned_prefix + \"*\")"
                    },
                    "id": "#bowtie-alignreads.cwl/aligned_fastq"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(default_output_filename(\".bw\"))"
                    },
                    "id": "#bowtie-alignreads.cwl/log_file"
                },
                {
                    "type": "int",
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "$(default_output_filename(\".bw\"))",
                        "outputEval": "${\n  var mappedRegex = /alignment\\:.*/;\n  return parseInt(self[0].contents.match(mappedRegex)[0].split(\" \")[1]);\n}\n"
                    },
                    "id": "#bowtie-alignreads.cwl/mapped_reads_number"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "outputBinding": {
                        "glob": "$(inputs.multimapped_prefix + \"*\")"
                    },
                    "id": "#bowtie-alignreads.cwl/multimapped_fastq"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "outputBinding": {
                        "glob": "*ref*.map*"
                    },
                    "id": "#bowtie-alignreads.cwl/refout_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "$(default_output_filename())"
                    },
                    "id": "#bowtie-alignreads.cwl/sam_file"
                },
                {
                    "type": "int",
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "$(default_output_filename(\".bw\"))",
                        "outputEval": "${\n  var totalRegex = /processed\\:.*/;\n  return parseInt(self[0].contents.match(totalRegex)[0].split(\" \")[1]);\n}\n"
                    },
                    "id": "#bowtie-alignreads.cwl/total_reads_number"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "outputBinding": {
                        "glob": "$(inputs.unaligned_prefix + \"*\")"
                    },
                    "id": "#bowtie-alignreads.cwl/unaligned_fastq"
                },
                {
                    "type": "int",
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "$(default_output_filename(\".bw\"))",
                        "outputEval": "${\n  var unmappedRegex = /align\\:.*/;\n  return parseInt(self[0].contents.match(unmappedRegex)[0].split(\" \")[1]);\n}\n"
                    },
                    "id": "#bowtie-alignreads.cwl/unmapped_reads_number"
                }
            ],
            "baseCommand": [
                "bowtie"
            ],
            "arguments": [
                {
                    "valueFrom": "${\n  if (inputs.upstream_filelist && inputs.downstream_filelist){\n    return \"-1\";\n  }\n  return null;\n}\n",
                    "position": 82
                },
                {
                    "valueFrom": "${\n  if (inputs.upstream_filelist && inputs.downstream_filelist){\n    return \"-2\";\n  }\n  return null;\n}\n",
                    "position": 84
                },
                {
                    "valueFrom": "${\n  return ' 2> ' + default_output_filename(\".bw\");\n}\n",
                    "position": 100000,
                    "shellQuote": false
                }
            ],
            "doc": "Tool maps input raw reads files to reference genome using Bowtie.\n\n`default_output_filename` function returns default name for SAM output and log files. In case when `sam` and\n`output_filename` inputs are not set, default filename will have `.sam` extension but format may not correspond SAM\nspecification. To set output filename manually use `output_filename` input. Default output filename is based on\n`output_filename` or basename of `upstream_filelist`, `downstream_filelist` or `crossbow_filelist` file (if array,\nthe first file in array is taken). If function is called without argenments and `output_filename` input is set, it\nwill be returned from the function.\n\nFor single-end input data any of the `upstream_filelist` or `downstream_filelist` inputs can be used.\n\nLog filename (`log_file` output) is generated by `default_output_filename` function with ex='.bw'\n\n`indices_folder` defines folder to contain Bowtie indices. Based on the first found file with `rev.1.ebwt` or\n`rev.1.ebwtl` extension, bowtie index prefix is returned from input's `valueFrom` field.\n",
            "id": "#bowtie-alignreads.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#bowtie-metadata.yaml",
                "http://schema.org/name": "bowtie2",
                "http://schema.org/about": "Bowtie is an ultrafast, memory-efficient short read aligner. It aligns short DNA sequences (reads)\nto the human genome at a rate of over 25 million 35-bp reads per hour. Bowtie indexes the genome\nwith a Burrows-Wheeler index to keep its memory footprint small: typically about 2.2 GB for\nthe human genome (2.9 GB for paired-end).\n",
                "http://schema.org/url": "http://bowtie-bio.sourceforge.net/index.shtml",
                "http://schema.org/codeRepository": "https://github.com/BenLangmead/bowtie",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/GPL-3.0"
                ],
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "1.2.0",
                    "http://schema.org/applicationCategory": "command line tool"
                },
                "http://schema.org/programmingLanguage": "C++",
                "http://schema.org/publication": [
                    {
                        "class": "http://schema.org/ScholarlyArticle",
                        "id": "#bowtie-metadata.yaml/10.1186/gb-2009-10-3-r25",
                        "datePublished": "04 March 2009",
                        "http://schema.org/name": "Ultrafast and memory-efficient alignment of short DNA sequences to the human genome",
                        "http://schema.org/author": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Ben Langmead"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Steven L Salzberg"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Cole Trapnell"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Mihai Pop"
                            }
                        ],
                        "http://schema.org/url": "https://genomebiology.biomedcentral.com/articles/10.1186/gb-2009-10-3-r25"
                    }
                ],
                "http://schema.org/discussionUrl": [
                    "https://github.com/BenLangmead/bowtie/issues"
                ],
                "http://schema.org/creator": [
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "Johns Hopkins University",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Ben Langmead",
                                "http://schema.org/description": "Main author"
                            }
                        ]
                    }
                ]
            },
            "http://schema.org/name": "bowtie",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/bowtie.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Andrey Kartashov",
                                            "http://schema.org/email": "mailto:Andrey.Kartashov@cchmc.org",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0001-9102-5681"
                                                }
                                            ]
                                        },
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Usage:\nbowtie [options]* <ebwt> {-1 <m1> -2 <m2> | --12 <r> | <s>} [<hit>]\n\n  <m1>    Comma-separated list of files containing upstream mates (or the\n          sequences themselves, if -c is set) paired with mates in <m2>\n  <m2>    Comma-separated list of files containing downstream mates (or the\n          sequences themselves if -c is set) paired with mates in <m1>\n  <r>     Comma-separated list of files containing Crossbow-style reads.  Can be\n          a mixture of paired and unpaired.  Specify \"-\" for stdin.\n  <s>     Comma-separated list of files containing unpaired reads, or the\n          sequences themselves, if -c is set.  Specify \"-\" for stdin.\n  <hit>   File to write hits to (default: stdout)\nInput:\n  -q                 query input files are FASTQ .fq/.fastq (default)\n  -f                 query input files are (multi-)FASTA .fa/.mfa\n  -r                 query input files are raw one-sequence-per-line\n  -c                 query sequences given on cmd line (as <mates>, <singles>)\n  -C                 reads and index are in colorspace\n  -Q/--quals <file>  QV file(s) corresponding to CSFASTA inputs; use with -f -C\n  --Q1/--Q2 <file>   same as -Q, but for mate files 1 and 2 respectively\n  -s/--skip <int>    skip the first <int> reads/pairs in the input\n  -u/--qupto <int>   stop after first <int> reads/pairs (excl. skipped reads)\n  -5/--trim5 <int>   trim <int> bases from 5' (left) end of reads\n  -3/--trim3 <int>   trim <int> bases from 3' (right) end of reads\n  --phred33-quals    input quals are Phred+33 (default)\n  --phred64-quals    input quals are Phred+64 (same as --solexa1.3-quals)\n  --solexa-quals     input quals are from GA Pipeline ver. < 1.3\n  --solexa1.3-quals  input quals are from GA Pipeline ver. >= 1.3\n  --integer-quals    qualities are given as space-separated integers (not ASCII)\n  --large-index      force usage of a 'large' index, even if a small one is present\nAlignment:\n  -v <int>           report end-to-end hits w/ <=v mismatches; ignore qualities\n    or\n  -n/--seedmms <int> max mismatches in seed (can be 0-3, default: -n 2)\n  -e/--maqerr <int>  max sum of mismatch quals across alignment for -n (def: 70)\n  -l/--seedlen <int> seed length for -n (default: 28)\n  --nomaqround       disable Maq-like quality rounding for -n (nearest 10 <= 30)\n  -I/--minins <int>  minimum insert size for paired-end alignment (default: 0)\n  -X/--maxins <int>  maximum insert size for paired-end alignment (default: 250)\n  --fr/--rf/--ff     -1, -2 mates align fw/rev, rev/fw, fw/fw (default: --fr)\n  --nofw/--norc      do not align to forward/reverse-complement reference strand\n  --maxbts <int>     max # backtracks for -n 2/3 (default: 125, 800 for --best)\n  --pairtries <int>  max # attempts to find mate for anchor hit (default: 100)\n  -y/--tryhard       try hard to find valid alignments, at the expense of speed\n  --chunkmbs <int>   max megabytes of RAM for best-first search frames (def: 64)\n --reads-per-batch   # of reads to read from input file at once (default: 16)\nReporting:\n  -k <int>           report up to <int> good alignments per read (default: 1)\n  -a/--all           report all alignments per read (much slower than low -k)\n  -m <int>           suppress all alignments if > <int> exist (def: no limit)\n  -M <int>           like -m, but reports 1 random hit (MAPQ=0); requires --best\n  --best             hits guaranteed best stratum; ties broken by quality\n  --strata           hits in sub-optimal strata aren't reported (requires --best)\nOutput:\n  -t/--time          print wall-clock time taken by search phases\n  -B/--offbase <int> leftmost ref offset = <int> in bowtie output (default: 0)\n  --quiet            print nothing but the alignments\n  --refout           write alignments to files refXXXXX.map, 1 map per reference\n  --refidx           refer to ref. seqs by 0-based index rather than name\n  --al <fname>       write aligned reads/pairs to file(s) <fname>\n  --un <fname>       write unaligned reads/pairs to file(s) <fname>\n  --max <fname>      write reads/pairs over -m limit to file(s) <fname>\n  --suppress <cols>  suppresses given columns (comma-delim'ed) in default output\n  --fullref          write entire ref name (default: only up to 1st space)\nColorspace:\n  --snpphred <int>   Phred penalty for SNP when decoding colorspace (def: 30)\n     or\n  --snpfrac <dec>    approx. fraction of SNP bases (e.g. 0.001); sets --snpphred\n  --col-cseq         print aligned colorspace seqs as colors, not decoded bases\n  --col-cqual        print original colorspace quals, not decoded quals\n  --col-keepends     keep nucleotides at extreme ends of decoded alignment\nSAM:\n  -S/--sam           write hits in SAM format\n  --mapq <int>       default mapping quality (MAPQ) to print for SAM alignments\n  --sam-nohead       supppress header lines (starting with @) for SAM output\n  --sam-nosq         supppress @SQ header lines for SAM output\n  --sam-RG <text>    add <text> (usually \"lab=value\") to @RG line of SAM header\nPerformance:\n  -o/--offrate <int> override offrate of index; must be >= index's offrate\n  -p/--threads <int> number of alignment threads to launch (default: 1)\n  --mm               use memory-mapped I/O for index; many 'bowtie's can share\n  --shmem            use shared mem for index; many 'bowtie's can share\nOther:\n  --seed <int>       seed for random number generator\n  --verbose          verbose output (for debugging)\n  --version          print version information and quit\n  -h/--help          print this usage message\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function() { return inputs.output_filename == \"\" ? inputs.bed_file.nameroot+\".peaks.bed\":inputs.output_filename; };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/clip-toolkit:v0.0.1"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 30
                    },
                    "doc": "IInput BED file of unique CLIP tags",
                    "id": "#clip-toolkit-tag2peak.cwl/bed_file"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "prefix": "-big"
                    },
                    "doc": "Big input file",
                    "id": "#clip-toolkit-tag2peak.cwl/big"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "prefix": "--out-boundary"
                    },
                    "doc": "Output cluster boundaries",
                    "id": "#clip-toolkit-tag2peak.cwl/cluster_boundaries"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "enum",
                            "symbols": [
                                "#clip-toolkit-tag2peak.cwl/default_gene_bed/mm10",
                                "#clip-toolkit-tag2peak.cwl/default_gene_bed/hg19"
                            ]
                        }
                    ],
                    "inputBinding": {
                        "prefix": "--dbkey"
                    },
                    "doc": "Species to retrieve the default gene bed file (mm10|hg19)",
                    "id": "#clip-toolkit-tag2peak.cwl/default_gene_bed"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "prefix": "--gene"
                    },
                    "doc": "Custom gene bed file for scan statistics (will override --dbkey)",
                    "id": "#clip-toolkit-tag2peak.cwl/gene_bed_file"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "prefix": "--out-half-PH"
                    },
                    "doc": "Output half peak height boundaries",
                    "id": "#clip-toolkit-tag2peak.cwl/half_peak_height_boundaries"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "prefix": "--maxPH"
                    },
                    "doc": "Max peak height to calculate p-value(-1, no limit if < 0)",
                    "id": "#clip-toolkit-tag2peak.cwl/max_peak_height"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "prefix": "--minPH"
                    },
                    "doc": "Min peak height (2)",
                    "id": "#clip-toolkit-tag2peak.cwl/min_peak_height"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "prefix": "--multi-test"
                    },
                    "doc": "Do Bonferroni multiple test correction",
                    "id": "#clip-toolkit-tag2peak.cwl/multi_test"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 31,
                        "valueFrom": "$(default_output_filename())"
                    },
                    "default": "",
                    "doc": "Output BED file name",
                    "id": "#clip-toolkit-tag2peak.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "prefix": "-p"
                    },
                    "doc": "threshold of p-value to call peak (0.01)",
                    "id": "#clip-toolkit-tag2peak.cwl/p_value_threshold"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "prefix": "--gap"
                    },
                    "doc": "Merge cluster peaks closer than the gap (-1, no merge if < 0)",
                    "id": "#clip-toolkit-tag2peak.cwl/peak_gap"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "prefix": "--prefix"
                    },
                    "doc": "Prefix of peak id (Peak)",
                    "id": "#clip-toolkit-tag2peak.cwl/peak_id_prefix"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "prefix": "-ss"
                    },
                    "doc": "Separate the two strands",
                    "id": "#clip-toolkit-tag2peak.cwl/separate_strands"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "prefix": "--skip-out-of-range-peaks"
                    },
                    "doc": "Skip peaks with PH > maxPH",
                    "id": "#clip-toolkit-tag2peak.cwl/skip_out_of_range_peaks"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "prefix": "--use-expr"
                    },
                    "doc": "Use expression levels given in the score column in the custom gene bed file for normalization",
                    "id": "#clip-toolkit-tag2peak.cwl/use_expression"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "prefix": "--valley-depth"
                    },
                    "doc": "Depth of valley if valley seeking (0.9)",
                    "id": "#clip-toolkit-tag2peak.cwl/valley_depth"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "prefix": "--valley-seeking"
                    },
                    "doc": "Find candidate peaks by valley seeking",
                    "id": "#clip-toolkit-tag2peak.cwl/valley_seeking"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(default_output_filename())"
                    },
                    "doc": "BED file of called peaks",
                    "id": "#clip-toolkit-tag2peak.cwl/peaks_bed"
                },
                {
                    "type": "File",
                    "id": "#clip-toolkit-tag2peak.cwl/stderr_log",
                    "outputBinding": {
                        "glob": "tag2peak_stderr.log"
                    }
                },
                {
                    "type": "File",
                    "id": "#clip-toolkit-tag2peak.cwl/stdout_log",
                    "outputBinding": {
                        "glob": "tag2peak_stdout.log"
                    }
                }
            ],
            "baseCommand": [
                "tag2peak.pl"
            ],
            "stdout": "tag2peak_stdout.log",
            "stderr": "tag2peak_stderr.log",
            "doc": "detecting peaks from CLIP data\nUsage: tag2peak.pl [options] <tag.bed> <peak.bed>\n <tag.bed> : BED file of unique CLIP tags, input\n <peak.bed>: BED file of called peaks, output\nOptions:\n -big                   : big input file\n -ss                    : separate the two strands\n --valley-seeking       : find candidate peaks by valley seeking\n --valley-depth [float] : depth of valley if valley seeking (0.9)\n --out-boundary [string]: output cluster boundaries\n --out-half-PH  [string]: output half peak height boundaries\n --dbkey        [string]: species to retrieve the default gene bed file (mm10|hg19)\n --gene         [string]: custom gene bed file for scan statistics (will override --dbkey)\n --use-expr             : use expression levels given in the score column in the custom gene bed file for normalization\n -p             [float] : threshold of p-value to call peak (0.01)\n --multi-test           : do Bonferroni multiple test correction\n -minPH         [int]   : min peak height (2)\n -maxPH         [int]   : max peak height to calculate p-value(-1, no limit if < 0)\n --skip-out-of-range-peaks: skip peaks with PH > maxPH\n -gap           [int]   : merge cluster peaks closer than the gap (-1, no merge if < 0)\n --prefix       [string]: prefix of peak id (Peak)\n -c             [dir]   : cache dir\n --keep-cache           : keep cache when the job done\n -v                     : verbose\n",
            "id": "#clip-toolkit-tag2peak.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "ResourceRequirement",
                    "coresMin": 8,
                    "coresMax": 16,
                    "ramMin": 32000
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "scidap/clipper:v0.2.0"
                }
            ],
            "baseCommand": [
                "clipper"
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 8,
                        "prefix": "--gene"
                    },
                    "id": "#clipper.cwl/gene"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--bam"
                    },
                    "id": "#clipper.cwl/input_file"
                },
                {
                    "type": "string",
                    "default": "",
                    "inputBinding": {
                        "position": 10,
                        "prefix": "--outfile",
                        "valueFrom": "${\n  if (inputs.outfile == \"\") {\n    return inputs.input_file.nameroot + \".peakClusters.bed\";\n  }\n  else {\n    return inputs.outfile;\n  }\n}\n"
                    },
                    "id": "#clipper.cwl/outfile"
                },
                {
                    "type": "boolean",
                    "default": true,
                    "inputBinding": {
                        "position": 9,
                        "prefix": "--save-pickle"
                    },
                    "id": "#clipper.cwl/savepickle"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 0,
                        "prefix": "--species"
                    },
                    "doc": "species: one of ce10 ce11 dm3 hg19 GRCh38 mm9 mm10",
                    "id": "#clipper.cwl/species"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n  if (inputs.outfile == \"\") {\n    return inputs.input_file.nameroot + \".peakClusters.bed\";\n  }\n  else {\n    return inputs.outfile;\n  }\n}\n"
                    },
                    "id": "#clipper.cwl/output_bed"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n  if (inputs.outfile == \"\") {\n    return inputs.input_file.nameroot + \".peakClusters.bed.pickle\";\n  }\n  else {\n    return inputs.outfile + \".pickle\";\n  }\n}\n"
                    },
                    "id": "#clipper.cwl/output_pickle"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n  if (inputs.outfile == \"\") {\n    return inputs.input_file.nameroot + \".peakClusters.bed.tsv\";\n  }\n  else {\n    return inputs.outfile + \".tsv\";\n  }\n}\n"
                    },
                    "id": "#clipper.cwl/output_tsv"
                }
            ],
            "doc": "CLIPper is a tool to define peaks in your CLIP-seq dataset.\nCLIPper was developed in the Yeo Lab at the University of California, San Diego.\n  Usage: clipper --bam CLIP-seq_reads.srt.bam --species hg19 --outfile CLIP-seq_reads.srt.peaks.bed",
            "id": "#clipper.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/scidap:v0.0.3"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "File",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "inputBinding": {
                        "position": 6
                    },
                    "doc": "Compressed or uncompressed FASTQ file(s)\n",
                    "id": "#extract-fastq.cwl/compressed_file"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "#!/bin/bash\nshopt -s nocaseglob\nset -- \"$0\" \"$@\"\n\nfunction extract {\n  FILE=$1\n  COMBINED=$2\n  T=`file -b \"${FILE}\" | awk '{print $1}'`\n  case \"${T}\" in\n    \"bzip2\"|\"gzip\"|\"Zip\")\n      7z e -so \"${FILE}\" >> \"${COMBINED}\"\n      ;;\n    \"ASCII\")\n      cat \"${FILE}\" >> \"${COMBINED}\" || true\n      ;;\n    *)\n      echo \"Error: file type unknown\"\n      rm -f \"${COMBINED}\"\n      exit 1\n  esac\n} \n\nCOMBINED=\"\"\nfor FILE in \"$@\"; do\n    BASENAME=$(basename \"$FILE\")\n    ROOTNAME=\"${BASENAME%.*}\"\n    EXT_LIST=( \".fastq\" \".fq\" )\n    for ITEM in $EXT_LIST; do\n      if [[ $ROOTNAME == *$ITEM ]]; then\n        ROOTNAME=\"${ROOTNAME%.*}\"\n      fi\n    done\n    if [[ $COMBINED == \"\" ]]; then\n      COMBINED=\"${ROOTNAME}\"\n    else\n      COMBINED=\"${COMBINED}\"_\"${ROOTNAME}\"\n    fi\ndone\nCOMBINED=\"${COMBINED}\".fastq\n\nfor FILE in \"$@\"; do\n    echo \"Extracting:\" $FILE;\n    extract \"${FILE}\" \"${COMBINED}\"\ndone;\n",
                    "inputBinding": {
                        "position": 5
                    },
                    "doc": "Bash script to extract compressed FASTQ file\n",
                    "id": "#extract-fastq.cwl/script"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*"
                    },
                    "id": "#extract-fastq.cwl/fastq_file"
                }
            ],
            "baseCommand": [
                "bash",
                "-c"
            ],
            "doc": "Tool to decompress input FASTQ file(s).\nIf several FASTQ files are provided, they will be concatenated in the order that corresponds to files in input.\nBash script's logic:\n- disable case sensitive glob check\n- check if root name of input file already include '.fastq' or '.fq' extension. If yes, set DEFAULT_EXT to \"\",\n  otherwise use '.fastq'\n- check file type, decompress if needed\n- return 1, if file type is not recognized\nThis script also works of input file doesn't have any extension at all\n",
            "id": "#extract-fastq.cwl",
            "http://schema.org/name": "extract-fastq",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/extract-fastq.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Tool to decompress input FASTQ file"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function() { return inputs.input_file.location.split('/').slice(-1)[0].split('.').slice(0,-1).join('.') + \".fastxstat\" };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/fastx_toolkit:v0.0.14"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 10,
                        "prefix": "-i"
                    },
                    "doc": "FASTA/Q input file. If FASTA file is given, only nucleotides distribution is calculated (there's no quality info)\n",
                    "id": "#fastx-quality-stats.cwl/input_file"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 5,
                        "prefix": "-N"
                    },
                    "doc": "New output format (with more information per nucleotide/cycle).\ncycle (previously called 'column') = cycle number\nmax-count\nFor each nucleotide in the cycle (ALL/A/C/G/T/N):\n    count   = number of bases found in this column.\n    min     = Lowest quality score value found in this column.\n    max     = Highest quality score value found in this column.\n    sum     = Sum of quality score values for this column.\n    mean    = Mean quality score value for this column.\n    Q1\t= 1st quartile quality score.\n    med\t= Median quality score.\n    Q3\t= 3rd quartile quality score.\n    IQR\t= Inter-Quartile range (Q3-Q1).\n    lW\t= 'Left-Whisker' value (for boxplotting).\n    rW\t= 'Right-Whisker' value (for boxplotting).\n",
                    "id": "#fastx-quality-stats.cwl/new_output_format"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 11,
                        "prefix": "-o",
                        "valueFrom": "${\n    if (self == \"\"){\n      return default_output_filename();\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "default": "",
                    "doc": "Output file to store generated statistics. If not provided - return from default_output_filename function\n",
                    "id": "#fastx-quality-stats.cwl/output_filename"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n    if (inputs.output_filename == \"\"){\n      return default_output_filename();\n    } else {\n      return inputs.output_filename;\n    }\n}\n"
                    },
                    "doc": "Generated statistics file",
                    "id": "#fastx-quality-stats.cwl/statistics_file"
                }
            ],
            "baseCommand": [
                "fastx_quality_stats"
            ],
            "doc": "Tool calculates statistics on the base of FASTQ file quality scores.\nIf `output_filename` is not provided call function `default_output_filename` to return default output file name\ngenerated as `input_file` basename + `.fastxstat` extension.\n",
            "id": "#fastx-quality-stats.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#fastx-toolkit-metadata.yaml",
                "http://schema.org/name": "fastx-toolkit",
                "http://schema.org/about": "The FASTX-Toolkit is a collection of command line tools for Short-Reads FASTA/FASTQ files preprocessing. Available Tools:\n  FASTQ-to-FASTA converter (Convert FASTQ files to FASTA files)\n  FASTQ Information (Chart Quality Statistics and Nucleotide Distribution)\n  FASTQ/A Collapser (Collapsing identical sequences in a FASTQ/A file into a single sequence)\n  FASTQ/A Trimmer (Shortening reads in a FASTQ or FASTQ files, removing barcodes or noise)\n  FASTQ/A Renamer (Renames the sequence identifiers in FASTQ/A file)\n  FASTQ/A Clipper (Removing sequencing adapters/linkers)\n  FASTQ/A Reverse-Complement (Producing the Reverse-complement of each sequence in a FASTQ/FASTA file)\n  FASTQ/A Barcode splitter (Splitting a FASTQ/FASTA files containning multiple samples)\n  FASTA Formatter (changes the width of sequences line in a FASTA file)\n  FASTA Nucleotide Changer (Convets FASTA sequences from/to RNA/DNA)\n  FASTQ Quality Filter (Filters sequences based on quality)\n  FASTQ Quality Trimmer (Trims (cuts) sequences based on quality)\n  FASTQ Masker (Masks nucleotides with 'N' (or other character) based on quality)\n",
                "http://schema.org/url": "http://hannonlab.cshl.edu/fastx_toolkit/index.html",
                "http://schema.org/codeRepository": "https://github.com/agordon/fastx_toolkit",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/GPL-3.0"
                ],
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "0.0.14",
                    "http://schema.org/applicationCategory": "command line tool"
                },
                "http://schema.org/programmingLanguage": "C",
                "http://schema.org/discussionUrl": [
                    "https://github.com/agordon/fastx_toolkit/issues"
                ],
                "http://schema.org/creator": [
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "The Hannon Lab",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Assaf Gordon",
                                "http://schema.org/description": "Main author"
                            }
                        ]
                    }
                ]
            },
            "http://schema.org/name": "fastx-quality-stats",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/fastx-quality-stats.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "usage: fastx_quality_stats [-h] [-N] [-i INFILE] [-o OUTFILE]\nPart of FASTX Toolkit 0.0.14 by A. Gordon (assafgordon@gmail.com)\n   [-h] = This helpful help screen.\n   [-i INFILE]  = FASTQ input file. default is STDIN.\n   [-o OUTFILE] = TEXT output file. default is STDOUT.\n   [-N]         = New output format (with more information per nucleotide/cycle).\nThe *OLD* output TEXT file will have the following fields (one row per column):\n\tcolumn\t= column number (1 to 36 for a 36-cycles read solexa file)\n\tcount   = number of bases found in this column.\n\tmin     = Lowest quality score value found in this column.\n\tmax     = Highest quality score value found in this column.\n\tsum     = Sum of quality score values for this column.\n\tmean    = Mean quality score value for this column.\n\tQ1\t= 1st quartile quality score.\n\tmed\t= Median quality score.\n\tQ3\t= 3rd quartile quality score.\n\tIQR\t= Inter-Quartile range (Q3-Q1).\n\tlW\t= 'Left-Whisker' value (for boxplotting).\n\trW\t= 'Right-Whisker' value (for boxplotting).\n\tA_Count\t= Count of 'A' nucleotides found in this column.\n\tC_Count\t= Count of 'C' nucleotides found in this column.\n\tG_Count\t= Count of 'G' nucleotides found in this column.\n\tT_Count\t= Count of 'T' nucleotides found in this column.\n\tN_Count = Count of 'N' nucleotides found in this column.\n\tmax-count = max. number of bases (in all cycles)\nThe *NEW* output format:\n\tcycle (previously called 'column') = cycle number\n\tmax-count\n  For each nucleotide in the cycle (ALL/A/C/G/T/N):\n\t\tcount   = number of bases found in this column.\n\t\tmin     = Lowest quality score value found in this column.\n\t\tmax     = Highest quality score value found in this column.\n\t\tsum     = Sum of quality score values for this column.\n\t\tmean    = Mean quality score value for this column.\n\t\tQ1\t= 1st quartile quality score.\n\t\tmed\t= Median quality score.\n\t\tQ3\t= 3rd quartile quality score.\n\t\tIQR\t= Inter-Quartile range (Q3-Q1).\n\t\tlW\t= 'Left-Whisker' value (for boxplotting).\n\t\trW\t= 'Right-Whisker' value (for boxplotting).\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function(ext) { var root = inputs.input_filename.basename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.input_filename.basename+ext:root+ext; };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/iaintersect:v0.0.2"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 2,
                        "prefix": "--a=",
                        "separate": false
                    },
                    "doc": "Annotation file, tsv",
                    "id": "#iaintersect.cwl/annotation_filename"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 7,
                        "prefix": "--sam_ignorechr=",
                        "separate": false
                    },
                    "doc": "The chromosome to be ignored, string\n",
                    "id": "#iaintersect.cwl/ignore_chr"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--in=",
                        "separate": false
                    },
                    "doc": "Input filename with MACS2 peak calling results, tsv",
                    "id": "#iaintersect.cwl/input_filename"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 4,
                        "prefix": "--log=",
                        "separate": false,
                        "valueFrom": "${\n    if (self == \"\"){\n      return default_output_filename('_iaintersect.log');\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "default": "",
                    "doc": "Log filename\n",
                    "id": "#iaintersect.cwl/log_filename"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 3,
                        "prefix": "--out=",
                        "separate": false,
                        "valueFrom": "${\n    if (self == \"\"){\n      return default_output_filename('_iaintersect.tsv');\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "default": "",
                    "doc": "Base output file name, tsv\n",
                    "id": "#iaintersect.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 5,
                        "prefix": "--promoter=",
                        "separate": false
                    },
                    "doc": "Promoter region around TSS, base pairs\n",
                    "id": "#iaintersect.cwl/promoter_bp"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 6,
                        "prefix": "--upstream=",
                        "separate": false
                    },
                    "doc": "Upstream region before promoter, base pairs\n",
                    "id": "#iaintersect.cwl/upstream_bp"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n  if (inputs.log_filename == \"\"){\n    return default_output_filename('_iaintersect.log');\n  } else {\n    return inputs.log_filename;\n  }\n}\n"
                    },
                    "id": "#iaintersect.cwl/log_file"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n  if (inputs.output_filename == \"\"){\n    return default_output_filename('_iaintersect.tsv');\n  } else {\n    return inputs.output_filename;\n  }\n}\n"
                    },
                    "id": "#iaintersect.cwl/result_file"
                }
            ],
            "baseCommand": [
                "iaintersect"
            ],
            "doc": "Tool assigns each peak obtained from MACS2 to a gene and region (upstream, promoter, exon, intron, intergenic)\n\n`default_output_filename` function returns output filename with sufix set as `ext` argument. Function is called when\neither `output_filename` or `log_filename` inputs are not provided.\n",
            "id": "#iaintersect.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#iaintersect-metadata.yaml",
                "http://schema.org/name": "iaintersect",
                "http://schema.org/about": "Based on BioWardrobe iaintersect (https://github.com/cincinnati-childrens-hospital/biowardrobe/tree/master/src/iaintersect) The software assigns each peak obtained from MACS2 to a gene and region (upstream, promoter, exon, intron, intergenic).\n",
                "http://schema.org/url": "https://github.com/Barski-lab/iaintersect/blob/master/README.md",
                "http://schema.org/codeRepository": "https://github.com/Barski-lab/iaintersect",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/MIT"
                ],
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "0.0.1",
                    "http://schema.org/applicationCategory": "command line tool"
                },
                "http://schema.org/programmingLanguage": "C++",
                "http://schema.org/discussionUrl": [
                    "https://github.com/Barski-lab/iaintersect/issues"
                ],
                "http://schema.org/creator": [
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "Barski Research Lab",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Andrey Kartashov",
                                "http://schema.org/description": "Main author"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Michael Kotliar",
                                "http://schema.org/description": "Update to work with files instead of DB"
                            }
                        ]
                    }
                ]
            },
            "http://schema.org/name": "iaintersect",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/iaintersect.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Usage:\n  iaintersect [options] --in=pathToFile --a=pathtoFile --out=pathToFile\n  --a                        \tTab-separated annotation file\n  --in                       \tInput filename with MACS2 peak calling results, xls\n  --log                      \tLog filename (default is ./logfile_def.log)\n  --out                      \tBase output file name\n  --promoter                 \tPromoter region around TSS in bp\n  --sam_ignorechr            \tThe chromosome to be ignored\n  --upstream                 \tUpstream region before promoter in bp\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var get_output_filename = function() { if (inputs.output_filename) { return inputs.output_filename; } return inputs.unsorted_file.location.split('/').slice(-1)[0]; };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/scidap:v0.0.2"
                }
            ],
            "inputs": [
                {
                    "type": {
                        "type": "array",
                        "items": "string",
                        "inputBinding": {
                            "prefix": "-k"
                        }
                    },
                    "inputBinding": {
                        "position": 1
                    },
                    "doc": "-k, --key=POS1[,POS2]\nstart a key at POS1, end it at POS2 (origin 1)\n",
                    "id": "#linux-sort.cwl/key"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Name for generated output file\n",
                    "id": "#linux-sort.cwl/output_filename"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 4
                    },
                    "id": "#linux-sort.cwl/unsorted_file"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "id": "#linux-sort.cwl/sorted_file",
                    "outputBinding": {
                        "glob": "$(get_output_filename())"
                    }
                }
            ],
            "stdout": "$(get_output_filename())",
            "baseCommand": [
                "sort"
            ],
            "doc": "Tool sorts data from `unsorted_file` by key\n\n`default_output_filename` function returns file name identical to `unsorted_file`, if `output_filename` is not provided.\n",
            "id": "#linux-sort.cwl",
            "http://schema.org/name": "linux-sort",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/linux-sort.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Andrey Kartashov",
                                            "http://schema.org/email": "mailto:Andrey.Kartashov@cchmc.org",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0001-9102-5681"
                                                }
                                            ]
                                        },
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InitialWorkDirRequirement",
                    "listing": "${\n  return  [\n            {\n              \"entry\": inputs.sort_input,\n              \"entryname\": inputs.sort_input.basename,\n              \"writable\": true\n            }\n          ]\n}\n"
                },
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var ext = function() { if (inputs.csi && !inputs.bai){ return '.csi'; } else { return '.bai'; } };",
                        "var default_bam = function() { if (inputs.trigger == true){ return inputs.sort_input.location.split('/').slice(-1)[0].split('.').slice(0,-1).join('.')+\".bam\"; } else { return inputs.sort_input.location.split('/').slice(-1)[0]; } };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/samtools:v1.4"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "Generate BAI-format index for BAM files [default]. If input isn't cram.\n",
                    "id": "#samtools-sort-index.cwl/bai"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "#!/bin/bash\nif [ \"$0\" = \"true\" ]\nthen\n  echo \"Run: samtools index \" ${@:1}\n  samtools index \"${@:1}\"\nelse\n  echo \"Skip samtools index \" ${@:1}\nfi\n",
                    "inputBinding": {
                        "position": 20
                    },
                    "doc": "Bash function to run samtools index with all input parameters or skip it if trigger is false\n",
                    "id": "#samtools-sort-index.cwl/bash_script_index"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "#!/bin/bash\nif [ \"$0\" = \"true\" ]\nthen\n  echo \"Run: samtools sort \" ${@:1}\n  samtools sort \"${@:1}\"\nelse\n  echo \"Skip samtools sort \" ${@:1}\nfi\n",
                    "inputBinding": {
                        "position": 5
                    },
                    "doc": "Bash function to run samtools sort with all input parameters or skip it if trigger is false\n",
                    "id": "#samtools-sort-index.cwl/bash_script_sort"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "Generate CSI-format index for BAM files. If input isn't cram.\n",
                    "id": "#samtools-sort-index.cwl/csi"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 24,
                        "prefix": "-m"
                    },
                    "doc": "Set minimum interval size for CSI indices to 2^INT [14]\n",
                    "id": "#samtools-sort-index.cwl/csi_interval"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 11,
                        "prefix": "-l"
                    },
                    "doc": "SORT: desired compression level for the final output file, ranging from 0 (uncompressed)\nor 1 (fastest but minimal compression) to 9 (best compression but slowest to write),\nsimilarly to gzip(1)'s compression level setting.\nIf -l is not used, the default compression level will apply.\n",
                    "id": "#samtools-sort-index.cwl/sort_compression_level"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 16
                    },
                    "doc": "Input only in.sam|in.bam|in.cram. Optionally could be supplemented with index file in secondaryFiles\n",
                    "id": "#samtools-sort-index.cwl/sort_input"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 12,
                        "prefix": "-o",
                        "valueFrom": "${\n    if (self == \"\" || inputs.trigger == false){\n      return default_bam();\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "default": "",
                    "doc": "Write the final sorted output to FILE. Only out.bam|out.cram.\nIf output file extension is set to SAM, tool will fail on the index step\n",
                    "id": "#samtools-sort-index.cwl/sort_output_filename"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "Set number of sorting and compression threads [1] (Only for sorting)\n",
                    "id": "#samtools-sort-index.cwl/threads"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "default": true,
                    "doc": "If true - run samtools, if false - return sort_input and optional index file in secondaryFiles, previously staged\ninto output directory.\n",
                    "id": "#samtools-sort-index.cwl/trigger"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n    if (inputs.sort_output_filename == \"\" || inputs.trigger == false){\n      return default_bam();\n    } else {\n      return inputs.sort_output_filename;\n    }\n}\n"
                    },
                    "secondaryFiles": "${ if (inputs.trigger == true){ return self.basename + ext(); } else { return inputs.sort_input.secondaryFiles?inputs.sort_input.secondaryFiles:\"null\"; } }",
                    "id": "#samtools-sort-index.cwl/bam_bai_pair"
                }
            ],
            "baseCommand": [
                "bash",
                "-c"
            ],
            "arguments": [
                {
                    "valueFrom": "${ return inputs.trigger ? \"true\" : \"false\" }\n",
                    "position": 6
                },
                {
                    "valueFrom": "bam",
                    "position": 13,
                    "prefix": "-O"
                },
                {
                    "valueFrom": "$(inputs.threads?inputs.threads:1)",
                    "position": 15,
                    "prefix": "-@"
                },
                {
                    "valueFrom": ";",
                    "position": 17,
                    "shellQuote": false
                },
                {
                    "valueFrom": "bash",
                    "position": 18
                },
                {
                    "valueFrom": "-c",
                    "position": 19
                },
                {
                    "valueFrom": "${ return inputs.trigger ? \"true\" : \"false\" }\n",
                    "position": 21
                },
                {
                    "valueFrom": "$(inputs.bai?'-b':inputs.csi?'-c':[])",
                    "position": 23
                },
                {
                    "valueFrom": "$(inputs.threads?inputs.threads:1)",
                    "position": 25,
                    "prefix": "-@"
                },
                {
                    "valueFrom": "${\n    if (inputs.sort_output_filename == \"\" || inputs.trigger == false){\n      return default_bam();\n    } else {\n      return inputs.sort_output_filename;\n    }\n}\n",
                    "position": 26
                },
                {
                    "valueFrom": "${\n    if (inputs.sort_output_filename == \"\" || inputs.trigger == false){\n      return default_bam() + ext();\n    } else {\n      return inputs.sort_output_filename + ext();\n    }\n}\n",
                    "position": 27
                }
            ],
            "doc": "Tool to sort and index input BAM/SAM/CRAM.\nIf input `trigger` is set to `true` or isn't set at all (`true` is used by default), run `samtools sort` and\n`samtools index`, return sorted BAM and BAI/CSI index file.\nIf input `trigger` is set to `false`, return unchanged `sort_input` (BAM/SAM/CRAM) and index (BAI/CSI, if provided in\n`secondaryFiles`) files, previously staged into output directory.\n\nBefore execution `baseCommand`, `sort_input` and `secondaryFiles` (if provided) are staged into directory\nset as docker parameter `--workdir` (tool's output directory), using `InitialWorkDirRequirement`. Setting\n`writable: true` makes cwl-runner to make copies of the `sort_input` and `secondaryFiles` (if provided) and mount\nthem to docker container with `rw` mode as part of `--workdir` (if set to false, the files staged into output\ndirectory will be mounted to docker container separately with `ro` mode). Because both `samtools sort` and\n`samtools index` can overwrite files with the same names (and in case of `samtools sort` even the input file can be\noverwritten), we don't need to rename any of the staged files.\n\nTrigger logic is implemented in two bash scripts set by default as `bash_script_sort` and `bash_script_index` inputs.\nFor both of then, if the first argument $0 (which is `trigger` input) is true, run `samtools sort/index` with the rest\nof the arguments. If $0 is not true, skip `samtools sort/index` and return `sort_input` and `secondaryFiles`\n(if provided) staged into output directory.\n\nInput `trigger` is Boolean, but returns String, because of `valueFrom` field. The `valueFrom` is used, because if `trigger`\nis false, cwl-runner doesn't append this argument at all to the the `baseCommand` - new feature of CWL v1.0.2. Alternatively,\n`prefix` field could be used, but it causes changing in script logic.\n\nIf using `sort_output_filename`, the output file extension should be `*.bam`, because `samtools sort` defines the output\nfile format on the base of the file extension. If `*.sam` is sed as output filename, it cannot be usefully indexed\nby `samtools index`.\n\n`default_bam` function is used to generate output filename for `samtools sort` if input `sort_output_filename` is not\nset or when `trigger` is false and we need to return `sort_input` and `secondaryFiles` (if provided) files staged into\noutput directory. Output filename is generated on the base of `sort_input` basename with `.bam` extension by default.\n\n`ext` function is used to return the index file extension (BAI/CSI) based on `csi` and `bai` inputs according to the\nfollowing logic\n  `csi` &&  `bai`  => BAI\n !`csi` && !`bai ` => BAI\n  `csi` && !`bai ` => CSI\n",
            "id": "#samtools-sort-index.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#samtools-metadata.yaml",
                "http://schema.org/name": "samtools",
                "http://schema.org/about": "A suite of programs for interacting with high-throughput sequencing data. It consists of three separate repositories: Samtools (Reading/writing/editing/indexing/viewing SAM/BAM/CRAM format), BCFtools (Reading/writing BCF2/VCF/gVCF files and calling/filtering/summarising SNP and short indel sequence variants) and HTSlib (A C library for reading/writing high-throughput sequencing data).\n",
                "http://schema.org/url": "http://www.htslib.org/",
                "http://schema.org/codeRepository": "https://github.com/samtools/samtools.git",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/MIT",
                    "https://opensource.org/licenses/BSD-3-Clause"
                ],
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "1.4",
                    "http://schema.org/applicationCategory": "commandline tool"
                },
                "http://schema.org/programmingLanguage": "C, Perl",
                "http://schema.org/publication": [
                    {
                        "class": "http://schema.org/ScholarlyArticle",
                        "id": "#btr509",
                        "http://schema.org/name": "(Li, 2011) A statistical framework for SNP calling, mutation discovery, association mapping and population genetical parameter estimation from sequencing data. Bioinformatics.",
                        "http://schema.org/url": "http://www.ncbi.nlm.nih.gov/pubmed/21903627"
                    },
                    {
                        "class": "http://schema.org/ScholarlyArticle",
                        "id": "#btr076",
                        "http://schema.org/name": "(Li, 2011) Improving SNP discovery by base alignment quality. Bioinformatics.",
                        "http://schema.org/url": "http://www.ncbi.nlm.nih.gov/pubmed/21320865"
                    },
                    {
                        "class": "http://schema.org/ScholarlyArticle",
                        "id": "#btp352",
                        "http://schema.org/name": "(Li et al., 2009) The Sequence Alignment/Map format and SAMtools. Bioinformatics.",
                        "http://schema.org/url": "http://www.ncbi.nlm.nih.gov/pubmed/19505943"
                    }
                ],
                "http://schema.org/discussionUrl": [
                    "https://lists.sourceforge.net/lists/listinfo/samtools-help",
                    "https://lists.sourceforge.net/lists/listinfo/samtools-devel"
                ],
                "http://schema.org/creator": [
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "Sanger Institute",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Heng Li",
                                "http://schema.org/description": "wrote most of the initial source codes of SAMtools and various converters."
                            }
                        ]
                    },
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "Broad Institute",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Bob Handsaker",
                                "http://schema.org/description": "A major contributor to the\nSAM/BAM specification. He designed and implemented the BGZF format, the\nunderlying indexable compression format for the BAM format. BGZF does\nnot support arithmetic between file offsets.\n"
                            }
                        ]
                    },
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "Beijing Genome Institute",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Jue Ruan",
                                "http://schema.org/description": "Designed and implemented the\nRAZF format, an alternative indexable compression format. RAZF is no longer\nused by or provided with SAMtools. Source code remains available in older\nSAMtools 0.1.x releases and from the standalone branch in the repository.\n"
                            }
                        ]
                    },
                    {
                        "class": "http://schema.org/Person",
                        "http://schema.org/name": "Colin Hercus",
                        "http://schema.org/description": "updated novo2sam.pl to support gapped alignment by novoalign."
                    },
                    {
                        "class": "http://schema.org/Person",
                        "http://schema.org/name": "Petr Danecek",
                        "http://schema.org/description": "contributed the header parsing library sam_header.c and sam2vcf.pl script."
                    }
                ]
            },
            "http://schema.org/name": "samtools-sort-index",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/samtools-sort-index.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Usage: samtools sort [options...] [in.bam]\nOptions:\n  -l INT     Set compression level, from 0 (uncompressed) to 9 (best)\n  -m INT     Set maximum memory per thread; suffix K/M/G recognized [768M]\n  -n         Sort by read name\n  -o FILE    Write final output to FILE rather than standard output\n  -T PREFIX  Write temporary files to PREFIX.nnnn.bam\n      --input-fmt-option OPT[=VAL]\n               Specify a single input file format option in the form\n               of OPTION or OPTION=VALUE\n  -O, --output-fmt FORMAT[,OPT[=VAL]]...\n               Specify output format (SAM, BAM, CRAM)\n      --output-fmt-option OPT[=VAL]\n               Specify a single output file format option in the form\n               of OPTION or OPTION=VALUE\n      --reference FILE\n               Reference sequence FASTA FILE [null]\n  -@, --threads INT\n               Number of additional threads to use [0]\n\nUsage: samtools index [-bc] [-m INT] <in.bam> [out.index]\nOptions:\n  -b       Generate BAI-format index for BAM files [default]\n  -c       Generate CSI-format index for BAM files\n  -m INT   Set minimum interval size for CSI indices to 2^INT [14]\n  -@ INT   Sets the number of threads [none]\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_name_prefix = function() { if (Array.isArray(inputs.readFilesIn) && inputs.readFilesIn.length > 0){ return inputs.readFilesIn[0].location.split('/').slice(-1)[0].split('.').slice(0,-1).join('.')+\".\"; } else { return inputs.readFilesIn.location.split('/').slice(-1)[0].split('.').slice(0,-1).join('.')+\".\"; } };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "scidap/star:v2.7.5c"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignEndsProtrude"
                    },
                    "doc": "0    ConcordantPair\nint, string:\nallow protrusion of alignment ends, i.e. start (end) of the +strand mate downstream of the start (end) of the -strand mate\n  1st word: int: maximum number of protrusion bases allowed\n  2nd word: string:\n    ConcordantPair ... report alignments with non-zero protrusion as concordant pairs\n    DiscordantPair ... report alignments with non-zero protrusion as discordant pairs\n",
                    "id": "#star-alignreads.cwl/alignEndsProtrude"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignEndsType"
                    },
                    "doc": "Local\nstring: type of read ends alignment\nLocal           ... standard local alignment with soft-clipping allowed\nEndToEnd        ... force end-to-end read alignment, do not soft-clip\nExtend5pOfRead1 ... fully extend only the 5p of the read1, all other ends: local alignment\nExtend5pOfReads12 ... fully extend only the 5p of the both read1 and read2, all other ends: local alignment\n",
                    "id": "#star-alignreads.cwl/alignEndsType"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignIntronMax"
                    },
                    "doc": "0\nmaximum intron size, if 0, max intron size will be determined by (2^winBinNbits)*winAnchorDistNbins\n",
                    "id": "#star-alignreads.cwl/alignIntronMax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignIntronMin"
                    },
                    "doc": "21\nminimum intron size: genomic gap is considered intron if its length>=alignIntronMin,\notherwise it is considered Deletion\n",
                    "id": "#star-alignreads.cwl/alignIntronMin"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignMatesGapMax"
                    },
                    "doc": "0\nmaximum gap between two mates, if 0, max intron gap will be determined by (2^winBinNbits)*winAnchorDistNbins\n",
                    "id": "#star-alignreads.cwl/alignMatesGapMax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignSJDBoverhangMin"
                    },
                    "doc": "3\nint>0: minimum overhang (i.e. block size) for annotated (sjdb) spliced alignments\n",
                    "id": "#star-alignreads.cwl/alignSJDBoverhangMin"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignSJoverhangMin"
                    },
                    "doc": "5\nint>0: minimum overhang (i.e. block size) for spliced alignments\n",
                    "id": "#star-alignreads.cwl/alignSJoverhangMin"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignSJstitchMismatchNmax"
                    },
                    "doc": "0 -1 0 0\n4*int>=0: maximum number of mismatches for stitching of the splice junctions (-1: no limit).\n(1) non-canonical motifs, (2) GT/AG and CT/AC motif, (3) GC/AG and CT/GC motif, (4) AT/AC and GT/AT motif.\n",
                    "id": "#star-alignreads.cwl/alignSJstitchMismatchNmax"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignSoftClipAtReferenceEnds"
                    },
                    "doc": "Yes\nstring: allow the soft-clipping of the alignments past the end of the chromosomes\nYes ... allow\nNo  ... prohibit, useful for compatibility with Cufflinks\n",
                    "id": "#star-alignreads.cwl/alignSoftClipAtReferenceEnds"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignSplicedMateMapLmin"
                    },
                    "doc": "0\nint>0: minimum mapped length for a read mate that is spliced\n",
                    "id": "#star-alignreads.cwl/alignSplicedMateMapLmin"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignSplicedMateMapLminOverLmate"
                    },
                    "doc": "0.66\nfloat>0: alignSplicedMateMapLmin normalized to mate length\n",
                    "id": "#star-alignreads.cwl/alignSplicedMateMapLminOverLmate"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignTranscriptsPerReadNmax"
                    },
                    "doc": "10000\nint>0: max number of different alignments per read to consider\n",
                    "id": "#star-alignreads.cwl/alignTranscriptsPerReadNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignTranscriptsPerWindowNmax"
                    },
                    "doc": "100\nint>0: max number of transcripts per window\n",
                    "id": "#star-alignreads.cwl/alignTranscriptsPerWindowNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--alignWindowsPerReadNmax"
                    },
                    "doc": "10000\nint>0: max number of windows per read\n",
                    "id": "#star-alignreads.cwl/alignWindowsPerReadNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--bamRemoveDuplicatesMate2basesN"
                    },
                    "doc": "int>0: number of bases from the 5' of mate 2 to use in collapsing (e.g. for\nRAMPAGE)\n",
                    "id": "#star-alignreads.cwl/bamRemoveDuplicatesMate2basesN"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--bamRemoveDuplicatesType"
                    },
                    "doc": "string: mark duplicates in the BAM file, for now only works with (i) sorted BAM feeded with inputBAMfile, and (ii) for paired-end alignments only\n-                       ... no duplicate removal/marking\nUniqueIdentical         ... mark all multimappers, and duplicate unique mappers. The coordinates, FLAG, CIGAR must be identical\nUniqueIdenticalNotMulti  ... mark duplicate unique mappers but not multimappers.\n",
                    "id": "#star-alignreads.cwl/bamRemoveDuplicatesType"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimFilter"
                    },
                    "doc": "banGenomicN\nstring(s): different filters for chimeric alignments\nNone ... no filtering\nbanGenomicN ... Ns are not allowed in the genome sequence around the chimeric junction\n",
                    "id": "#star-alignreads.cwl/chimFilter"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimJunctionOverhangMin"
                    },
                    "doc": "20\nint>=0: minimum overhang for a chimeric junction\n",
                    "id": "#star-alignreads.cwl/chimJunctionOverhangMin"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimMainSegmentMultNmax"
                    },
                    "doc": "int>=1: maximum number of multi-alignments for the main chimeric segment. =1 will prohibit multimapping main segments.\n",
                    "id": "#star-alignreads.cwl/chimMainSegmentMultNmax"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimOutType"
                    },
                    "doc": "string(s): type of chimeric output\n1st word:\nSeparateSAMold  ... output old SAM into separate Chimeric.out.sam file\nWithinBAM       ... output into main aligned BAM files (Aligned.*.bam)\n2nd word:\nWithinBAM HardClip  ... hard-clipping in the CIGAR for supplemental chimeric alignments (defaultif no 2nd word is present)\nWithinBAM SoftClip  ... soft-clipping in the CIGAR for supplemental chimeric alignments\n",
                    "id": "#star-alignreads.cwl/chimOutType"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimScoreDropMax"
                    },
                    "doc": "20\nint>=0: max drop (difference) of chimeric score (the sum of scores of all chimeric\nsegements) from the read length\n",
                    "id": "#star-alignreads.cwl/chimScoreDropMax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimScoreJunctionNonGTAG"
                    },
                    "doc": "-1\nint: penalty for a non-GT/AG chimeric junction\n",
                    "id": "#star-alignreads.cwl/chimScoreJunctionNonGTAG"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimScoreMin"
                    },
                    "doc": "0\nint>=0: minimum total (summed) score of the chimeric segments\n",
                    "id": "#star-alignreads.cwl/chimScoreMin"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimScoreSeparation"
                    },
                    "doc": "int>=0: minimum difference (separation) between the best chimeric score and\nthe next one\n",
                    "id": "#star-alignreads.cwl/chimScoreSeparation"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimSegmentMin"
                    },
                    "doc": "0\nint>=0: minimum length of chimeric segment length, if ==0, no chimeric output\n",
                    "id": "#star-alignreads.cwl/chimSegmentMin"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chimSegmentReadGapMax"
                    },
                    "doc": "0\nint>=0: maximum gap in the read sequence between chimeric segments\n",
                    "id": "#star-alignreads.cwl/chimSegmentReadGapMax"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--clip3pAdapterMMp"
                    },
                    "doc": "double(s): max proportion of mismatches for 3p adpater clipping for each\nmate.  If one value is given, it will be assumed the same for both mates.\n",
                    "id": "#star-alignreads.cwl/clip3pAdapterMMp"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--clip3pAdapterSeq"
                    },
                    "doc": "string(s): adapter sequences to clip from 3p of each mate.  If one value is\ngiven, it will be assumed the same for both mates.\n",
                    "id": "#star-alignreads.cwl/clip3pAdapterSeq"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--clip3pAfterAdapterNbases"
                    },
                    "doc": "int(s): number of bases to clip from 3p of each mate after the adapter\nclipping. If one value is given, it will be assumed the same for both\nmates.\n",
                    "id": "#star-alignreads.cwl/clip3pAfterAdapterNbases"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--clip3pNbases"
                    },
                    "doc": "int(s): number(s) of bases to clip from 3p of each mate. If one value is\ngiven, it will be assumed the same for both mates.\n",
                    "id": "#star-alignreads.cwl/clip3pNbases"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--clip5pNbases"
                    },
                    "doc": "int(s): number(s) of bases to clip from 5p of each mate. If one value is\ngiven, it will be assumed the same for both mates.\n",
                    "id": "#star-alignreads.cwl/clip5pNbases"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--genomeChrBinNbits"
                    },
                    "doc": "int: =log2(chrBin), where chrBin is the size of the bins for genome\nstorage: each chromosome will occupy an integer number of bins\n",
                    "id": "#star-alignreads.cwl/genomeChrBinNbits"
                },
                {
                    "type": "Directory",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--genomeDir"
                    },
                    "doc": "string: path to the directory where genome files are stored\n",
                    "id": "#star-alignreads.cwl/genomeDir"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--genomeLoad"
                    },
                    "doc": "NoSharedMemory\nstring: mode of shared memory usage for the genome files\nLoadAndKeep     ... load genome into shared and keep it in memory after run\nLoadAndRemove   ... load genome into shared but remove it after run\nLoadAndExit     ... load genome into shared memory and exit, keeping the genome in memory for future runs\nRemove          ... do not map anything, just remove loaded genome from memory\nNoSharedMemory  ... do not use shared memory, each job will have its own private copy of the genome\n",
                    "id": "#star-alignreads.cwl/genomeLoad"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--genomeSAindexNbases"
                    },
                    "doc": "int: length (bases) of the SA pre-indexing string. Typically between 10 and\n15. Longer strings will use much more memory, but allow faster searches.\n",
                    "id": "#star-alignreads.cwl/genomeSAindexNbases"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--genomeSAsparseD"
                    },
                    "doc": "int>0: suffux array sparsity, i.e. distance between indices: use bigger\nnumbers to decrease needed RAM at the cost of mapping speed reduction\n",
                    "id": "#star-alignreads.cwl/genomeSAsparseD"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--inputBAMfile"
                    },
                    "doc": "string: path to BAM input file, to be used with --runMode\ninputAlignmentsFromBAM\n",
                    "id": "#star-alignreads.cwl/inputBAMfile"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitBAMsortRAM"
                    },
                    "doc": "int>=0: maximum available RAM for sorting BAM. If =0, it will be set to the\ngenome index size. 0 value can only be used with --genomeLoad\nNoSharedMemory option.\n",
                    "id": "#star-alignreads.cwl/limitBAMsortRAM"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitGenomeGenerateRAM"
                    },
                    "doc": "31000000000\nint>0: maximum available RAM (bytes) for genome generation\n",
                    "id": "#star-alignreads.cwl/limitGenomeGenerateRAM"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitIObufferSize"
                    },
                    "doc": "150000000\nint>0: max available buffers size (bytes) for input/output, per thread\n",
                    "id": "#star-alignreads.cwl/limitIObufferSize"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitOutSAMoneReadBytes"
                    },
                    "doc": "100000\nint>0: max size of the SAM record for one read. Recommended value: >(2*(LengthMate1+LengthMate2+100)*outFilterMultimapNmax\n",
                    "id": "#star-alignreads.cwl/limitOutSAMoneReadBytes"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitOutSJcollapsed"
                    },
                    "doc": "1000000\nint>0: max number of collapsed junctions\n",
                    "id": "#star-alignreads.cwl/limitOutSJcollapsed"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitOutSJoneRead"
                    },
                    "doc": "1000\nint>0: max number of junctions for one read (including all multi-mappers)\n",
                    "id": "#star-alignreads.cwl/limitOutSJoneRead"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--limitSjdbInsertNsj"
                    },
                    "doc": "1000000\nint>=0: maximum number of junction to be inserted to the genome on the fly at\nthe mapping stage, including those from annotations and those detected in the\n1st step of the 2-pass run\n",
                    "id": "#star-alignreads.cwl/limitSjdbInsertNsj"
                },
                {
                    "type": "int",
                    "default": 10,
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outBAMcompression"
                    },
                    "doc": "int: -1 to 10  BAM compression level, -1=default compression (6?), 0=no\ncompression, 10=maximum compression\n",
                    "id": "#star-alignreads.cwl/outBAMcompression"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outBAMsortingThreadN"
                    },
                    "doc": "int: >=0: number of threads for BAM sorting. 0 will default to\nmin(6,--runThreadN).\n",
                    "id": "#star-alignreads.cwl/outBAMsortingThreadN"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFileNamePrefix",
                        "valueFrom": "${\n    if (self == \"\"){\n      return default_output_name_prefix();\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "default": "",
                    "doc": "string: output files name prefix (including full or relative path). Can\nonly be defined on the command line.\n",
                    "id": "#star-alignreads.cwl/outFileNamePrefix"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterIntronMotifs"
                    },
                    "doc": "None\nstring: filter alignment using their motifs\nNone                           ... no filtering\nRemoveNoncanonical             ... filter out alignments that contain non-canonical junctions\nRemoveNoncanonicalUnannotated  ... filter out alignments that contain non-canonical unannotated junctions when using annotated splice junctions database. The annotated non-canonical junctions will be kept.\n",
                    "id": "#star-alignreads.cwl/outFilterIntronMotifs"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMatchNmin"
                    },
                    "doc": "0\nint: alignment will be output only if the number of matched bases is higher\nthan this value\n",
                    "id": "#star-alignreads.cwl/outFilterMatchNmin"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMatchNminOverLread"
                    },
                    "doc": "0.66\nfloat: outFilterMatchNmin normalized to read length (sum of mates'' lengths\nfor paired-end reads)\n",
                    "id": "#star-alignreads.cwl/outFilterMatchNminOverLread"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMismatchNmax"
                    },
                    "doc": "10\nint: alignment will be output only if it has no more mismatches than this value\n",
                    "id": "#star-alignreads.cwl/outFilterMismatchNmax"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMismatchNoverLmax"
                    },
                    "doc": "float: alignment will be output only if its ratio of mismatches to *mapped* length is less than or equal to this value.\n",
                    "id": "#star-alignreads.cwl/outFilterMismatchNoverLmax"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMismatchNoverReadLmax"
                    },
                    "doc": "1\nfloat: alignment will be output only if its ratio of mismatches to *read* length\nis less than this value\n",
                    "id": "#star-alignreads.cwl/outFilterMismatchNoverReadLmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMultimapNmax"
                    },
                    "doc": "int: maximum number of loci the read is allowed to map to. Alignments (all of them) will be output only if\nthe read maps to no more loci than this value.\nOtherwise no alignments will be output, and the read will be counted as \"mapped to too many loci\"\nin the Log.final.out.\n",
                    "id": "#star-alignreads.cwl/outFilterMultimapNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterMultimapScoreRange"
                    },
                    "doc": "1\nint: the score range below the maximum score for multimapping alignments\n",
                    "id": "#star-alignreads.cwl/outFilterMultimapScoreRange"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterScoreMin"
                    },
                    "doc": "0\nint: alignment will be output only if its score is higher than or equal to this value\n",
                    "id": "#star-alignreads.cwl/outFilterScoreMin"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterScoreMinOverLread"
                    },
                    "doc": "0.66\nfloat: outFilterScoreMin normalized to read length (sum of mates'' lengths for\npaired-end reads)\n",
                    "id": "#star-alignreads.cwl/outFilterScoreMinOverLread"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outFilterType"
                    },
                    "doc": "Normal\nstring: type of filtering\nNormal  ... standard filtering using only current alignment\nBySJout ... keep only those reads that contain junctions that passed filtering into SJ.out.tab\n",
                    "id": "#star-alignreads.cwl/outFilterType"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outMultimapperOrder"
                    },
                    "doc": "Old_2.4\nstring: order of multimapping alignments in the output files\nOld_2.4             ... quasi-random order used before 2.5.0\nRandom              ... random order of alignments for each multi-mapper. Read mates (pairs) are always adjacent, all alignment for each read stay together. This option will become default in the future releases.\n",
                    "id": "#star-alignreads.cwl/outMultimapperOrder"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outQSconversionAdd"
                    },
                    "doc": "int: add this number to the quality score (e.g. to convert from Illumina to Sanger, use -31)\n",
                    "id": "#star-alignreads.cwl/outQSconversionAdd"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "enum",
                            "name": "#star-alignreads.cwl/outReadsUnmapped/unmapped",
                            "symbols": [
                                "#star-alignreads.cwl/outReadsUnmapped/unmapped/None",
                                "#star-alignreads.cwl/outReadsUnmapped/unmapped/Fastx"
                            ]
                        }
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outReadsUnmapped"
                    },
                    "doc": "None\nstring: output of unmapped and partially mapped (i.e. mapped only one mate of a paired end read) reads in separate file(s).\nNone    ... no output\nFastx   ... output in separate fasta/fastq files, Unmapped.out.mate1/2\n",
                    "id": "#star-alignreads.cwl/outReadsUnmapped"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMattrIHstart"
                    },
                    "doc": "1\nint>=0:                     start value for the IH attribute. 0 may be required\nby some downstream software, such as Cufflinks or StringTie.\n",
                    "id": "#star-alignreads.cwl/outSAMattrIHstart"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMattrRGline"
                    },
                    "doc": "-\nstring(s): SAM/BAM read group line. The first word contains the read group identifier and must start with \"ID:\", e.g. --outSAMattrRGline ID:xxx CN:yy \"DS:z z z\".\nxxx will be added as RG tag to each output alignment. Any spaces in the tag values have to be double quoted.\nComma separated RG lines correspons to different (comma separated) input files in --readFilesIn. Commas have to be surrounded by spaces, e.g.\n--outSAMattrRGline ID:xxx , ID:zzz \"DS:z z\" , ID:yyy DS:yyyy\n",
                    "id": "#star-alignreads.cwl/outSAMattrRGline"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMattributes"
                    },
                    "doc": "Standard\nstring: a string of desired SAM attributes, in the order desired for the output SAM\nNH HI AS nM NM MD jM jI XS ch ... any combination in any order\nStandard   ... NH HI AS nM\nAll        ... NH HI AS nM NM MD jM jI ch\nNone       ... no attributes\n",
                    "id": "#star-alignreads.cwl/outSAMattributes"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMfilter"
                    },
                    "doc": "None\nstring(s): filter the output into main SAM/BAM files\nKeepOnlyAddedReferences ... only keep the reads for which all alignments are to the extra reference sequences added with --genomeFastaFiles at the mapping stage.\nKeepAllAddedReferences ...  keep all alignments to the extra reference sequences added with --genomeFastaFiles at the mapping stage.\n",
                    "id": "#star-alignreads.cwl/outSAMfilter"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMflagAND"
                    },
                    "doc": "65535\nint: 0 to 65535: sam FLAG will be bitwise AND''d with this value, i.e. FLAG=FLAG\n& outSAMflagOR. This is applied after all flags have been set by STAR, but before\noutSAMflagOR. Can be used to unset specific bits that are not set otherwise.\n",
                    "id": "#star-alignreads.cwl/outSAMflagAND"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMflagOR"
                    },
                    "doc": "int: 0 to 65535: sam FLAG will be bitwise OR'd with this value, i.e.\nFLAG=FLAG | outSAMflagOR. This is applied after all flags have been set by\nSTAR, and after outSAMflagAND. Can be used to set specific bits that are\nnot set otherwise.\n",
                    "id": "#star-alignreads.cwl/outSAMflagOR"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMheaderCommentFile"
                    },
                    "doc": "-\nstring: path to the file with @CO (comment) lines of the SAM header\n",
                    "id": "#star-alignreads.cwl/outSAMheaderCommentFile"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMheaderHD"
                    },
                    "doc": "-\nstrings: @HD (header) line of the SAM header\n",
                    "id": "#star-alignreads.cwl/outSAMheaderHD"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMheaderPG"
                    },
                    "doc": "-\nstrings: extra @PG (software) line of the SAM header (in addition to STAR)\n",
                    "id": "#star-alignreads.cwl/outSAMheaderPG"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMmapqUnique"
                    },
                    "doc": "255\nint: 0 to 255: the MAPQ value for unique mappers\n",
                    "id": "#star-alignreads.cwl/outSAMmapqUnique"
                },
                {
                    "type": "string",
                    "default": "Full",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMmode"
                    },
                    "doc": "string: mode of SAM output\nNone ... no SAM output\nFull ... full SAM output\nNoQS ... full SAM but without quality scores\n",
                    "id": "#star-alignreads.cwl/outSAMmode"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMmultNmax"
                    },
                    "doc": "-1\nint: max number of multiple alignments for a read that will be output to the\nSAM/BAM files. -1 ... all alignments (up to --outFilterMultimapNmax) will be\noutput\n",
                    "id": "#star-alignreads.cwl/outSAMmultNmax"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMorder"
                    },
                    "doc": "Paired\nstring: type of sorting for the SAM output\nPaired: one mate after the other for all paired alignments\nPairedKeepInputOrder: one mate after the other for all paired alignments, the order is kept the same as in the input FASTQ files\n",
                    "id": "#star-alignreads.cwl/outSAMorder"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMprimaryFlag"
                    },
                    "doc": "OneBestScore\nstring: which alignments are considered primary - all others will be marked with 0x100 bit in the FLAG\nOneBestScore ... only one alignment with the best score is primary\nAllBestScore ... all alignments with the best score are primary\n",
                    "id": "#star-alignreads.cwl/outSAMprimaryFlag"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMreadID"
                    },
                    "doc": "Standard\nstring: read ID record type\nStandard ... first word (until space) from the FASTx read ID line, removing /1,/2 from the end\nNumber   ... read number (index) in the FASTx file\n",
                    "id": "#star-alignreads.cwl/outSAMreadID"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMstrandField"
                    },
                    "doc": "None\nstring: Cufflinks-like strand field flag\nNone        ... not used\nintronMotif ... strand derived from the intron motif. Reads with inconsistent and/or non-canonical introns are filtered out.\n",
                    "id": "#star-alignreads.cwl/outSAMstrandField"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "string"
                    },
                    "default": [
                        "BAM",
                        "Unsorted"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMtype"
                    },
                    "doc": "strings: type of SAM/BAM output\n1st word:\nBAM  ... output BAM without sorting\nSAM  ... output SAM without sorting\nNone ... no SAM/BAM output\n2nd, 3rd:\nUnsorted           ... standard unsorted\nSortedByCoordinate ... sorted by coordinate. This option will allocate extra memory for sorting which can be specified by --limitBAMsortRAM.\n",
                    "id": "#star-alignreads.cwl/outSAMtype"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSAMunmapped"
                    },
                    "doc": "string(s): output of unmapped reads in the SAM format\n  1st word:\n    None   ... no output\n    Within ... output unmapped reads within the main SAM file (i.e. Aligned.out.sam)\n  2nd word:\n    KeepPairs ... record unmapped mate for each alignment, and, in case of unsorted output, keep it adjacent to its mapped mate.\n    Only affects multi-mapping reads\n",
                    "id": "#star-alignreads.cwl/outSAMunmapped"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSJfilterCountTotalMin"
                    },
                    "doc": "3   1   1   1 \\n4 integers: minimum total (multi-mapping+unique) read count\nper junction for: (1) non-canonical motifs, (2) GT/AG and CT/AC motif, (3)\nGC/AG and CT/GC motif, (4) AT/AC and GT/AT motif. -1 means no output for that\nmotif\\nJunctions are output if one of outSJfilterCountUniqueMin OR outSJfilterCountTotalMin\nconditions are satisfied\\ndoes not apply to annotated junctions\n",
                    "id": "#star-alignreads.cwl/outSJfilterCountTotalMin"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSJfilterCountUniqueMin"
                    },
                    "doc": "3   1   1   1 \\n4 integers: minimum uniquely mapping read count per junction\nfor: (1) non-canonical motifs, (2) GT/AG and CT/AC motif, (3) GC/AG and CT/GC\nmotif, (4) AT/AC and GT/AT motif. -1 means no output for that motif\\nJunctions\nare output if one of outSJfilterCountUniqueMin OR outSJfilterCountTotalMin\nconditions are satisfied\\ndoes not apply to annotated junctions\n",
                    "id": "#star-alignreads.cwl/outSJfilterCountUniqueMin"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSJfilterDistToOtherSJmin"
                    },
                    "doc": "10  0   5   10\n4 integers>=0: minimum allowed distance to other junctions' donor/acceptor\ndoes not apply to annotated junctions\n",
                    "id": "#star-alignreads.cwl/outSJfilterDistToOtherSJmin"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSJfilterIntronMaxVsReadN"
                    },
                    "doc": "50000 100000 200000\nN integers>=0: maximum gap allowed for junctions supported by 1,2,3,,,N reads\ni.e. by default junctions supported by 1 read can have gaps <=50000b, by 2 reads: <=100000b, by 3 reads: <=200000. by >=4 reads any gap <=alignIntronMax\ndoes not apply to annotated junctions\n",
                    "id": "#star-alignreads.cwl/outSJfilterIntronMaxVsReadN"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSJfilterOverhangMin"
                    },
                    "doc": "30  12  12  12\n4 integers:    minimum overhang length for splice junctions on both sides for: (1) non-canonical motifs, (2) GT/AG and CT/AC motif, (3) GC/AG and CT/GC motif, (4) AT/AC and GT/AT motif. -1 means no output for that motif\ndoes not apply to annotated junctions\n",
                    "id": "#star-alignreads.cwl/outSJfilterOverhangMin"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outSJfilterReads"
                    },
                    "doc": "All\nstring: which reads to consider for collapsed splice junctions output\nAll: all reads, unique- and multi-mappers\nUnique: uniquely mapping reads only\n",
                    "id": "#star-alignreads.cwl/outSJfilterReads"
                },
                {
                    "type": "string",
                    "default": "Log",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outStd"
                    },
                    "doc": "Log\nstring: which output will be directed to stdout (standard out)\nLog                    ... log messages\nSAM                    ... alignments in SAM format (which normally are output to Aligned.out.sam file), normal standard output will go into Log.std.out\nBAM_Unsorted           ... alignments in BAM format, unsorted. Requires --outSAMtype BAM Unsorted\nBAM_SortedByCoordinate ... alignments in BAM format, unsorted. Requires --outSAMtype BAM SortedByCoordinate\nBAM_Quant              ... alignments to transcriptome in BAM format, unsorted. Requires --quantMode TranscriptomeSAM\n",
                    "id": "#star-alignreads.cwl/outStd"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outTmpDir"
                    },
                    "doc": "string: path to a directory that will be used as temporary by STAR. All contents of this directory will be removed!\n- the temp directory will default to outFileNamePrefix_STARtmp\n",
                    "id": "#star-alignreads.cwl/outTmpDir"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outWigNorm"
                    },
                    "doc": "RPM\nstring: type of normalization for the signal\nRPM    ... reads per million of mapped reads\nNone   ... no normalization, \"raw\" counts\n",
                    "id": "#star-alignreads.cwl/outWigNorm"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outWigReferencesPrefix"
                    },
                    "doc": "string: prefix matching reference names to include in the output wiggle\nfile, e.g. \"chr\", default \"-\" - include all references\n",
                    "id": "#star-alignreads.cwl/outWigReferencesPrefix"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outWigStrand"
                    },
                    "doc": "Stranded\nstring: strandedness of wiggle/bedGraph output\nStranded   ...  separate strands, str1 and str2\nUnstranded ...  collapsed strands\n",
                    "id": "#star-alignreads.cwl/outWigStrand"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--outWigType"
                    },
                    "doc": "None\nstring(s): type of signal output, e.g. \"bedGraph\" OR \"bedGraph read1_5p\". Requires sorted BAM: --outSAMtype BAM SortedByCoordinate .\n1st word:\nNone       ... no signal output\nbedGraph   ... bedGraph format\nwiggle     ... wiggle format\n2nd word:\nread1_5p   ... signal from only 5' of the 1st read, useful for CAGE/RAMPAGE etc\nread2      ... signal from only 2nd read\n",
                    "id": "#star-alignreads.cwl/outWigType"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--parametersFiles"
                    },
                    "doc": "string: name of a user-defined parameters file, \"-\": none. Can only be\ndefined on the command line.\n",
                    "id": "#star-alignreads.cwl/parametersFiles"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--quantMode"
                    },
                    "doc": "-\nstring(s): types of quantification requested\n-                ... none\nTranscriptomeSAM ... output SAM/BAM alignments to transcriptome into a separate file\nGeneCounts       ... count reads per gene\n",
                    "id": "#star-alignreads.cwl/quantMode"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--quantTranscriptomeBAMcompression"
                    },
                    "doc": "int: -1 to 10  transcriptome BAM compression level, -1=default compression\n(6?), 0=no compression, 10=maximum compression\n",
                    "id": "#star-alignreads.cwl/quantTranscriptomeBAMcompression"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--quantTranscriptomeBan"
                    },
                    "doc": "IndelSoftclipSingleend\nstring: prohibit various alignment type\nIndelSoftclipSingleend  ... prohibit indels, soft clipping and single-end alignments - compatible with RSEM\nSingleend               ... prohibit single-end alignments\n",
                    "id": "#star-alignreads.cwl/quantTranscriptomeBan"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--readFilesCommand"
                    },
                    "doc": "string(s): command line to execute for each of the input file. This command should generate FASTA or FASTQ text and send it to stdout\nFor example: zcat - to uncompress .gz files, bzcat - to uncompress .bz2 files, etc.\n",
                    "id": "#star-alignreads.cwl/readFilesCommand"
                },
                {
                    "type": [
                        "File",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--readFilesIn"
                    },
                    "doc": "string(s): paths to files that contain input read1 (and, if needed,  read2)\n",
                    "id": "#star-alignreads.cwl/readFilesIn"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--readMapNumber"
                    },
                    "doc": "-1\nint: number of reads to map from the beginning of the file\n-1: map all reads\n",
                    "id": "#star-alignreads.cwl/readMapNumber"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--readMatesLengthsIn"
                    },
                    "doc": "string: Equal/NotEqual - lengths of names,sequences,qualities for both\nmates are the same  / not the same. NotEqual is safe in all situations.\n",
                    "id": "#star-alignreads.cwl/readMatesLengthsIn"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--readNameSeparator"
                    },
                    "doc": "string(s): character(s) separating the part of the read names that will be\ntrimmed in output (read name after space is always trimmed)\n",
                    "id": "#star-alignreads.cwl/readNameSeparator"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--runDirPerm"
                    },
                    "doc": "User_RWX\nstring: permissions for the directories created at the run-time.\nUser_RWX ... user-read/write/execute\nAll_RWX  ... all-read/write/execute (same as chmod 777)\n",
                    "id": "#star-alignreads.cwl/runDirPerm"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--runRNGseed"
                    },
                    "doc": "777\nint: random number generator seed.\n",
                    "id": "#star-alignreads.cwl/runRNGseed"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreDelBase"
                    },
                    "doc": "-2\ndeletion extension penalty per base (in addition to scoreDelOpen)\n",
                    "id": "#star-alignreads.cwl/scoreDelBase"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreDelOpen"
                    },
                    "doc": "-2\ndeletion open penalty\n",
                    "id": "#star-alignreads.cwl/scoreDelOpen"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreGap"
                    },
                    "doc": "0\nint: splice junction penalty (independent on intron motif)\n",
                    "id": "#star-alignreads.cwl/scoreGap"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreGapATAC"
                    },
                    "doc": "-8\nAT/AC  and GT/AT junction penalty  (in addition to scoreGap)\n",
                    "id": "#star-alignreads.cwl/scoreGapATAC"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreGapGCAG"
                    },
                    "doc": "-4\nGC/AG and CT/GC junction penalty (in addition to scoreGap)\n",
                    "id": "#star-alignreads.cwl/scoreGapGCAG"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreGapNoncan"
                    },
                    "doc": "-8\nint: non-canonical junction penalty (in addition to scoreGap)\n",
                    "id": "#star-alignreads.cwl/scoreGapNoncan"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreGenomicLengthLog2scale"
                    },
                    "doc": "-0.25\nextra score logarithmically scaled with genomic length of the alignment: scoreGenomicLengthLog2scale*log2(genomicLength)\n",
                    "id": "#star-alignreads.cwl/scoreGenomicLengthLog2scale"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreInsBase"
                    },
                    "doc": "-2\ninsertion extension penalty per base (in addition to scoreInsOpen)\n",
                    "id": "#star-alignreads.cwl/scoreInsBase"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreInsOpen"
                    },
                    "doc": "-2\ninsertion open penalty\n",
                    "id": "#star-alignreads.cwl/scoreInsOpen"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--scoreStitchSJshift"
                    },
                    "doc": "1\nmaximum score reduction while searching for SJ boundaries inthe stitching step\n",
                    "id": "#star-alignreads.cwl/scoreStitchSJshift"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedMultimapNmax"
                    },
                    "doc": "10000\nint>0: only pieces that map fewer than this value are utilized in the stitching\nprocedure\n",
                    "id": "#star-alignreads.cwl/seedMultimapNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedNoneLociPerWindow"
                    },
                    "doc": "10\nnint>0: max number of one seed loci per window\n",
                    "id": "#star-alignreads.cwl/seedNoneLociPerWindow"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedPerReadNmax"
                    },
                    "doc": "1000\nint>0: max number of seeds per read\n",
                    "id": "#star-alignreads.cwl/seedPerReadNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedPerWindowNmax"
                    },
                    "doc": "50\nint>0: max number of seeds per window\n",
                    "id": "#star-alignreads.cwl/seedPerWindowNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedSearchLmax"
                    },
                    "doc": "0\nint>=0: defines the maximum length of the seeds, if =0 max seed lengthis infinite\n",
                    "id": "#star-alignreads.cwl/seedSearchLmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedSearchStartLmax"
                    },
                    "doc": "50\nint>0: defines the search start point through the read - the read is split into\npieces no longer than this value\n",
                    "id": "#star-alignreads.cwl/seedSearchStartLmax"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedSearchStartLmaxOverLread"
                    },
                    "doc": "1.0\nfloat: seedSearchStartLmax normalized to read length (sum of mates'' lengths\nfor paired-end reads)\n",
                    "id": "#star-alignreads.cwl/seedSearchStartLmaxOverLread"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbFileChrStartEnd"
                    },
                    "doc": "string(s): path to the files with genomic coordinates (chr <tab> start <tab>\nend <tab> strand) for the splice junction introns. Multiple files can be supplied\nwand will be concatenated.\n",
                    "id": "#star-alignreads.cwl/sjdbFileChrStartEnd"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbGTFchrPrefix"
                    },
                    "doc": "string: prefix for chromosome names in a GTF file (e.g. 'chr' for using\nENSMEBL annotations with UCSC geneomes)\n",
                    "id": "#star-alignreads.cwl/sjdbGTFchrPrefix"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbGTFfeatureExon"
                    },
                    "doc": "exon\nstring: feature type in GTF file to be used as exons for building\ntranscripts\n",
                    "id": "#star-alignreads.cwl/sjdbGTFfeatureExon"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbGTFfile"
                    },
                    "doc": "string: path to the GTF file with annotations\n",
                    "id": "#star-alignreads.cwl/sjdbGTFfile"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbGTFtagExonParentGene"
                    },
                    "doc": "gene_id\nstring: tag name to be used as exons'' gene-parents (default \"gene_id\" works\nfor GTF files)\n",
                    "id": "#star-alignreads.cwl/sjdbGTFtagExonParentGene"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbGTFtagExonParentTranscript"
                    },
                    "doc": "transcript_id\nstring: tag name to be used as exons' transcript-parents (default\n\"transcript_id\" works for GTF files)\n",
                    "id": "#star-alignreads.cwl/sjdbGTFtagExonParentTranscript"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbInsertSave"
                    },
                    "doc": "Basic\nstring: which files to save when sjdb junctions are inserted on the fly at the mapping step\nBasic ... only small junction / transcript files\nAll   ... all files including big Genome, SA and SAindex - this will create a complete genome directory\n",
                    "id": "#star-alignreads.cwl/sjdbInsertSave"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbOverhang"
                    },
                    "doc": "100\nint>0: length of the donor/acceptor sequence on each side of the junctions,\nideally = (mate_length - 1)\n",
                    "id": "#star-alignreads.cwl/sjdbOverhang"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sjdbScore"
                    },
                    "doc": "2\nint: extra alignment score for alignmets that cross database junctions\n",
                    "id": "#star-alignreads.cwl/sjdbScore"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sysShell"
                    },
                    "doc": "string: path to the shell binary, preferrably bash, e.g. /bin/bash.\n- ... the default shell is executed, typically /bin/sh. This was reported to fail on some Ubuntu systems - then you need to specify path to bash.\n",
                    "id": "#star-alignreads.cwl/sysShell"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--runThreadN"
                    },
                    "doc": "1\nint: number of threads to run STAR\n",
                    "id": "#star-alignreads.cwl/threads"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--twopass1readsN"
                    },
                    "doc": "int: number of reads to process for the 1st step. Use very large number (or\ndefault -1) to map all reads in the first step.\n",
                    "id": "#star-alignreads.cwl/twopass1readsN"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--twopassMode"
                    },
                    "doc": "None\nstring: 2-pass mapping mode.\nNone        ... 1-pass mapping\nBasic       ... basic 2-pass mapping, with all 1st pass junctions inserted into the genome indices on the fly\n",
                    "id": "#star-alignreads.cwl/twopassMode"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--winAnchorDistNbins"
                    },
                    "doc": "9\nint>0: max number of bins between two anchors that allows aggregation of anchors\ninto one window\n",
                    "id": "#star-alignreads.cwl/winAnchorDistNbins"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--winAnchorMultimapNmax"
                    },
                    "doc": "50\nint>0: max number of loci anchors are allowed to map to\n",
                    "id": "#star-alignreads.cwl/winAnchorMultimapNmax"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--winBinNbits"
                    },
                    "doc": "16\nint>0: =log2(winBin), where winBin is the size of the bin for the windows/clustering,\neach window will occupy an integer number of bins.\n",
                    "id": "#star-alignreads.cwl/winBinNbits"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--winFlankNbins"
                    },
                    "doc": "4\nint>0: log2(winFlank), where win Flank is the size of the left and right flanking\nregions for each window\n",
                    "id": "#star-alignreads.cwl/winFlankNbins"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--winReadCoverageBasesMin"
                    },
                    "doc": "int>0: minimum number of bases covered by the seeds in a window , for STARlong algorithm only.\n",
                    "id": "#star-alignreads.cwl/winReadCoverageBasesMin"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--winReadCoverageRelativeMin"
                    },
                    "doc": "float>=0: minimum relative coverage of the read sequence by the seeds in a window, for STARlong algorithm only.\n",
                    "id": "#star-alignreads.cwl/winReadCoverageRelativeMin"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*Aligned.out*"
                    },
                    "id": "#star-alignreads.cwl/aligned_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "*Log.final.out*"
                    },
                    "id": "#star-alignreads.cwl/log_final"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "*Log.out*"
                    },
                    "id": "#star-alignreads.cwl/log_out"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "*Log.progress.out*"
                    },
                    "id": "#star-alignreads.cwl/log_progress"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "*SJ.out.tab*"
                    },
                    "id": "#star-alignreads.cwl/log_sj"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "*Log.std.out*"
                    },
                    "id": "#star-alignreads.cwl/log_std"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "*Log.final.out*",
                        "outputEval": "${\n  var uniquelyMappedReadsNumberRegex = /Uniquely mapped reads number.*/;\n  return parseInt(self[0].contents.match(uniquelyMappedReadsNumberRegex)[0].split(/\\|\\t/g)[1]);\n}\n"
                    },
                    "id": "#star-alignreads.cwl/uniquely_mapped_reads_number"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "*Unmapped.out.mate1*"
                    },
                    "id": "#star-alignreads.cwl/unmapped_mate_1_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "*Unmapped.out.mate2*"
                    },
                    "id": "#star-alignreads.cwl/unmapped_mate_2_file"
                }
            ],
            "baseCommand": [
                "STAR"
            ],
            "arguments": [
                "--runMode",
                "alignReads"
            ],
            "doc": "Tool runs STAR alignReads.\n\n`default_output_name_prefix` function returns output files prefix if `outFileNamePrefix` is not set. By default prefix\nis equal to basename of `readFilesIn`.\n",
            "id": "#star-alignreads.cwl",
            "http://schema.org/name": "star-alignreads",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/star-alignreads.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#star-metadata.yaml",
                "http://schema.org/name": "STAR",
                "http://schema.org/about": "Aligns RNA-seq reads to a reference genome using uncompressed suffix arrays.\nSTAR has a potential for accurately aligning long (several kilobases) reads that\nare emerging from the third-generation sequencing technologies\n",
                "http://schema.org/url": "https://github.com/alexdobin/STAR",
                "http://schema.org/codeRepository": "https://github.com/alexdobin/STAR",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/GPL-3.0"
                ],
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "v2.5.3a",
                    "http://schema.org/applicationCategory": "command line tool"
                },
                "http://schema.org/programmingLanguage": "C++",
                "http://schema.org/publication": [
                    {
                        "class": "http://schema.org/ScholarlyArticle",
                        "id": "#star-metadata.yaml/ 10.1093/bioinformatics/bts635",
                        "datePublished": "1 January 2013",
                        "http://schema.org/name": "STAR: ultrafast universal RNA-seq aligner",
                        "http://schema.org/author": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Alexander Dobin"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Carrie A. Davis"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Felix Schlesinger"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Jorg Drenkow"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Chris Zaleski"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Sonali Jha"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Philippe Batut"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Mark Chaisson"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Thomas R. Gingeras1"
                            }
                        ],
                        "http://schema.org/url": "https://academic.oup.com/bioinformatics/article/29/1/15/272537"
                    }
                ],
                "http://schema.org/discussionUrl": [
                    "https://github.com/alexdobin/STAR/issues"
                ],
                "http://schema.org/creator": [
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "Cold Spring Harbor Laboratory, Cold Spring Harbor, NY, USA",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Alexander Dobin",
                                "http://schema.org/description": "main author"
                            }
                        ]
                    }
                ]
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Usage: STAR  [options]... --genomeDir REFERENCE   --readFilesIn R1.fq R2.fq\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_log_name = function() { var lognames = {}; lognames[\"pair\"] = (inputs.paired && inputs.input_file_pair) ? inputs.input_file_pair.basename+'_trimming_report.txt':null; lognames[\"single\"] = inputs.input_file.basename+'_trimming_report.txt'; return lognames; }"
                    ]
                },
                {
                    "class": "InitialWorkDirRequirement",
                    "listing": "${\n  var listing = [inputs.input_file]\n  if (inputs.input_file_pair){\n    listing.push(inputs.input_file_pair);\n  }\n  return listing;\n}\n"
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "scidap/trimgalore:v0.6.6"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 10,
                        "prefix": "-a"
                    },
                    "doc": "Adapter sequence to be trimmed. If not specified explicitly, Trim Galore will\ntry to auto-detect whether the Illumina universal, Nextera transposase or Illumina\nsmall RNA adapter sequence was used. Also see '--illumina', '--nextera' and\n'--small_rna'. If no adapter can be detected within the first 1 million sequences\nof the first file specified Trim Galore defaults to '--illumina'.\n",
                    "id": "#trimgalore.cwl/adapter"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 11,
                        "prefix": "-a2"
                    },
                    "doc": "Optional adapter sequence to be trimmed off read 2 of paired-end files. This\noption requires '--paired' to be specified as well. If the libraries to be trimmed\nare smallRNA then a2 will be set to the Illumina small RNA 5' adapter automatically\n(GATCGTCGGACT).\n",
                    "id": "#trimgalore.cwl/adapter_pair"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "#!/bin/bash\nif [ \"$0\" = \"true\" ]\nthen\n  echo \"Run: trimgalore \" ${@:1}\n  trim_galore \"${@:1}\"\nelse\n  echo \"Skip run trimgalore\"\nfi\n",
                    "inputBinding": {
                        "position": 1
                    },
                    "doc": "Bash function to run trimgalore with all input parameters or skip it if trigger is false\n",
                    "id": "#trimgalore.cwl/bash_script"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 25,
                        "prefix": "--clip_R1"
                    },
                    "doc": "Instructs Trim Galore to remove <int> bp from the 5' end of read 1 (or single-end\nreads). This may be useful if the qualities were very poor, or if there is some\nsort of unwanted bias at the 5' end.\nDefault: OFF.\n",
                    "id": "#trimgalore.cwl/clip_R1"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 26,
                        "prefix": "--clip_R2"
                    },
                    "doc": "Instructs Trim Galore to remove <int> bp from the 5' end of read 2 (paired-end reads\nonly). This may be useful if the qualities were very poor, or if there is some sort\nof unwanted bias at the 5' end. For paired-end BS-Seq, it is recommended to remove\nthe first few bp because the end-repair reaction may introduce a bias towards low\nmethylation. Please refer to the M-bias plot section in the Bismark User Guide for\nsome examples.\nDefault: OFF.\n",
                    "id": "#trimgalore.cwl/clip_R2"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 19,
                        "prefix": "--dont_gzip"
                    },
                    "doc": "Output files won't be compressed with GZIP. This option overrides --gzip.\n",
                    "id": "#trimgalore.cwl/dont_gzip"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 17,
                        "prefix": "-e"
                    },
                    "doc": "Maximum allowed error rate (no. of errors divided by the length of the matching region)\nDefault: 0.1\n",
                    "id": "#trimgalore.cwl/error_rate"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 18,
                        "prefix": "--gzip"
                    },
                    "doc": "Compress the output file with GZIP. If the input files are GZIP-compressed\nthe output files will automatically be GZIP compressed as well. As of v0.2.8 the\ncompression will take place on the fly.\n",
                    "id": "#trimgalore.cwl/gzip"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 12,
                        "prefix": "--illumina"
                    },
                    "doc": "Adapter sequence to be trimmed is the first 13bp of the Illumina universal adapter\n'AGATCGGAAGAGC' instead of the default auto-detection of adapter sequence.\n",
                    "id": "#trimgalore.cwl/illumina"
                },
                {
                    "type": [
                        "File"
                    ],
                    "inputBinding": {
                        "position": 100
                    },
                    "doc": "Input FASTQ file\n",
                    "id": "#trimgalore.cwl/input_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 101,
                        "valueFrom": "${\n    if (!inputs.paired){\n      return null;\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "doc": "Input FASTQ file, if paired-end data is analysed\n",
                    "id": "#trimgalore.cwl/input_file_pair"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 20,
                        "prefix": "--length"
                    },
                    "doc": "Discard reads that became shorter than length INT because of either\nquality or adapter trimming. A value of '0' effectively disables\nthis behaviour.\nDefault: 20 bp.\n",
                    "id": "#trimgalore.cwl/length"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 35,
                        "prefix": "-r1"
                    },
                    "doc": "Unpaired single-end read length cutoff needed for read 1 to be written to\n'.unpaired_1.fq' output file. These reads may be mapped in single-end mode.\nDefault: 35 bp.\n",
                    "id": "#trimgalore.cwl/length_1"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 36,
                        "prefix": "-r2"
                    },
                    "doc": "Unpaired single-end read length cutoff needed for read 2 to be written to\n'.unpaired_2.fq' output file. These reads may be mapped in single-end mode.\nDefault: 35 bp.\n",
                    "id": "#trimgalore.cwl/length_2"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 15,
                        "prefix": "--max_length"
                    },
                    "doc": "Discard reads that are longer than <INT> bp after trimming. This is only advised for\nsmallRNA sequencing to remove non-small RNA sequences.\n",
                    "id": "#trimgalore.cwl/max_length"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 21,
                        "prefix": "--max_n"
                    },
                    "doc": "The total number of Ns (as integer) a read may contain before it will be removed altogether.\nIn a paired-end setting, either read exceeding this limit will result in the entire\npair being removed from the trimmed output files.\n",
                    "id": "#trimgalore.cwl/max_n"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 13,
                        "prefix": "--nextera"
                    },
                    "doc": "Adapter sequence to be trimmed is the first 12bp of the Nextera adapter\n'CTGTCTCTTATA' instead of the default auto-detection of adapter sequence.\n",
                    "id": "#trimgalore.cwl/nextera"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 23,
                        "prefix": "--no_report_file"
                    },
                    "doc": "If specified no report file will be generated.\n",
                    "id": "#trimgalore.cwl/no_report_file"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 30,
                        "prefix": "--non_directional"
                    },
                    "doc": "Selecting this option for non-directional RRBS libraries will screen\nquality-trimmed sequences for 'CAA' or 'CGA' at the start of the read\nand, if found, removes the first two basepairs. Like with the option\n'--rrbs' this avoids using cytosine positions that were filled-in\nduring the end-repair step. '--non_directional' requires '--rrbs' to\nbe specified as well. Note that this option does not set '--clip_r2 2' in\npaired-end mode.\n",
                    "id": "#trimgalore.cwl/non_directional"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 32,
                        "prefix": "--paired",
                        "valueFrom": "${\n    if (!inputs.input_file_pair){\n      return null;\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "doc": "This option performs length trimming of quality/adapter/RRBS trimmed reads for\npaired-end files. To pass the validation test, both sequences of a sequence pair\nare required to have a certain minimum length which is governed by the option\n--length (see above). If only one read passes this length threshold the\nother read can be rescued (see option --retain_unpaired). Using this option lets\nyou discard too short read pairs without disturbing the sequence-by-sequence order\nof FastQ files which is required by many aligners.\n",
                    "id": "#trimgalore.cwl/paired"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 6,
                        "prefix": "--phred33"
                    },
                    "doc": "Instructs Cutadapt to use ASCII+33 quality scores as Phred scores\n(Sanger/Illumina 1.9+ encoding) for quality trimming.\nDefault: ON.\n",
                    "id": "#trimgalore.cwl/phred33"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 7,
                        "prefix": "--phred64"
                    },
                    "doc": "Instructs Cutadapt to use ASCII+64 quality scores as Phred scores\n(Illumina 1.5 encoding) for quality trimming.\n",
                    "id": "#trimgalore.cwl/phred64"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 5,
                        "prefix": "-q"
                    },
                    "doc": "Trim low-quality ends from reads in addition to adapter removal.\nDefault Phred score: 20.\n",
                    "id": "#trimgalore.cwl/quality"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 34,
                        "prefix": "--retain_unpaired"
                    },
                    "doc": "If only one of the two paired-end reads became too short, the longer\nread will be written to either '.unpaired_1.fq' or '.unpaired_2.fq'\noutput files. The length cutoff for unpaired single-end reads is\ngoverned by the parameters -r1/--length_1 and -r2/--length_2.\nDefault: OFF.\n",
                    "id": "#trimgalore.cwl/retain_unpaired"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 29,
                        "prefix": "--rrbs"
                    },
                    "doc": "Specifies that the input file was an MspI digested RRBS sample (recognition\nsite: CCGG). Single-end or Read 1 sequences (paired-end) which were adapter-trimmed\nwill have a further 2 bp removed from their 3' end. Sequences which were merely\ntrimmed because of poor quality will not be shortened further. Read 2 of paired-end\nlibraries will in addition have the first 2 bp removed from the 5' end (by setting\n'--clip_r2 2'). This is to avoid using artificial methylation calls from the filled-in\ncytosine positions close to the 3' MspI site in sequenced fragments.\nThis option is not recommended for users of the NuGEN ovation RRBS System 1-16\nkit (see below).\n",
                    "id": "#trimgalore.cwl/rrbs"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 14,
                        "prefix": "--small_rna"
                    },
                    "doc": "Adapter sequence to be trimmed is the first 12bp of the Illumina Small RNA 3' Adapter\n'TGGAATTCTCGG' instead of the default auto-detection of adapter sequence. Selecting\nto trim smallRNA adapters will also lower the --length value to 18bp. If the smallRNA\nlibraries are paired-end then a2 will be set to the Illumina small RNA 5' adapter\nautomatically (GATCGTCGGACT) unless -a 2 had been defined explicitly.\n",
                    "id": "#trimgalore.cwl/small_rna"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 16,
                        "prefix": "--stringency"
                    },
                    "doc": "Overlap with adapter sequence required to trim a sequence. Defaults to a\nvery stringent setting of 1, i.e. even a single bp of overlapping sequence\nwill be trimmed off from the 3' end of any read.\n",
                    "id": "#trimgalore.cwl/stringency"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 24,
                        "prefix": "--suppress_warn"
                    },
                    "doc": "If specified any output to STDOUT or STDERR will be suppressed.\n",
                    "id": "#trimgalore.cwl/suppress_warn"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 27,
                        "prefix": "--three_prime_clip_R1"
                    },
                    "doc": "Instructs Trim Galore to remove <int> bp from the 3' end of read 1 (or single-end\nreads) AFTER adapter/quality trimming has been performed. This may remove some unwanted\nbias from the 3' end that is not directly related to adapter sequence or basecall quality.\nDefault: OFF.\n",
                    "id": "#trimgalore.cwl/three_prime_clip_R1"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 28,
                        "prefix": "--three_prime_clip_R2"
                    },
                    "doc": "Instructs Trim Galore to remove <int> bp from the 3' end of read 2 AFTER\nadapter/quality trimming has been performed. This may remove some unwanted bias from\nthe 3' end that is not directly related to adapter sequence or basecall quality.\nDefault: OFF.\n",
                    "id": "#trimgalore.cwl/three_prime_clip_R2"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "default": true,
                    "inputBinding": {
                        "position": 2,
                        "valueFrom": "$(self?\"true\":\"false\")"
                    },
                    "doc": "If true - run trimgalore, if false - return input fastq files, previously staged into output directory.\nUse valueFrom to return string instead of boolean to make sure that value is printed to command line in both cases\n",
                    "id": "#trimgalore.cwl/trigger"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 33,
                        "prefix": "--trim1"
                    },
                    "doc": "Trims 1 bp off every read from its 3' end. This may be needed for FastQ files that\nare to be aligned as paired-end data with Bowtie.\n",
                    "id": "#trimgalore.cwl/trim1"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 22,
                        "prefix": "--trim-n"
                    },
                    "doc": "Removes Ns from either side of the read. This option does currently not work in RRBS mode.\n",
                    "id": "#trimgalore.cwl/trim_n"
                }
            ],
            "outputs": [
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "$(default_log_name()['single'])"
                    },
                    "id": "#trimgalore.cwl/report_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "$(default_log_name()['pair'])"
                    },
                    "id": "#trimgalore.cwl/report_file_pair"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n    if (inputs.trigger == false){\n      return inputs.input_file.basename;\n    } else if (inputs.paired && inputs.input_file_pair){\n      return \"*_val_1.fq*\";\n    } else {\n      return \"*_trimmed.fq*\";\n    }\n}\n"
                    },
                    "id": "#trimgalore.cwl/trimmed_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "${\n    if (inputs.paired && inputs.input_file_pair && inputs.trigger == false){\n      return inputs.input_file_pair.basename;\n    } else {\n      return \"*_val_2.fq*\";\n    }\n}\n"
                    },
                    "id": "#trimgalore.cwl/trimmed_file_pair"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "*_unpaired_1.fq*"
                    },
                    "id": "#trimgalore.cwl/unpaired_file_1"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "*_unpaired_2.fq*"
                    },
                    "id": "#trimgalore.cwl/unpaired_file_2"
                }
            ],
            "baseCommand": [
                "bash",
                "-c"
            ],
            "doc": "Tool runs Trimgalore - the wrapper around Cutadapt and FastQC to consistently apply adapter and quality trimming\nto FastQ files.\n\n`default_log_name` function returns names for generated log files (for both paired-end and single-end cases).\n`trim_galore` itself doesn't support setting custom names for output files.\n\nFor paired-end data processing both `input_file_pair` and `paired` should be set. If either of them is not set,\nthe other one becomes unset automatically.\n\nIf input trigger was set to false, skip running trimaglore and return unchanged input files\n",
            "id": "#trimgalore.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#trimgalore-metadata.yaml",
                "http://schema.org/name": "TrimGalore",
                "http://schema.org/about": "A wrapper around Cutadapt and FastQC to consistently apply adapter and quality trimming to FastQ files,\nwith extra functionality for RRBS data\n",
                "http://schema.org/url": "https://www.bioinformatics.babraham.ac.uk/projects/trim_galore/",
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "v0.4.4",
                    "http://schema.org/applicationCategory": "commandline tool",
                    "http://schema.org/downloadURL": "https://github.com/FelixKrueger/TrimGalore"
                },
                "http://schema.org/programmingLanguage": "Perl",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/GPL-3.0"
                ]
            },
            "http://schema.org/name": "trimgalore",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/trimgalore.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "trim_galore [options] <filename(s)>\n\nCurrently NOT supported as cwl tool options:\n\n-h/--help               Print this help message and exits.\n\n-v/--version            Print the version information and exits.\n\n--fastqc                Run FastQC in the default mode on the FastQ file once trimming is complete.\n\n--fastqc_args \"<ARGS>\"  Passes extra arguments to FastQC. If more than one argument is to be passed\n                        to FastQC they must be in the form \"arg1 arg2 etc.\". An example would be:\n                        --fastqc_args \"--nogroup --outdir /home/\". Passing extra arguments will\n                        automatically invoke FastQC, so --fastqc does not have to be specified\n                        separately.\n\n--path_to_cutadapt </path/to/cutadapt>     You may use this option to specify a path to the Cutadapt executable,\n                        e.g. /my/home/cutadapt-1.7.1/bin/cutadapt. Else it is assumed that Cutadapt is in\n                        the PATH.\n\n-o/--output_dir <DIR>   If specified all output will be written to this directory instead of the current\n                        directory.\n\n--keep                  Keep the quality trimmed intermediate file. Default: off, which means\n                        the temporary file is being deleted after adapter trimming. Only has\n                        an effect for RRBS samples since other FastQ files are not trimmed\n                        for poor qualities separately.\n\n\nGeneral options:\n\n-q/--quality <INT>      Trim low-quality ends from reads in addition to adapter removal. For\n                        RRBS samples, quality trimming will be performed first, and adapter\n                        trimming is carried in a second round. Other files are quality and adapter\n                        trimmed in a single pass. The algorithm is the same as the one used by BWA\n                        (Subtract INT from all qualities; compute partial sums from all indices\n                        to the end of the sequence; cut sequence at the index at which the sum is\n                        minimal). Default Phred score: 20.\n\n--phred33               Instructs Cutadapt to use ASCII+33 quality scores as Phred scores\n                        (Sanger/Illumina 1.9+ encoding) for quality trimming. Default: ON.\n\n--phred64               Instructs Cutadapt to use ASCII+64 quality scores as Phred scores\n                        (Illumina 1.5 encoding) for quality trimming.\n\n-a/--adapter <STRING>   Adapter sequence to be trimmed. If not specified explicitly, Trim Galore will\n                        try to auto-detect whether the Illumina universal, Nextera transposase or Illumina\n                        small RNA adapter sequence was used. Also see '--illumina', '--nextera' and\n                        '--small_rna'. If no adapter can be detected within the first 1 million sequences\n                        of the first file specified Trim Galore defaults to '--illumina'.\n\n-a2/--adapter2 <STRING> Optional adapter sequence to be trimmed off read 2 of paired-end files. This\n                        option requires '--paired' to be specified as well. If the libraries to be trimmed\n                        are smallRNA then a2 will be set to the Illumina small RNA 5' adapter automatically\n                        (GATCGTCGGACT).\n\n--illumina              Adapter sequence to be trimmed is the first 13bp of the Illumina universal adapter\n                        'AGATCGGAAGAGC' instead of the default auto-detection of adapter sequence.\n\n--nextera               Adapter sequence to be trimmed is the first 12bp of the Nextera adapter\n                        'CTGTCTCTTATA' instead of the default auto-detection of adapter sequence.\n\n--small_rna             Adapter sequence to be trimmed is the first 12bp of the Illumina Small RNA 3' Adapter\n                        'TGGAATTCTCGG' instead of the default auto-detection of adapter sequence. Selecting\n                        to trim smallRNA adapters will also lower the --length value to 18bp. If the smallRNA\n                        libraries are paired-end then a2 will be set to the Illumina small RNA 5' adapter\n                        automatically (GATCGTCGGACT) unless -a 2 had been defined explicitly.\n\n--max_length <INT>      Discard reads that are longer than <INT> bp after trimming. This is only advised for\n                        smallRNA sequencing to remove non-small RNA sequences.\n\n\n--stringency <INT>      Overlap with adapter sequence required to trim a sequence. Defaults to a\n                        very stringent setting of 1, i.e. even a single bp of overlapping sequence\n                        will be trimmed off from the 3' end of any read.\n\n-e <ERROR RATE>         Maximum allowed error rate (no. of errors divided by the length of the matching\n                        region) (default: 0.1)\n\n--gzip                  Compress the output file with GZIP. If the input files are GZIP-compressed\n                        the output files will automatically be GZIP compressed as well. As of v0.2.8 the\n                        compression will take place on the fly.\n\n--dont_gzip             Output files won't be compressed with GZIP. This option overrides --gzip.\n\n--length <INT>          Discard reads that became shorter than length INT because of either\n                        quality or adapter trimming. A value of '0' effectively disables\n                        this behaviour. Default: 20 bp.\n\n                        For paired-end files, both reads of a read-pair need to be longer than\n                        <INT> bp to be printed out to validated paired-end files (see option --paired).\n                        If only one read became too short there is the possibility of keeping such\n                        unpaired single-end reads (see --retain_unpaired). Default pair-cutoff: 20 bp.\n\n--max_n COUNT           The total number of Ns (as integer) a read may contain before it will be removed altogether.\n                        In a paired-end setting, either read exceeding this limit will result in the entire\n                        pair being removed from the trimmed output files.\n\n--trim-n                Removes Ns from either side of the read. This option does currently not work in RRBS mode.\n\n--no_report_file        If specified no report file will be generated.\n\n--suppress_warn         If specified any output to STDOUT or STDERR will be suppressed.\n\n--clip_R1 <int>         Instructs Trim Galore to remove <int> bp from the 5' end of read 1 (or single-end\n                        reads). This may be useful if the qualities were very poor, or if there is some\n                        sort of unwanted bias at the 5' end. Default: OFF.\n\n--clip_R2 <int>         Instructs Trim Galore to remove <int> bp from the 5' end of read 2 (paired-end reads\n                        only). This may be useful if the qualities were very poor, or if there is some sort\n                        of unwanted bias at the 5' end. For paired-end BS-Seq, it is recommended to remove\n                        the first few bp because the end-repair reaction may introduce a bias towards low\n                        methylation. Please refer to the M-bias plot section in the Bismark User Guide for\n                        some examples. Default: OFF.\n\n--three_prime_clip_R1 <int>     Instructs Trim Galore to remove <int> bp from the 3' end of read 1 (or single-end\n                        reads) AFTER adapter/quality trimming has been performed. This may remove some unwanted\n                        bias from the 3' end that is not directly related to adapter sequence or basecall quality.\n                        Default: OFF.\n\n--three_prime_clip_R2 <int>     Instructs Trim Galore to remove <int> bp from the 3' end of read 2 AFTER\n                        adapter/quality trimming has been performed. This may remove some unwanted bias from\n                        the 3' end that is not directly related to adapter sequence or basecall quality.\n                        Default: OFF.\n\n--path_to_cutadapt </path/to/cutadapt>     You may use this option to specify a path to the Cutadapt executable,\n                        e.g. /my/home/cutadapt-1.7.1/bin/cutadapt. Else it is assumed that Cutadapt is in\n                        the PATH.\n\n\nRRBS-specific options (MspI digested material):\n\n--rrbs                  Specifies that the input file was an MspI digested RRBS sample (recognition\n                        site: CCGG). Single-end or Read 1 sequences (paired-end) which were adapter-trimmed\n                        will have a further 2 bp removed from their 3' end. Sequences which were merely\n                        trimmed because of poor quality will not be shortened further. Read 2 of paired-end\n                        libraries will in addition have the first 2 bp removed from the 5' end (by setting\n                        '--clip_r2 2'). This is to avoid using artificial methylation calls from the filled-in\n                        cytosine positions close to the 3' MspI site in sequenced fragments.\n                        This option is not recommended for users of the NuGEN ovation RRBS System 1-16\n                        kit (see below).\n\n--non_directional       Selecting this option for non-directional RRBS libraries will screen\n                        quality-trimmed sequences for 'CAA' or 'CGA' at the start of the read\n                        and, if found, removes the first two basepairs. Like with the option\n                        '--rrbs' this avoids using cytosine positions that were filled-in\n                        during the end-repair step. '--non_directional' requires '--rrbs' to\n                        be specified as well. Note that this option does not set '--clip_r2 2' in\n                        paired-end mode.\n\n\nNote for RRBS using the NuGEN Ovation RRBS System 1-16 kit:\n\nOwing to the fact that the NuGEN Ovation kit attaches a varying number of nucleotides (0-3) after each MspI\nsite Trim Galore should be run WITHOUT the option --rrbs. This trimming is accomplished in a subsequent\ndiversity trimming step afterwards (see their manual).\n\n\nNote for RRBS using MseI:\n\nIf your DNA material was digested with MseI (recognition motif: TTAA) instead of MspI it is NOT necessary\nto specify --rrbs or --non_directional since virtually all reads should start with the sequence\n'TAA', and this holds true for both directional and non-directional libraries. As the end-repair of 'TAA'\nrestricted sites does not involve any cytosines it does not need to be treated especially. Instead, simply\nrun Trim Galore! in the standard (i.e. non-RRBS) mode.\n\n\nPaired-end specific options:\n\n--paired                This option performs length trimming of quality/adapter/RRBS trimmed reads for\n                        paired-end files. To pass the validation test, both sequences of a sequence pair\n                        are required to have a certain minimum length which is governed by the option\n                        --length (see above). If only one read passes this length threshold the\n                        other read can be rescued (see option --retain_unpaired). Using this option lets\n                        you discard too short read pairs without disturbing the sequence-by-sequence order\n                        of FastQ files which is required by many aligners.\n\n                        Trim Galore! expects paired-end files to be supplied in a pairwise fashion, e.g.\n                        file1_1.fq file1_2.fq SRR2_1.fq.gz SRR2_2.fq.gz ... .\n\n-t/--trim1              Trims 1 bp off every read from its 3' end. This may be needed for FastQ files that\n                        are to be aligned as paired-end data with Bowtie. This is because Bowtie (1) regards\n                        alignments like this:\n\n                          R1 --------------------------->     or this:    ----------------------->  R1\n                          R2 <---------------------------                       <-----------------  R2\n\n                        as invalid (whenever a start/end coordinate is contained within the other read).\n                        NOTE: If you are planning to use Bowtie2, BWA etc. you don't need to specify this option.\n\n--retain_unpaired       If only one of the two paired-end reads became too short, the longer\n                        read will be written to either '.unpaired_1.fq' or '.unpaired_2.fq'\n                        output files. The length cutoff for unpaired single-end reads is\n                        governed by the parameters -r1/--length_1 and -r2/--length_2. Default: OFF.\n\n-r1/--length_1 <INT>    Unpaired single-end read length cutoff needed for read 1 to be written to\n                        '.unpaired_1.fq' output file. These reads may be mapped in single-end mode.\n                        Default: 35 bp.\n\n-r2/--length_2 <INT>    Unpaired single-end read length cutoff needed for read 2 to be written to\n                        '.unpaired_2.fq' output file. These reads may be mapped in single-end mode.\n                        Default: 35 bp."
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function() { var basename = inputs.bedgraph_file.location.split('/').slice(-1)[0]; var root = basename.split('.').slice(0,-1).join('.'); var ext = \".bigWig\"; return (root == \"\")?basename+ext:root+ext; };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/ucscuserapps:v358"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 10
                    },
                    "doc": "Four column bedGraph file: <chrom> <start> <end> <value>\n",
                    "id": "#ucsc-bedgraphtobigwig.cwl/bedgraph_file"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "separate": false,
                        "position": 7,
                        "prefix": "-blockSize="
                    },
                    "doc": "Number of items to bundle in r-tree.  Default 256\n",
                    "id": "#ucsc-bedgraphtobigwig.cwl/block_size"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 11
                    },
                    "doc": "Two-column chromosome length file: <chromosome name> <size in bases>\n",
                    "id": "#ucsc-bedgraphtobigwig.cwl/chrom_length_file"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "separate": false,
                        "position": 6,
                        "prefix": "-itemsPerSlot="
                    },
                    "doc": "Number of data points bundled at lowest level. Default 1024\n",
                    "id": "#ucsc-bedgraphtobigwig.cwl/items_per_slot"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 12,
                        "valueFrom": "${\n    if (self == \"\"){\n      return default_output_filename();\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "default": "",
                    "doc": "If set, writes the output bigWig file to output_filename,\notherwise generates filename from default_output_filename()\n",
                    "id": "#ucsc-bedgraphtobigwig.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 5,
                        "prefix": "-unc"
                    },
                    "doc": "Disable compression\n",
                    "id": "#ucsc-bedgraphtobigwig.cwl/unc"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n    if (inputs.output_filename == \"\"){\n      return default_output_filename();\n    } else {\n      return inputs.output_filename;\n    }\n}\n"
                    },
                    "id": "#ucsc-bedgraphtobigwig.cwl/bigwig_file"
                }
            ],
            "baseCommand": [
                "bedGraphToBigWig"
            ],
            "doc": "Tool converts bedGraph to bigWig file.\n\n`default_output_filename` function returns filename for generated bigWig if `output_filename` is not provided.\nDefault filename is generated on the base of `bedgraph_file` basename with the updated to `*.bigWig` extension.\n",
            "id": "#ucsc-bedgraphtobigwig.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#ucsc-metadata.yaml",
                "http://schema.org/name": "UCSC userApps",
                "http://schema.org/about": "UCSC genome browser bioinformatic utilities\n",
                "http://schema.org/url": "https://genome.ucsc.edu/util.html",
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "v358",
                    "http://schema.org/applicationCategory": "commandline tool",
                    "http://schema.org/downloadURL": "http://hgdownload.cse.ucsc.edu/admin/exe/userApps.v358.src.tgz"
                },
                "http://schema.org/programmingLanguage": "C++",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/GPL-3.0"
                ]
            },
            "http://schema.org/name": "ucsc-bedgraphtobigwig",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/ucsc-bedgraphtobigwig.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Andrey Kartashov",
                                            "http://schema.org/email": "mailto:Andrey.Kartashov@cchmc.org",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0001-9102-5681"
                                                }
                                            ]
                                        },
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "usage:\n   bedGraphToBigWig in.bedGraph chrom.sizes out.bw\nwhere in.bedGraph is a four column file in the format:\n      <chrom> <start> <end> <value>\nand chrom.sizes is a two-column file/URL: <chromosome name> <size in bases>\nand out.bw is the output indexed big wig file.\nIf the assembly <db> is hosted by UCSC, chrom.sizes can be a URL like\n  http://hgdownload.cse.ucsc.edu/goldenPath/<db>/bigZips/<db>.chrom.sizes\nor you may use the script fetchChromSizes to download the chrom.sizes file.\nIf not hosted by UCSC, a chrom.sizes file can be generated by running\ntwoBitInfo on the assembly .2bit file.\nThe input bedGraph file must be sorted, use the unix sort command:\n  sort -k1,1 -k2,2n unsorted.bedGraph > sorted.bedGraph\noptions:\n   -blockSize=N - Number of items to bundle in r-tree.  Default 256\n   -itemsPerSlot=N - Number of data points bundled at lowest level. Default 1024\n   -unc - If set, do not use compression.\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function() { var ext = inputs.bam_file.basename.split('.').slice(-1)[0]; var root = inputs.bam_file.basename.split('.').slice(0,-1).join('.'); return inputs.output_filename?inputs.output_filename:root+\"_dedup.\"+ext; };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "quay.io/biocontainers/umi_tools:1.0.1--py38h0213d0e_2"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "secondaryFiles": [
                        ".bai"
                    ],
                    "inputBinding": {
                        "prefix": "-I"
                    },
                    "doc": "Input BAM file",
                    "id": "#umi-tools-dedup.cwl/bam_file"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "prefix": "--multimapping-detection-method=",
                        "separate": false,
                        "position": 11
                    },
                    "default": "",
                    "doc": "Some aligners identify multimapping using bam tags.\nSetting this option to NH, X0 or XT will use these\ntags when selecting the best read amongst reads with\nthe same position and umi [default=none]\n",
                    "id": "#umi-tools-dedup.cwl/multimapping_detection_method"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "prefix": "-S",
                        "position": 8,
                        "valueFrom": "$(default_output_filename())"
                    },
                    "default": "",
                    "doc": "Output filename",
                    "id": "#umi-tools-dedup.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "prefix": "--output-stats=",
                        "separate": false,
                        "position": 10,
                        "valueFrom": "$(self + \"_umi_tools_stats.log\")"
                    },
                    "default": "",
                    "doc": "Specify location to output stats",
                    "id": "#umi-tools-dedup.cwl/output_stats"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "prefix": "--paired"
                    },
                    "doc": "Inputs BAM file is paired end - output both read pairs.\nThis will also force the use of the template length to\ndetermine reads with the same mapping coordinates.\n",
                    "id": "#umi-tools-dedup.cwl/paired_end"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(default_output_filename())"
                    },
                    "id": "#umi-tools-dedup.cwl/dedup_bam_file"
                },
                {
                    "$import": "#umi-tools-dedup.cwl/output_stats"
                },
                {
                    "type": "File",
                    "id": "#umi-tools-dedup.cwl/stderr_log",
                    "outputBinding": {
                        "glob": "umi_tools_dedup_stderr.log"
                    }
                },
                {
                    "type": "File",
                    "id": "#umi-tools-dedup.cwl/stdout_log",
                    "outputBinding": {
                        "glob": "umi_tools_dedup_stdout.log"
                    }
                }
            ],
            "baseCommand": [
                "umi_tools",
                "dedup"
            ],
            "stdout": "umi_tools_dedup_stdout.log",
            "stderr": "umi_tools_dedup_stderr.log",
            "doc": "Deduplicate BAM files based on the first mapping co-ordinate and the UMI attached to the read\nOnly -I, --paired and -S parameters are implemented.\n",
            "id": "#umi-tools-dedup.cwl",
            "http://schema.org/name": "umi-tools-dedup",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "UMI-Tools: Version 1.0.1\n\ndedup - Deduplicate reads using UMI and mapping coordinates\n\nUsage: umi_tools dedup [OPTIONS] [--stdin=IN_BAM] [--stdout=OUT_BAM]\n\n      note: If --stdout is ommited, standard out is output. To\n            generate a valid BAM file on standard out, please\n            redirect log with --log=LOGFILE or --log2stderr\n\nFor full UMI-tools documentation, see https://umi-tools.readthedocs.io/en/latest/\n\nOptions:\n  --version             show program's version number and exit\n\n  dedup-specific options:\n    --output-stats=STATS\n                        Specify location to output stats\n\n  Barcode extraction options:\n    --extract-umi-method=GET_UMI_METHOD\n                        how is the read UMI +/ cell barcode encoded?\n                        [default=read_id]\n    --umi-separator=UMI_SEP\n                        separator between read id and UMI\n    --umi-tag=UMI_TAG   tag containing umi\n    --umi-tag-split=UMI_TAG_SPLIT\n                        split UMI in tag and take the first element\n    --umi-tag-delimiter=UMI_TAG_DELIM\n                        concatenate UMI in tag separated by delimiter\n    --cell-tag=CELL_TAG\n                        tag containing cell barcode\n    --cell-tag-split=CELL_TAG_SPLIT\n                        split cell barcode in tag and take the first\n                        elementfor e.g 10X GEM tags\n    --cell-tag-delimiter=CELL_TAG_DELIM\n                        concatenate cell barcode in tag separated by delimiter\n\n  UMI grouping options:\n    --method=METHOD     method to use for umi grouping [default=directional]\n    --edit-distance-threshold=THRESHOLD\n                        Edit distance theshold at which to join two UMIs when\n                        grouping UMIs. [default=1]\n    --spliced-is-unique\n                        Treat a spliced read as different to an unspliced one\n                        [default=False]\n    --soft-clip-threshold=SOFT_CLIP_THRESHOLD\n                        number of bases clipped from 5' end before read is\n                        counted as spliced [default=4]\n    --read-length       use read length in addition to position and UMI to\n                        identify possible duplicates [default=False]\n\n  single-cell RNA-Seq options:\n    --per-gene          Group/Dedup/Count per gene. Must combine with either\n                        --gene-tag or --per-contig\n    --gene-tag=GENE_TAG\n                        Gene is defined by this bam tag [default=none]\n    --assigned-status-tag=ASSIGNED_TAG\n                        Bam tag describing whether read is assigned to a gene\n                        By defualt, this is set as the same tag as --gene-tag\n    --skip-tags-regex=SKIP_REGEX\n                        Used with --gene-tag. Ignore reads where the gene-tag\n                        matches this regex\n    --per-contig        group/dedup/count UMIs per contig (field 3 in BAM;\n                        RNAME), e.g for transcriptome where contig = gene\n    --gene-transcript-map=GENE_TRANSCRIPT_MAP\n                        File mapping transcripts to genes (tab separated)\n    --per-cell          group/dedup/count per cell\n\n  group/dedup options:\n    --buffer-whole-contig\n                        Read whole contig before outputting bundles:\n                        guarantees that no reads are missed, but increases\n                        memory usage\n    --multimapping-detection-method=DETECTION_METHOD\n                        Some aligners identify multimapping using bam tags.\n                        Setting this option to NH, X0 or XT will use these\n                        tags when selecting the best read amongst reads with\n                        the same position and umi [default=none]\n\n  SAM/BAM options:\n    --mapping-quality=MAPPING_QUALITY\n                        Minimum mapping quality for a read to be retained\n                        [default=0]\n    --unmapped-reads=UNMAPPED_READS\n                        How to handle unmapped reads. Options are 'discard',\n                        'use' or 'correct' [default=discard]\n    --chimeric-pairs=CHIMERIC_PAIRS\n                        How to handle chimeric read pairs. Options are\n                        'discard', 'use' or 'correct' [default=use]\n    --unpaired-reads=UNPAIRED_READS\n                        How to handle unpaired reads. Options are 'discard',\n                        'use' or 'correct' [default=use]\n    --ignore-umi        Ignore UMI and dedup only on position\n    --chrom=CHROM       Restrict to one chromosome\n    --subset=SUBSET     Use only a fraction of reads, specified by subset\n    -i, --in-sam        Input file is in sam format [default=False]\n    --paired            paired input BAM. [default=False]\n    -o, --out-sam       Output alignments in sam format [default=False]\n    --no-sort-output    Don't Sort the output\n\n  input/output options:\n    -I FILE, --stdin=FILE\n                        file to read stdin from [default = stdin].\n    -L FILE, --log=FILE\n                        file with logging information [default = stdout].\n    -E FILE, --error=FILE\n                        file with error information [default = stderr].\n    -S FILE, --stdout=FILE\n                        file where output is to go [default = stdout].\n    --temp-dir=FILE     Directory for temporary files. If not set, the bash\n                        environmental variable TMPDIR is used[default = None].\n    --log2stderr        send logging information to stderr [default = False].\n    --compresslevel=COMPRESSLEVEL\n                        Level of Gzip compression to use. Default (6)\n                        matchesGNU gzip rather than python gzip default (which\n                        is 9)\n\n  profiling options:\n    --timeit=TIMEIT_FILE\n                        store timeing information in file [none].\n    --timeit-name=TIMEIT_NAME\n                        name in timing file for this class of jobs [all].\n    --timeit-header     add header for timing information [none].\n\n  common options:\n    -v LOGLEVEL, --verbose=LOGLEVEL\n                        loglevel [1]. The higher, the more output.\n    -h, --help          output short help (command line options only).\n    --help-extended     Output full documentation\n    --random-seed=RANDOM_SEED\n                        random seed to initialize number generator with\n                        [none].\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "quay.io/biocontainers/umi_tools:0.5.5--py36h470a237_0"
                }
            ],
            "inputs": [
                {
                    "type": "string",
                    "inputBinding": {
                        "prefix": "--bc-pattern=",
                        "separate": false
                    },
                    "doc": "Barcode pattern 1",
                    "id": "#umi-tools-extract.cwl/bc_pattern_1"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "prefix": "--bc-pattern2=",
                        "separate": false
                    },
                    "doc": "Barcode pattern 2",
                    "id": "#umi-tools-extract.cwl/bc_pattern_2"
                },
                {
                    "type": {
                        "type": "enum",
                        "symbols": [
                            "#umi-tools-extract.cwl/extract_method/string",
                            "#umi-tools-extract.cwl/extract_method/regex"
                        ]
                    },
                    "inputBinding": {
                        "prefix": "--extract-method=",
                        "separate": false
                    },
                    "default": "regex",
                    "doc": "How to extract the umi +/- cell barcodes, choose from string or regex",
                    "id": "#umi-tools-extract.cwl/extract_method"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "prefix": "-I"
                    },
                    "doc": "Input FASTQ file 1",
                    "id": "#umi-tools-extract.cwl/fastq_file_1"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "prefix": "--read2-in=",
                        "separate": false
                    },
                    "doc": "Input FASTQ file 2",
                    "id": "#umi-tools-extract.cwl/fastq_file_2"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "prefix": "-S"
                    },
                    "doc": "Output filename 1",
                    "id": "#umi-tools-extract.cwl/output_filename_1"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "prefix": "--read2-out=",
                        "separate": false
                    },
                    "doc": "Output filename 2",
                    "id": "#umi-tools-extract.cwl/output_filename_2"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "id": "#umi-tools-extract.cwl/stderr_log",
                    "outputBinding": {
                        "glob": "umi_tools_extract_stderr.log"
                    }
                },
                {
                    "type": "File",
                    "id": "#umi-tools-extract.cwl/stdout_log",
                    "outputBinding": {
                        "glob": "umi_tools_extract_stdout.log"
                    }
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_filename_1)"
                    },
                    "id": "#umi-tools-extract.cwl/umi_fastq_file_1"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "$(inputs.output_filename_2)"
                    },
                    "id": "#umi-tools-extract.cwl/umi_fastq_file_2"
                }
            ],
            "baseCommand": [
                "umi_tools",
                "extract"
            ],
            "stdout": "umi_tools_extract_stdout.log",
            "stderr": "umi_tools_extract_stderr.log",
            "doc": "Extract UMI barcode from a read and add it to the read name, leaving\nany sample barcode in place. Can deal with paired end reads and UMIs\nsplit across the paired ends. Can also optionally extract cell\nbarcodes and append these to the read name also. See the section below\nfor an explanation for how to encode the barcode pattern(s) to\nspecficy the position of the UMI +/- cell barcode.\n",
            "id": "#umi-tools-extract.cwl",
            "http://schema.org/name": "umi-tools-extract",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Usage:\nextract.py - Extract UMI from fastq\n====================================================\n\n:Author: Ian Sudbery, Tom Smith\n:Release: $Id$\n:Date: |today|\n:Tags: Python UMI\n\nPurpose\n-------\n\nExtract UMI barcode from a read and add it to the read name, leaving\nany sample barcode in place. Can deal with paired end reads and UMIs\nsplit across the paired ends. Can also optionally extract cell\nbarcodes and append these to the read name also. See the section below\nfor an explanation for how to encode the barcode pattern(s) to\nspecficy the position of the UMI +/- cell barcode.\n\n\nFiltering and correcting cell barcodes\n--------------------------------------\n\numi_tools extract can optionally filter cell barcodes\n(--filter-cell-barcode) against a user-supplied whitelist\n(--whitelist). If a whitelist is not available for your data, e.g\nif you have performed droplet-based scRNA-Seq, you can use the\nwhitelist tool.\n\nCell barcodes which do not match the whitelist (user-generated or\nautomatically generated) can also be optionally corrected using the\n--error-correct-cell option.\n\nThe whitelist should be in the following format (tab-separated):\n\n    AAAAAA      AGAAAA\n    AAAATC\n    AAACAT\n    AAACTA      AAACTN,GAACTA\n    AAATAC\n    AAATCA      GAATCA\n    AAATGT      AAAGGT,CAATGT\n\nWhere column 1 is the whitelisted cell barcodes and column 2 is\nthe list (comma-separated) of other cell barcodes which should be\ncorrected to the barcode in column 1. If the --error-correct-cell\noption is not used, this column will be ignored. Any additional columns\nin the whitelist input, such as the counts columns from the output of\numi_tools whitelist, will be ignored.\n\n\nUsage:\n------\n\nFor single ended reads:\n        umi_tools extract --extract-method=string\n        --bc-pattern=[PATTERN] -L extract.log [OPTIONS]\n\nreads from stdin and outputs to stdout.\n\nFor paired end reads:\n        umi_tools extract --extract-method=string\n        --bc-pattern=[PATTERN] --bc-pattern2=[PATTERN]\n        --read2-in=[FASTQIN] --read2-out=[FASTQOUT] -L extract.log [OPTIONS]\n\nreads end one from stdin and end two from FASTQIN and outputs end one to stdin\nand end two to FASTQOUT.\n\n\nUsing regex and filtering against a whitelist of cell barcodes:\n        umi_tools extract --extract-method=regex --filter-cell-barcode\n        --bc-pattern=[REGEX] --whitlist=[WHITELIST_TSV]\n        -L extract.log [OPTIONS]\n\n\nBarcode extraction\n------------------\n\nThere are two methods enabled to extract the umi barcode (+/- cell\nbarcode). For both methods, the patterns should be provided using the\n--bc-pattern and --bc-pattern2 options. The method is specified using\nthe --extract-method option\n\n-'string':\n      This should be used where the barcodes are always in the same\n      place in the read.\n\n      - N = UMI position (required)\n      - C = cell barcode position (optional)\n      - X = sample position (optional)\n\n      Bases with Ns and Cs will be extracted and added to the read\n      name. The corresponding sequence qualities will be removed from\n      the read. Bases with an X will be reattached to the read.\n\n      E.g. If the pattern is NNNNCC,\n      Then the read:\n      @HISEQ:87:00000000 read1\n      AAGGTTGCTGATTGGATGGGCTAG\n      DA1AEBFGGCG01DFH00B1FF0B\n      +\n      will become:\n      @HISEQ:87:00000000_TT_AAGG read1\n      GCTGATTGGATGGGCTAG\n      1AFGGCG01DFH00B1FF0B\n      +\n\n      where 'TT' is the cell barcode and 'AAGG' is the UMI.\n\n-'regex'\n      This method allows for more flexible barcode extraction and\n      should be used where the cell barcodes are variable in\n      length. Alternatively, the regex option can also be used to\n      filter out reads which do not contain an expected adapter\n      sequence.\n\n      The expected groups in the regex are:\n\n      umi_n = UMI positions, where n can be any value (required)\n      cell_n = cell barcode positions, where n can be any value (optional)\n      discard_n = positions to discard, where n can be any value (optional)\n\n      UMI positions and cell barcode positions will be extracted and\n      added to the read name. The corresponding sequence qualities\n      will be removed from the read. Discard bases and the\n      corresponding quality scores will be removed from the read. All\n      bases matched by other groups or components of the regex will be\n      reattached to the read sequence\n\n      For example, the following regex can be used to extract reads\n      from the Klein et al inDrop data:\n\n      (?P<cell_1>.{8,12})(?P<discard_1>GAGTGATTGCTTGTGACGCCTT)(?P<cell_2>.{8}\n)(?P<umi_1>.{6})T{3}.*\n\n      Where only reads with a 3' T-tail and GAGTGATTGCTTGTGACGCCTT in\n      the correct position to yield two cell barcodes of 8-12 and 8bp\n      respectively, and a 6bp UMI will be retained.\n\n      You can also specify fuzzy matching to allow errors. For example if\n      the discard group above was specified as below this would enable\n      matches with up to 2 errors in the discard_1 group.\n\n      (?P<discard_1>GAGTGATTGCTTGTGACGCCTT){s<=2}\n\n      Note that all UMIs must be the same length for downstream\n      processing with dedup, group or count commands\n\n\nadditional whitelist/extract options\n-------------------------\n\n--3prime\n      By default the barcode is assumed to be on the 5' end of the\n      read, but use this option to sepecify that it is on the 3' end\n      instead. This option only works with --extract-method=string\n      since 3' encoding can be specified explicitly with a regex, e.g\n      \".*(?P<umi_1>.{5})$\"\n\n\n\nOptions:\n  --version             show program's version number and exit\n  -h, --help            show this help message and exit\n  -p PATTERN, --bc-pattern=PATTERN\n                        Barcode pattern\n  --bc-pattern2=PATTERN2\n                        Barcode pattern for paired reads\n  --3prime              barcode is on 3' end of read.\n  --read2-in=READ2_IN   file name for read pairs\n  --read2-out=READ2_OUT\n                        file to output processed paired read to\n  --read2-stdout        Paired reads, send read2 to stdout, discarding read1\n  --quality-filter-threshold=QUALITY_FILTER_THRESHOLD\n                        Remove reads where any UMI base quality score falls\n                        below this threshold\n  --quality-filter-mask=QUALITY_FILTER_MASK\n                        If a UMI base has a quality below this threshold,\n                        replace the base with 'N'\n  --quality-encoding=QUALITY_ENCODING\n                        Quality score encoding. Choose from 'phred33'[33-77]\n                        'phred64' [64-106] or 'solexa' [59-106]\n  --extract-method=EXTRACT_METHOD\n                        How to extract the umi +/- cell barcodes, Choose from\n                        'string' or 'regex'\n  --filter-cell-barcode\n                        Filter the cell barcodes\n  --error-correct-cell  Correct errors in the cell barcode\n  --whitelist=WHITELIST\n                        A whitelist of accepted cell barcodes\n  --blacklist=BLACKLIST\n                        A blacklist of accepted cell barcodes\n  --reads-subset=READS_SUBSET\n                        Only extract from the first N reads. If N is greater\n                        than the number of reads, all reads will be used\n  --reconcile-pairs     Allow the presences of reads in read2 input that\n                        arenot present in read1 input. This allows cell\n                        barcodefiltering of read1s without considering read2s\n\n  profiling options:\n    --timeit=TIMEIT_FILE\n                        store timeing information in file [none].\n    --timeit-name=TIMEIT_NAME\n                        name in timing file for this class of jobs [all].\n    --timeit-header     add header for timing information [none].\n\n  common options:\n    -v LOGLEVEL, --verbose=LOGLEVEL\n                        loglevel [1]. The higher, the more output.\n    -?                  output short help (command line options only.\n    --random-seed=RANDOM_SEED\n                        random seed to initialize number generator with\n                        [none].\n\n  Input/output options:\n    -I FILE, --stdin=FILE\n                        file to read stdin from [default = stdin].\n    -L FILE, --log=FILE\n                        file with logging information [default = stdout].\n    -E FILE, --error=FILE\n                        file with error information [default = stderr].\n    -S FILE, --stdout=FILE\n                        file where output is to go [default = stdout].\n    --log2stderr        send logging information to stderr [default = False].\n    --compresslevel=COMPRESSLEVEL\n                        Level of Gzip compression to use. Default (6)\n                        matchesGNU gzip rather than python gzip default (which\n                        is 9)"
        },
        {
            "class": "Workflow",
            "requirements": [
                {
                    "class": "SubworkflowFeatureRequirement"
                },
                {
                    "class": "ScatterFeatureRequirement"
                },
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "MultipleInputFeatureRequirement"
                }
            ],
            "sd:metadata": [
                "../metadata/rnaseq-header.cwl"
            ],
            "sd:upstream": {
                "genome_indices": "genome-indices.cwl"
            },
            "inputs": [
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "GTGTCAGTCACTTCCAGCGGG",
                    "sd:layout": {
                        "advanced": true
                    },
                    "label": "Adapter sequence to be trimmed",
                    "doc": "Adapter sequence to be trimmed. If not specified explicitly, Trim Galore will\ntry to auto-detect whether the Illumina universal, Nextera transposase or Illumina\nsmall RNA adapter sequence was used. Also see '--illumina', '--nextera' and\n'--small_rna'. If no adapter can be detected within the first 1 million sequences\nof the first file specified Trim Galore defaults to '--illumina'.\n",
                    "id": "#main/adapter"
                },
                {
                    "type": "File",
                    "sd:upstreamSource": "genome_indices/annotation",
                    "label": "Annotation file",
                    "format": "http://edamontology.org/format_3475",
                    "doc": "Tab-separated annotation file",
                    "id": "#main/annotation_file"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "(?P<umi_1>.{4})(?P<discard_1>G).*",
                    "sd:layout": {
                        "advanced": true
                    },
                    "label": "Barcode pattern",
                    "id": "#main/bc_pattern"
                },
                {
                    "type": "Directory",
                    "label": "BowTie Ribosomal Indices",
                    "sd:upstreamSource": "genome_indices/ribosomal_indices",
                    "doc": "Path to Bowtie generated indices",
                    "id": "#main/bowtie_indices_folder"
                },
                {
                    "type": "File",
                    "sd:upstreamSource": "genome_indices/chrom_length",
                    "label": "Chromosomes length file",
                    "format": "http://edamontology.org/format_2330",
                    "doc": "Chromosomes length file",
                    "id": "#main/chrom_length_file"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "default": 0,
                    "sd:layout": {
                        "advanced": true
                    },
                    "label": "Clip from 3p end",
                    "doc": "Number of bases to clip from the 3p end",
                    "id": "#main/clip_3p_end"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "default": 0,
                    "sd:layout": {
                        "advanced": true
                    },
                    "label": "Clip from 5p end",
                    "doc": "Number of bases to clip from the 5p end",
                    "id": "#main/clip_5p_end"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "sd:layout": {
                        "advanced": true
                    },
                    "label": "Chromosome to be excluded in rpkm calculation",
                    "doc": "Chromosome to be excluded in rpkm calculation",
                    "id": "#main/exclude_chr"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "enum",
                            "symbols": [
                                "#main/extract_method/string",
                                "#main/extract_method/regex"
                            ]
                        }
                    ],
                    "default": "regex",
                    "sd:layout": {
                        "advanced": true
                    },
                    "label": "UMI extract method 'string' or 'regex'",
                    "doc": "How to extract the umi +/- cell barcodes, Choose from\n'string' or 'regex'\n",
                    "id": "#main/extract_method"
                },
                {
                    "type": "File",
                    "label": "FASTQ input file",
                    "format": "http://edamontology.org/format_1930",
                    "doc": "Reads data in a FASTQ format, received after single end sequencing",
                    "id": "#main/fastq_file"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "mm10",
                    "label": "Species string for clipper (hg38, mm10)",
                    "doc": "species: one of ce10 ce11 dm3 hg19 GRCh38 mm9 mm10",
                    "id": "#main/species"
                },
                {
                    "type": "Directory",
                    "label": "STAR indices folder",
                    "sd:upstreamSource": "genome_indices/star_indices",
                    "doc": "Path to STAR generated indices",
                    "id": "#main/star_indices_folder"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "default": 2,
                    "sd:layout": {
                        "advanced": true
                    },
                    "doc": "Number of threads for those steps that support multithreading",
                    "label": "Number of threads",
                    "id": "#main/threads"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "label": "Fake ATDP results for BioWardrobe",
                    "format": "http://edamontology.org/format_3475",
                    "doc": "Average Tag Density generated results",
                    "outputSource": "#main/stats_and_transformations/fake_atdp_file",
                    "id": "#main/atdp_result"
                },
                {
                    "type": "File",
                    "format": "http://edamontology.org/format_2572",
                    "label": "Deduped BAM alignment file",
                    "doc": "Coordinate sorted BAM file and BAI index file (+index BAI)",
                    "outputSource": "#main/samtools_sort_index2/bam_bai_pair",
                    "sd:visualPlugins": [
                        {
                            "igvbrowser": {
                                "tab": "IGV Genome Browser",
                                "id": "#main/bambai_pair/igvbrowser",
                                "optional": true,
                                "type": "alignment",
                                "format": "#main/bambai_pair/bam",
                                "name": "#main/bambai_pair/BAM Track",
                                "displayMode": "SQUISHED"
                            }
                        }
                    ],
                    "id": "#main/bambai_pair"
                },
                {
                    "type": "File",
                    "format": "http://edamontology.org/format_3006",
                    "label": "BigWig file",
                    "doc": "Generated BigWig file",
                    "outputSource": "#main/bam_to_bigwig/bigwig_file",
                    "sd:visualPlugins": [
                        {
                            "igvbrowser": {
                                "tab": "IGV Genome Browser",
                                "id": "#main/bigwig/igvbrowser",
                                "type": "wig",
                                "name": "#main/bigwig/BigWig Track",
                                "height": 120
                            }
                        }
                    ],
                    "id": "#main/bigwig"
                },
                {
                    "type": "File",
                    "outputSource": "#main/clipper/output_bed",
                    "id": "#main/clipper_bed"
                },
                {
                    "type": "File",
                    "outputSource": "#main/clipper/output_pickle",
                    "id": "#main/clipper_pickle"
                },
                {
                    "type": "File",
                    "label": "deduped CLIP error log file",
                    "doc": "deduped CLIP error log file",
                    "outputSource": "#main/dedup_umi/stderr_log",
                    "id": "#main/dedup_error_log"
                },
                {
                    "type": "File",
                    "label": "deduped CLIP log file",
                    "doc": "deduped CLIP log file",
                    "outputSource": "#main/dedup_umi/stdout_log",
                    "id": "#main/dedup_log"
                },
                {
                    "type": "File",
                    "label": "deduped CLIP file",
                    "outputSource": "#main/dedup_umi/dedup_bam_file",
                    "id": "#main/dedup_output"
                },
                {
                    "type": "File",
                    "label": "clipped error log file",
                    "doc": "clipped error log file",
                    "outputSource": "#main/extract_umi/stderr_log",
                    "id": "#main/error_log"
                },
                {
                    "type": "File",
                    "label": "clipped extract log file",
                    "doc": "clipped extract log file",
                    "outputSource": "#main/extract_umi/stdout_log",
                    "id": "#main/extract_log"
                },
                {
                    "type": "File",
                    "label": "FASTQ statistics",
                    "format": "http://edamontology.org/format_2330",
                    "doc": "fastx_quality_stats generated FASTQ file quality statistics file",
                    "outputSource": "#main/fastx_quality_stats_after/statistics_file",
                    "sd:visualPlugins": [
                        {
                            "line": {
                                "tab": "QC Plots",
                                "Title": "After Clipper Base frequency plot",
                                "xAxisTitle": "Nucleotide position",
                                "yAxisTitle": "Frequency",
                                "colors": [
                                    "#b3de69",
                                    "#888888",
                                    "#fb8072",
                                    "#fdc381",
                                    "#99c0db"
                                ],
                                "data": [
                                    "$13",
                                    "$14",
                                    "$15",
                                    "$16",
                                    "$17"
                                ]
                            }
                        },
                        {
                            "boxplot": {
                                "tab": "QC Plots",
                                "Title": "After Clipper Quality Control",
                                "xAxisTitle": "Nucleotide position",
                                "yAxisTitle": "Quality score",
                                "colors": [
                                    "#b3de69",
                                    "#888888",
                                    "#fb8072",
                                    "#fdc381",
                                    "#99c0db"
                                ],
                                "data": [
                                    "$11",
                                    "$7",
                                    "$8",
                                    "$9",
                                    "$12"
                                ]
                            }
                        }
                    ],
                    "id": "#main/fastx_statistics_after"
                },
                {
                    "type": "File",
                    "label": "FASTQ statistics",
                    "format": "http://edamontology.org/format_2330",
                    "doc": "fastx_quality_stats generated FASTQ file quality statistics file",
                    "outputSource": "#main/fastx_quality_stats_original/statistics_file",
                    "sd:visualPlugins": [
                        {
                            "line": {
                                "tab": "QC Plots",
                                "Title": "Original Base frequency plot",
                                "xAxisTitle": "Nucleotide position",
                                "yAxisTitle": "Frequency",
                                "colors": [
                                    "#b3de69",
                                    "#888888",
                                    "#fb8072",
                                    "#fdc381",
                                    "#99c0db"
                                ],
                                "data": [
                                    "$13",
                                    "$14",
                                    "$15",
                                    "$16",
                                    "$17"
                                ]
                            }
                        },
                        {
                            "boxplot": {
                                "tab": "QC Plots",
                                "Title": "Original Quality Control",
                                "xAxisTitle": "Nucleotide position",
                                "yAxisTitle": "Quality score",
                                "colors": [
                                    "#b3de69",
                                    "#888888",
                                    "#fb8072",
                                    "#fdc381",
                                    "#99c0db"
                                ],
                                "data": [
                                    "$11",
                                    "$7",
                                    "$8",
                                    "$9",
                                    "$12"
                                ]
                            }
                        }
                    ],
                    "id": "#main/fastx_statistics_original"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "label": "Bowtie, STAR and GEEP mapping stats",
                    "format": "http://edamontology.org/format_2330",
                    "doc": "Processed and combined Bowtie & STAR aligner and GEEP logs",
                    "outputSource": "#main/stats_and_transformations/formatted_output_file",
                    "sd:visualPlugins": [
                        {
                            "tableView": {
                                "vertical": true,
                                "tab": "Overview"
                            }
                        }
                    ],
                    "sd:preview": {
                        "sd:visualPlugins": [
                            {
                                "pie": {
                                    "colors": [
                                        "#b3de69",
                                        "#99c0db",
                                        "#fdc381",
                                        "#fb8072"
                                    ],
                                    "data": [
                                        "$2",
                                        "$3",
                                        "$4",
                                        "$5"
                                    ]
                                }
                            }
                        ]
                    },
                    "id": "#main/get_formatted_stats"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "label": "Old Bowtie, STAR and GEEP combined log",
                    "format": "http://edamontology.org/format_2330",
                    "doc": "Processed and combined Bowtie & STAR aligner and GEEP logs",
                    "outputSource": "#main/stats_and_transformations/output_file",
                    "id": "#main/get_stat_log"
                },
                {
                    "type": "File",
                    "label": "Island intersect results",
                    "format": "http://edamontology.org/format_3475",
                    "doc": "Iaintersect generated results",
                    "outputSource": "#main/island_intersect/result_file",
                    "id": "#main/iaintersect_result"
                },
                {
                    "type": "File",
                    "outputSource": "#main/bamtobed/bed_file",
                    "id": "#main/output_bed"
                },
                {
                    "type": "File",
                    "outputSource": "#main/tagstopeak/peaks_bed",
                    "id": "#main/peaks_bed"
                },
                {
                    "type": "File",
                    "format": "http://edamontology.org/format_2330",
                    "label": "Bowtie alignment log",
                    "doc": "Bowtie alignment log file",
                    "outputSource": "#main/ribosomal_bowtie_aligner/log_file",
                    "id": "#main/rebosomal_bowtie_log"
                },
                {
                    "type": "File",
                    "format": "http://edamontology.org/format_2330",
                    "label": "STAR final log",
                    "doc": "STAR Log.final.out",
                    "outputSource": "#main/star_aligner/log_final",
                    "id": "#main/star_final_log"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "format": "http://edamontology.org/format_2330",
                    "label": "STAR log out",
                    "doc": "STAR Log.out",
                    "outputSource": "#main/star_aligner/log_out",
                    "id": "#main/star_out_log"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "format": "http://edamontology.org/format_2330",
                    "label": "STAR progress log",
                    "doc": "STAR Log.progress.out",
                    "outputSource": "#main/star_aligner/log_progress",
                    "id": "#main/star_progress_log"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "format": "http://edamontology.org/format_2330",
                    "label": "STAR sj log",
                    "doc": "STAR SJ.out.tab",
                    "outputSource": "#main/star_aligner/log_sj",
                    "id": "#main/star_sj_log"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "format": "http://edamontology.org/format_2330",
                    "label": "STAR stdout log",
                    "doc": "STAR Log.std.out",
                    "outputSource": "#main/star_aligner/log_std",
                    "id": "#main/star_stdout_log"
                },
                {
                    "type": "File",
                    "label": "Transformed peaks Mimics MACS2",
                    "format": "http://edamontology.org/format_3475",
                    "outputSource": "#main/stats_and_transformations/transformed_peaks",
                    "id": "#main/transformed_peaks"
                },
                {
                    "type": "File",
                    "label": "trimm report",
                    "format": "http://edamontology.org/format_2330",
                    "doc": "TrimGalore generated log",
                    "outputSource": "#main/trim_fastq/report_file",
                    "id": "#main/trim_report"
                }
            ],
            "steps": [
                {
                    "run": "#bam-bedgraph-bigwig.cwl",
                    "in": [
                        {
                            "source": "#main/samtools_sort_index2/bam_bai_pair",
                            "id": "#main/bam_to_bigwig/bam_file"
                        },
                        {
                            "source": "#main/extract_fastq/fastq_file",
                            "valueFrom": "$(self.location.split('/').slice(-1)[0].split('.').slice(0,-1).join('.')+'.bigWig')",
                            "id": "#main/bam_to_bigwig/bigwig_filename"
                        },
                        {
                            "source": "#main/chrom_length_file",
                            "id": "#main/bam_to_bigwig/chrom_length_file"
                        },
                        {
                            "source": "#main/star_aligner/uniquely_mapped_reads_number",
                            "id": "#main/bam_to_bigwig/mapped_reads_number"
                        }
                    ],
                    "out": [
                        "#main/bam_to_bigwig/bigwig_file"
                    ],
                    "id": "#main/bam_to_bigwig"
                },
                {
                    "run": "#bedtools-bamtobed.cwl",
                    "in": [
                        {
                            "source": "#main/samtools_sort_index2/bam_bai_pair",
                            "id": "#main/bamtobed/bam_file"
                        }
                    ],
                    "out": [
                        "#main/bamtobed/bed_file"
                    ],
                    "id": "#main/bamtobed"
                },
                {
                    "run": "#clipper.cwl",
                    "in": [
                        {
                            "source": "#main/samtools_sort_index2/bam_bai_pair",
                            "id": "#main/clipper/input_file"
                        },
                        {
                            "source": "#main/species",
                            "id": "#main/clipper/species"
                        }
                    ],
                    "out": [
                        "#main/clipper/output_tsv",
                        "#main/clipper/output_bed",
                        "#main/clipper/output_pickle"
                    ],
                    "id": "#main/clipper"
                },
                {
                    "run": "#umi-tools-dedup.cwl",
                    "in": [
                        {
                            "source": "#main/samtools_sort_index1/bam_bai_pair",
                            "id": "#main/dedup_umi/bam_file"
                        }
                    ],
                    "out": [
                        "#main/dedup_umi/dedup_bam_file",
                        "#main/dedup_umi/stdout_log",
                        "#main/dedup_umi/stderr_log"
                    ],
                    "id": "#main/dedup_umi"
                },
                {
                    "run": "#extract-fastq.cwl",
                    "in": [
                        {
                            "source": "#main/fastq_file",
                            "id": "#main/extract_fastq/compressed_file"
                        }
                    ],
                    "out": [
                        "#main/extract_fastq/fastq_file"
                    ],
                    "id": "#main/extract_fastq"
                },
                {
                    "run": "#umi-tools-extract.cwl",
                    "in": [
                        {
                            "source": "#main/bc_pattern",
                            "id": "#main/extract_umi/bc_pattern_1"
                        },
                        {
                            "source": "#main/extract_method",
                            "id": "#main/extract_umi/extract_method"
                        },
                        {
                            "source": "#main/extract_fastq/fastq_file",
                            "id": "#main/extract_umi/fastq_file_1"
                        },
                        {
                            "source": "#main/extract_fastq/fastq_file",
                            "valueFrom": "${\n  var _f = self.location.split('/').slice(-1)[0].split('.');\n  return _f.slice(0,-1).join('.') + '_extracted.' + _f.slice(-1)[0];\n}\n",
                            "id": "#main/extract_umi/output_filename_1"
                        }
                    ],
                    "out": [
                        "#main/extract_umi/umi_fastq_file_1",
                        "#main/extract_umi/stdout_log",
                        "#main/extract_umi/stderr_log"
                    ],
                    "id": "#main/extract_umi"
                },
                {
                    "run": "#fastx-quality-stats.cwl",
                    "in": [
                        {
                            "source": "#main/trim_fastq/trimmed_file",
                            "id": "#main/fastx_quality_stats_after/input_file"
                        }
                    ],
                    "out": [
                        "#main/fastx_quality_stats_after/statistics_file"
                    ],
                    "id": "#main/fastx_quality_stats_after"
                },
                {
                    "run": "#fastx-quality-stats.cwl",
                    "in": [
                        {
                            "source": "#main/extract_fastq/fastq_file",
                            "id": "#main/fastx_quality_stats_original/input_file"
                        }
                    ],
                    "out": [
                        "#main/fastx_quality_stats_original/statistics_file"
                    ],
                    "id": "#main/fastx_quality_stats_original"
                },
                {
                    "run": "#iaintersect.cwl",
                    "in": [
                        {
                            "source": "#main/annotation_file",
                            "id": "#main/island_intersect/annotation_filename"
                        },
                        {
                            "source": "#main/stats_and_transformations/transformed_peaks",
                            "id": "#main/island_intersect/input_filename"
                        },
                        {
                            "default": 1000,
                            "id": "#main/island_intersect/promoter_bp"
                        }
                    ],
                    "out": [
                        "#main/island_intersect/result_file",
                        "#main/island_intersect/log_file"
                    ],
                    "id": "#main/island_intersect"
                },
                {
                    "run": "#bowtie-alignreads.cwl",
                    "in": [
                        {
                            "default": true,
                            "id": "#main/ribosomal_bowtie_aligner/best"
                        },
                        {
                            "source": "#main/clip_3p_end",
                            "id": "#main/ribosomal_bowtie_aligner/clip_3p_end"
                        },
                        {
                            "source": "#main/clip_5p_end",
                            "id": "#main/ribosomal_bowtie_aligner/clip_5p_end"
                        },
                        {
                            "source": "#main/bowtie_indices_folder",
                            "id": "#main/ribosomal_bowtie_aligner/indices_folder"
                        },
                        {
                            "default": 1,
                            "id": "#main/ribosomal_bowtie_aligner/m"
                        },
                        {
                            "default": true,
                            "id": "#main/ribosomal_bowtie_aligner/sam"
                        },
                        {
                            "default": true,
                            "id": "#main/ribosomal_bowtie_aligner/strata"
                        },
                        {
                            "source": "#main/threads",
                            "id": "#main/ribosomal_bowtie_aligner/threads"
                        },
                        {
                            "source": "#main/trim_fastq/trimmed_file",
                            "id": "#main/ribosomal_bowtie_aligner/upstream_filelist"
                        },
                        {
                            "default": 3,
                            "id": "#main/ribosomal_bowtie_aligner/v"
                        }
                    ],
                    "out": [
                        "#main/ribosomal_bowtie_aligner/log_file"
                    ],
                    "id": "#main/ribosomal_bowtie_aligner"
                },
                {
                    "run": "#samtools-sort-index.cwl",
                    "in": [
                        {
                            "source": "#main/star_aligner/aligned_file",
                            "id": "#main/samtools_sort_index1/sort_input"
                        },
                        {
                            "source": "#main/extract_fastq/fastq_file",
                            "valueFrom": "$(self.location.split('/').slice(-1)[0].split('.').slice(0,-1).join('.')+'.bam')",
                            "id": "#main/samtools_sort_index1/sort_output_filename"
                        },
                        {
                            "source": "#main/threads",
                            "id": "#main/samtools_sort_index1/threads"
                        }
                    ],
                    "out": [
                        "#main/samtools_sort_index1/bam_bai_pair"
                    ],
                    "id": "#main/samtools_sort_index1"
                },
                {
                    "run": "#samtools-sort-index.cwl",
                    "in": [
                        {
                            "source": "#main/dedup_umi/dedup_bam_file",
                            "id": "#main/samtools_sort_index2/sort_input"
                        },
                        {
                            "source": "#main/extract_fastq/fastq_file",
                            "valueFrom": "$(self.location.split('/').slice(-1)[0].split('.').slice(0,-1).join('.')+'.bam')",
                            "id": "#main/samtools_sort_index2/sort_output_filename"
                        },
                        {
                            "source": "#main/threads",
                            "id": "#main/samtools_sort_index2/threads"
                        }
                    ],
                    "out": [
                        "#main/samtools_sort_index2/bam_bai_pair"
                    ],
                    "id": "#main/samtools_sort_index2"
                },
                {
                    "run": "#star-alignreads.cwl",
                    "in": [
                        {
                            "default": 1,
                            "id": "#main/star_aligner/alignSJDBoverhangMin"
                        },
                        {
                            "source": "#main/clip_3p_end",
                            "id": "#main/star_aligner/clip3pNbases"
                        },
                        {
                            "source": "#main/clip_5p_end",
                            "id": "#main/star_aligner/clip5pNbases"
                        },
                        {
                            "source": "#main/star_indices_folder",
                            "id": "#main/star_aligner/genomeDir"
                        },
                        {
                            "default": 5,
                            "id": "#main/star_aligner/outFilterMismatchNmax"
                        },
                        {
                            "default": 1,
                            "id": "#main/star_aligner/outFilterMultimapNmax"
                        },
                        {
                            "source": "#main/trim_fastq/trimmed_file",
                            "id": "#main/star_aligner/readFilesIn"
                        },
                        {
                            "default": 15,
                            "id": "#main/star_aligner/seedSearchStartLmax"
                        },
                        {
                            "source": "#main/threads",
                            "id": "#main/star_aligner/threads"
                        }
                    ],
                    "out": [
                        "#main/star_aligner/aligned_file",
                        "#main/star_aligner/log_final",
                        "#main/star_aligner/uniquely_mapped_reads_number",
                        "#main/star_aligner/log_out",
                        "#main/star_aligner/log_progress",
                        "#main/star_aligner/log_std",
                        "#main/star_aligner/log_sj"
                    ],
                    "id": "#main/star_aligner"
                },
                {
                    "in": [
                        {
                            "source": "#main/ribosomal_bowtie_aligner/log_file",
                            "id": "#main/stats_and_transformations/bowtie_log"
                        },
                        {
                            "source": "#main/dedup_umi/stdout_log",
                            "id": "#main/stats_and_transformations/dedup_log"
                        },
                        {
                            "source": "#main/clipper/output_bed",
                            "id": "#main/stats_and_transformations/peaks"
                        },
                        {
                            "source": "#main/star_aligner/log_final",
                            "id": "#main/stats_and_transformations/star_log"
                        }
                    ],
                    "out": [
                        "#main/stats_and_transformations/output_file",
                        "#main/stats_and_transformations/formatted_output_file",
                        "#main/stats_and_transformations/fake_atdp_file",
                        "#main/stats_and_transformations/transformed_peaks"
                    ],
                    "run": {
                        "cwlVersion": "v1.0",
                        "class": "CommandLineTool",
                        "requirements": [
                            {
                                "class": "ShellCommandRequirement"
                            },
                            {
                                "class": "InlineJavascriptRequirement",
                                "expressionLib": [
                                    "var get_output_filename = function() { return inputs.star_log.location.split('/').slice(-1)[0].replace(/_extracted_trimmed\\.*Log\\.final\\.out$/i,''); }"
                                ]
                            }
                        ],
                        "hints": [
                            {
                                "class": "DockerRequirement",
                                "dockerPull": "biowardrobe2/scidap:v0.0.3"
                            }
                        ],
                        "inputs": [
                            {
                                "type": "File",
                                "inputBinding": {
                                    "position": 7
                                },
                                "id": "#main/stats_and_transformations/a2fa6ee8-5cd6-4327-a9c6-5f081805f892/bowtie_log"
                            },
                            {
                                "type": "File",
                                "inputBinding": {
                                    "position": 8
                                },
                                "id": "#main/stats_and_transformations/a2fa6ee8-5cd6-4327-a9c6-5f081805f892/dedup_log"
                            },
                            {
                                "type": [
                                    "null",
                                    "string"
                                ],
                                "inputBinding": {
                                    "position": 9,
                                    "valueFrom": "$(get_output_filename())"
                                },
                                "default": "",
                                "id": "#main/stats_and_transformations/a2fa6ee8-5cd6-4327-a9c6-5f081805f892/output_filename"
                            },
                            {
                                "type": "File",
                                "inputBinding": {
                                    "position": 10
                                },
                                "id": "#main/stats_and_transformations/a2fa6ee8-5cd6-4327-a9c6-5f081805f892/peaks"
                            },
                            {
                                "type": [
                                    "null",
                                    "string"
                                ],
                                "default": "# !/usr/bin/env python\nimport sys, re, math\nTOTAL, ALIGNED, RIBO, MULTIMAPPED, USED = 0, 0, 0, 0, 0\nwith open(sys.argv[1], 'r') as star_log:\n    for line in star_log:\n        if 'Number of input reads' in line:\n            TOTAL = int(line.split('|')[1])\n        if 'Uniquely mapped reads number' in line:\n            ALIGNED = int(line.split('|')[1])\n        if 'Number of reads mapped to too many loci' in line:\n            MULTIMAPPED = int(line.split('|')[1])\nwith open(sys.argv[2], 'r') as bowtie_log:\n    for line in bowtie_log:\n        if 'alignment:' in line:\n            RIBO = int(line.split('alignment:')[1].split()[0])\nwith open(sys.argv[3], 'r') as dedup_log:\n    for line in dedup_log:\n        if 'Number of reads out:' in line:\n            USED = int(line.split('Number of reads out:')[1])\n\nprint TOTAL, ALIGNED, MULTIMAPPED, USED\n\nwith open(sys.argv[4]+\"_stats.tsv\", 'w') as fof:\n    fof.write(\"Reads total\\tReads used\\tMulti-mapped\\tDuplicates\\tUnmapped\\tRibosomal contamination\\n\")\n    fof.write(str(TOTAL) + \"\\t\" + str(USED) + \"\\t\" + str(MULTIMAPPED) + \"\\t\" + str(ALIGNED-USED) + \"\\t\" + str(TOTAL-ALIGNED-MULTIMAPPED) + \"\\t\" + str(RIBO) + \"\\n\")\n\n# TODO: Get rid of! No need without biowardrobe!\nwith open(sys.argv[4]+\"_atdp.tsv\", 'w') as fof:\n    fof.write(\"X\\tY\\n\")\n    for i in range(-5000, 5001):\n      fof.write(str(i) + \"\\t1\\n\")\n\n# TODO: Get rid of! No need with right iaintersect! clv toolkit\n#with open(sys.argv[4]+\"_macs_peaks.tsv\", 'w') as fof:\n#    fof.write(\"chr\\tstart\\tend\\tlength\\tabs_summit\\tpileup\\t-log10(pvalue)\\tfold_enrichment\\t-log10(qvalue)\\tname\\n\")\n#    with open(sys.argv[5], 'r') as peak_file:\n#        for line in peak_file:\n#          tmpa=line.split()\n#          pis=[x.split('=')[1] for x in re.split(r'[\\[\\]]',tmpa[3])[1:] if x.strip()]\n#          fof.write(tmpa[0]+\"\\t\"+tmpa[1]+\"\\t\"+tmpa[2]+\"\\t\"+str(int(tmpa[2])-int(tmpa[1]))+\"\\t0\\t\"+pis[1]+\"\\t\"+str(-math.log10(float(pis[3])))+\"\\t\"+tmpa[4]+\"\\t0\\t\"+tmpa[3]+\"\\n\")\n\n# TODO: Get rid of! No need with right iaintersect! clv toolkit\nwith open(sys.argv[4]+\"_macs_peaks.tsv\", 'w') as fof:\n    fof.write(\"chr\\tstart\\tend\\tlength\\tabs_summit\\tpileup\\t-log10(pvalue)\\tfold_enrichment\\t-log10(qvalue)\\tname\\n\")\n    with open(sys.argv[5], 'r') as peak_file:\n        for line in peak_file:\n          tmpa=line.split()\n          fof.write(tmpa[0]+\"\\t\"+tmpa[1]+\"\\t\"+tmpa[2]+\"\\t\"+str(int(tmpa[2])-int(tmpa[1]))+\"\\t0\\t0\\t\"+str(-math.log10(float(tmpa[4])))+\"\\t\"+tmpa[4]+\"\\t0\\t\"+tmpa[3]+\"\\n\")\n",
                                "inputBinding": {
                                    "position": 5
                                },
                                "id": "#main/stats_and_transformations/a2fa6ee8-5cd6-4327-a9c6-5f081805f892/script"
                            },
                            {
                                "type": "File",
                                "inputBinding": {
                                    "position": 6
                                },
                                "id": "#main/stats_and_transformations/a2fa6ee8-5cd6-4327-a9c6-5f081805f892/star_log"
                            }
                        ],
                        "outputs": [
                            {
                                "type": "File",
                                "outputBinding": {
                                    "glob": "$(get_output_filename()+\"_atdp.tsv\")"
                                },
                                "id": "#main/stats_and_transformations/a2fa6ee8-5cd6-4327-a9c6-5f081805f892/fake_atdp_file"
                            },
                            {
                                "type": "File",
                                "outputBinding": {
                                    "glob": "$(get_output_filename()+\"_stats.tsv\")"
                                },
                                "id": "#main/stats_and_transformations/a2fa6ee8-5cd6-4327-a9c6-5f081805f892/formatted_output_file"
                            },
                            {
                                "type": "File",
                                "id": "#main/stats_and_transformations/a2fa6ee8-5cd6-4327-a9c6-5f081805f892/output_file",
                                "outputBinding": {
                                    "glob": "$(get_output_filename()+\".stat\")"
                                }
                            },
                            {
                                "type": "File",
                                "outputBinding": {
                                    "glob": "$(get_output_filename()+\"_macs_peaks.tsv\")"
                                },
                                "id": "#main/stats_and_transformations/a2fa6ee8-5cd6-4327-a9c6-5f081805f892/transformed_peaks"
                            }
                        ],
                        "baseCommand": [
                            "python",
                            "-c"
                        ],
                        "stdout": "$(get_output_filename()+\".stat\")",
                        "id": "#main/stats_and_transformations/a2fa6ee8-5cd6-4327-a9c6-5f081805f892"
                    },
                    "id": "#main/stats_and_transformations"
                },
                {
                    "run": "#clip-toolkit-tag2peak.cwl",
                    "in": [
                        {
                            "source": "#main/bamtobed/bed_file",
                            "id": "#main/tagstopeak/bed_file"
                        },
                        {
                            "default": true,
                            "id": "#main/tagstopeak/big"
                        },
                        {
                            "source": "#main/tagstopeak_transformations/transformed_annotation",
                            "id": "#main/tagstopeak/gene_bed_file"
                        },
                        {
                            "default": true,
                            "id": "#main/tagstopeak/separate_strands"
                        },
                        {
                            "default": true,
                            "id": "#main/tagstopeak/valley_seeking"
                        }
                    ],
                    "out": [
                        "#main/tagstopeak/peaks_bed"
                    ],
                    "id": "#main/tagstopeak"
                },
                {
                    "in": [
                        {
                            "source": "#main/annotation_file",
                            "id": "#main/tagstopeak_transformations/annotation"
                        }
                    ],
                    "out": [
                        "#main/tagstopeak_transformations/transformed_annotation"
                    ],
                    "run": {
                        "cwlVersion": "v1.0",
                        "class": "CommandLineTool",
                        "requirements": [
                            {
                                "class": "ShellCommandRequirement"
                            }
                        ],
                        "hints": [
                            {
                                "class": "DockerRequirement",
                                "dockerPull": "biowardrobe2/scidap:v0.0.3"
                            }
                        ],
                        "inputs": [
                            {
                                "type": "File",
                                "inputBinding": {
                                    "position": 3
                                },
                                "id": "#main/tagstopeak_transformations/e054a631-1a60-4e89-9953-dd00a633513f/annotation"
                            },
                            {
                                "type": [
                                    "null",
                                    "string"
                                ],
                                "default": "# !/usr/bin/env python\nimport sys, re, math\nwith open(\"transformed_annotation.tsv\", 'w') as fof:\n    with open(sys.argv[1], 'r') as afile:\n        next(afile) # header line\n        for line in afile:\n          al=line.split()\n          # orig                      3                         6       7         8                            11\n          # bin     name    chrom   strand  txStart txEnd   cdsStart  cdsEnd  exonCount exonStarts  exonEnds  score   name2   cdsStartStat    cdsEndStat      exonFrames\n          # req\n          # chrom chromStart chromEnd name score strand thickStart thickEnd itemRgb blockCount blockSizes blockStarts\n          blkStarts= ','.join([str(int(x)-int(al[4])) for x in al[9].split(',') if x])\n          blkSizes=','.join([str(-int(e)+int(al[10].split(',')[i])) for i, e in enumerate([x for x in al[9].split(',') if x])])\n          fof.write(al[2]+\"\\t\"+al[4]+\"\\t\"+al[5]+\"\\t\"+al[1]+\"\\t\"+al[11]+\"\\t\"+al[3]+\"\\t\"+al[6]+\"\\t\"+al[7]+\"\\t0\\t\"+al[8]+\"\\t\"+blkSizes+\"\\t\"+blkStarts+\"\\n\")\n",
                                "inputBinding": {
                                    "position": 2
                                },
                                "id": "#main/tagstopeak_transformations/e054a631-1a60-4e89-9953-dd00a633513f/script"
                            }
                        ],
                        "outputs": [
                            {
                                "type": "File",
                                "outputBinding": {
                                    "glob": "transformed_annotation.tsv"
                                },
                                "id": "#main/tagstopeak_transformations/e054a631-1a60-4e89-9953-dd00a633513f/transformed_annotation"
                            }
                        ],
                        "baseCommand": [
                            "python",
                            "-c"
                        ],
                        "id": "#main/tagstopeak_transformations/e054a631-1a60-4e89-9953-dd00a633513f"
                    },
                    "id": "#main/tagstopeak_transformations"
                },
                {
                    "run": "#trimgalore.cwl",
                    "in": [
                        {
                            "source": "#main/adapter",
                            "id": "#main/trim_fastq/adapter"
                        },
                        {
                            "default": true,
                            "id": "#main/trim_fastq/dont_gzip"
                        },
                        {
                            "source": "#main/extract_umi/umi_fastq_file_1",
                            "id": "#main/trim_fastq/input_file"
                        },
                        {
                            "default": 30,
                            "id": "#main/trim_fastq/length"
                        }
                    ],
                    "out": [
                        "#main/trim_fastq/trimmed_file",
                        "#main/trim_fastq/report_file"
                    ],
                    "id": "#main/trim_fastq"
                }
            ],
            "label": "CLIP-Seq pipeline for single-read experiment NNNNG",
            "doc": "Cross-Linking ImmunoPrecipitation\n=================================\n\n`CLIP` (`cross-linking immunoprecipitation`) is a method used in molecular biology that combines UV cross-linking with\nimmunoprecipitation in order to analyse protein interactions with RNA or to precisely locate RNA modifications (e.g. m6A).\n(Uhl|Houwaart|Corrado|Wright|Backofen|2017)(Ule|Jensen|Ruggiu|Mele|2003)(Sugimoto|K\u00f6nig|Hussain|Zupan|2012)(Zhang|Darnell|2011)\n(Ke| Alemu| Mertens| Gantman|2015) CLIP-based techniques can be used to map RNA binding protein binding sites or RNA modification\nsites (Ke| Alemu| Mertens| Gantman|2015)(Ke| Pandya-Jones| Saito| Fak|2017) of interest on a genome-wide scale,\nthereby increasing the understanding of post-transcriptional regulatory networks.\n\nThe identification of sites where RNA-binding proteins (RNABPs) interact with target RNAs opens the door to understanding\nthe vast complexity of RNA regulation. UV cross-linking and immunoprecipitation (CLIP) is a transformative technology in which RNAs\npurified from _in vivo_ cross-linked RNA-protein complexes are sequenced to reveal footprints of RNABP:RNA contacts.\nCLIP combined with high-throughput sequencing (HITS-CLIP) is a generalizable strategy to produce transcriptome-wide maps of RNA\nbinding with higher accuracy and resolution than standard RNA immunoprecipitation (RIP) profiling or purely computational approaches.\n\nThe application of CLIP to Argonaute proteins has expanded the utility of this approach to mapping binding sites for microRNAs\nand other small regulatory RNAs. Finally, recent advances in data analysis take advantage of cross-link\u2013induced mutation sites\n(CIMS) to refine RNA-binding maps to single-nucleotide resolution. Once IP conditions are established, HITS-CLIP takes ~8 d to prepare\nRNA for sequencing. Established pipelines for data analysis, including those for CIMS, take 3\u20134 d.\n\nWorkflow\n--------\n\nCLIP begins with the in-vivo cross-linking of RNA-protein complexes using ultraviolet light (UV).\nUpon UV exposure, covalent bonds are formed between proteins and nucleic acids that are in close proximity.\n(Darnell|2012) The cross-linked cells are then lysed, and the protein of interest is isolated via immunoprecipitation.\nIn order to allow for sequence specific priming of reverse transcription, RNA adapters are ligated to the 3' ends,\nwhile radiolabeled phosphates are transferred to the 5' ends of the RNA fragments.\nThe RNA-protein complexes are then separated from free RNA using gel electrophoresis and membrane transfer.\nProteinase K digestion is then performed in order to remove protein from the RNA-protein complexes.\nThis step leaves a peptide at the cross-link site, allowing for the identification of the cross-linked nucleotide.\n(K\u00f6nig| McGlincy| Ule|2012) After ligating RNA linkers to the RNA 5' ends, cDNA is synthesized via RT-PCR.\nHigh-throughput sequencing is then used to generate reads containing distinct barcodes that identify the last cDNA nucleotide.\nInteraction sites can be identified by mapping the reads back to the transcriptome.",
            "id": "#main",
            "http://schema.org/name": "CLIP-Seq pipeline for single-read experiment NNNNG",
            "http://schema.org/alternateName": "CLIP-Seq workflow for single-read experiment",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/datirium/workflows/master/workflows/clipseq-se.cwl",
            "http://schema.org/codeRepository": "https://github.com/datirium/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Datirium, LLC",
                    "http://schema.org/member": [
                        {
                            "class": "http://schema.org/Person",
                            "http://schema.org/name": "Artem BArski",
                            "http://schema.org/email": "mailto:Artem.Barski@datirum.com"
                        },
                        {
                            "class": "http://schema.org/Person",
                            "http://schema.org/name": "Andrey Kartashov",
                            "http://schema.org/email": "mailto:Andrey.Kartashov@datirium.com",
                            "http://schema.org/sameAs": [
                                {
                                    "$import": "#0000-0001-9102-5681"
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    ],
    "cwlVersion": "v1.0",
    "$schemas": [
        "http://schema.org/version/9.0/schemaorg-current-http.rdf"
    ],
    "$namespaces": {
        "s": "http://schema.org/"
    }
}
